#ifdef ENABLE_AXISTRACE
/* TRACE ADDED BY THE TRACE INSTRUMENTOR TOOL */
#include "../common/AxisTrace.h"
#endif
/* -*- C++ -*- */

/*
 *   Copyright 2003-2004 The Apache Software Foundation.
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 *
 *
 * @author Susantha Kumara (skumara@virtusa.com)
 *
 */

#include "HandlerPool.h"

#include "AppScopeHandlerPool.h"

#include "RequestScopeHandlerPool.h"

#include "SessionScopeHandlerPool.h"

#include "../wsdd/WSDDDeployment.h"

#include <axis/AxisException.hpp>

#include "../common/AxisTrace.h"

#include "../platforms/PlatformAutoSense.hpp"

extern AXIS_CPP_NAMESPACE_PREFIX AppScopeHandlerPool* g_pAppScopeHandlerPool;

extern AXIS_CPP_NAMESPACE_PREFIX RequestScopeHandlerPool* g_pRequestScopeHandlerPool;

extern AXIS_CPP_NAMESPACE_PREFIX SessionScopeHandlerPool* g_pSessionScopeHandlerPool;

extern AXIS_CPP_NAMESPACE_PREFIX WSDDDeployment* g_pWSDDDeployment;

AXIS_CPP_NAMESPACE_START

HandlerPool::HandlerPool ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HandlerPool", "HandlerPool", this, 0);	  /* AUTOINSERTED TRACE */
	#endif


	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HandlerPool", "HandlerPool", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

HandlerPool::~HandlerPool ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HandlerPool", "~HandlerPool", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    for (list <HandlerChain*>::iterator it = m_ChainStore.begin ();
        it != m_ChainStore.end (); it++)
    {
        delete (*it);
    }
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HandlerPool", "~HandlerPool", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

int HandlerPool::getHandler (BasicHandler** ppHandler, string & sSessionId,
    int nScope, int nLibId)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HandlerPool", "getHandler", this, 4,
					TRACETYPE_POINTER, sizeof(BasicHandler*), ((void*)&ppHandler),
					TRACETYPE_STLSTRING, 0, ((void*)&sSessionId),
					TRACETYPE_INT, 0, ((void*)&nScope),
					TRACETYPE_INT, 0, ((void*)&nLibId));	  /* AUTOINSERTED TRACE */
	#endif

    *ppHandler = NULL;
    int Status = AXIS_FAIL;
    switch (nScope)
    {
        case AH_APPLICATION:
            do
            {
                if ((Status =
                    g_pAppScopeHandlerPool->getInstance (ppHandler,
                                                          nLibId)) ==
                    AXIS_SUCCESS)
                {
                    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (Status);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HandlerPool", "getHandler", this, 1,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return Status;
		#endif
	}

                }
                else if (Status == SERVER_ENGINE_HANDLER_BEING_USED)
                {             //block this thread not this object
                    PLATFORM_SLEEP(0);
                }
                else
                {
                    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (Status);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HandlerPool", "getHandler", this, 2,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return Status;
		#endif
	}

                }
            } while (Status == SERVER_ENGINE_HANDLER_BEING_USED);
            break;
        case AH_SESSION:
            	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (g_pSessionScopeHandlerPool->getInstance (sSessionId,
                ppHandler, nLibId));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HandlerPool", "getHandler", this, 3,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return g_pSessionScopeHandlerPool->getInstance (sSessionId,
                ppHandler, nLibId);
		#endif
	}

        case AH_REQUEST:
            	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (g_pRequestScopeHandlerPool->getInstance (ppHandler, nLibId));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HandlerPool", "getHandler", this, 4,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return g_pRequestScopeHandlerPool->getInstance (ppHandler, nLibId);
		#endif
	}

    }
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (Status);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HandlerPool", "getHandler", this, 5,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return Status;
		#endif
	}

}

int HandlerPool::poolHandler (string &sSessionId, BasicHandler* pHandler,
    int nScope, int nLibId, bool bWebService)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HandlerPool", "poolHandler", this, 5,
					TRACETYPE_STLSTRING, 0, ((void*)&sSessionId),
					TRACETYPE_POINTER, sizeof(BasicHandler), ((void*)&pHandler),
					TRACETYPE_INT, 0, ((void*)&nScope),
					TRACETYPE_INT, 0, ((void*)&nLibId),
					TRACETYPE_BOOL, 0, ((void*)&bWebService));	  /* AUTOINSERTED TRACE */
	#endif

    switch (nScope)
    {
        case AH_APPLICATION:
            g_pAppScopeHandlerPool->putInstance (pHandler, nLibId);
            break;
        case AH_SESSION:
            g_pSessionScopeHandlerPool->putInstance (sSessionId, pHandler,
                nLibId);
            break;
        case AH_REQUEST:
            g_pRequestScopeHandlerPool->putInstance (pHandler, nLibId, bWebService);
            break;
    }
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HandlerPool", "poolHandler", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}

}

int HandlerPool::getGlobalRequestFlowHandlerChain (HandlerChain** ppChain,
    string & sSessionId)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HandlerPool", "getGlobalRequestFlowHandlerChain", this, 2,
					TRACETYPE_POINTER, sizeof(HandlerChain*), ((void*)&ppChain),
					TRACETYPE_STLSTRING, 0, ((void*)&sSessionId));	  /* AUTOINSERTED TRACE */
	#endif

    const WSDDHandlerList* pHandlerList =
        g_pWSDDDeployment->getGlobalRequestFlowHandlers ();
    if (pHandlerList)
    {
        	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (getHandlerChain (sSessionId, ppChain, pHandlerList));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HandlerPool", "getGlobalRequestFlowHandlerChain", this, 1,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return getHandlerChain (sSessionId, ppChain, pHandlerList);
		#endif
	}

    }
    else
    {
        *ppChain = NULL;
        	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HandlerPool", "getGlobalRequestFlowHandlerChain", this, 2,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}
  // NO_HANDLERS_CONFIGURED
    }
}

int HandlerPool::getGlobalResponseFlowHandlerChain (HandlerChain** ppChain,
    string &sSessionId)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HandlerPool", "getGlobalResponseFlowHandlerChain", this, 2,
					TRACETYPE_POINTER, sizeof(HandlerChain*), ((void*)&ppChain),
					TRACETYPE_STLSTRING, 0, ((void*)&sSessionId));	  /* AUTOINSERTED TRACE */
	#endif

    const WSDDHandlerList *pHandlerList =
        g_pWSDDDeployment->getGlobalResponseFlowHandlers ();
    if (pHandlerList)
    {
        	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (getHandlerChain (sSessionId, ppChain, pHandlerList));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HandlerPool", "getGlobalResponseFlowHandlerChain", this, 1,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return getHandlerChain (sSessionId, ppChain, pHandlerList);
		#endif
	}

    }
    else
    {
        *ppChain = NULL;
        	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HandlerPool", "getGlobalResponseFlowHandlerChain", this, 2,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}
  // NO_HANDLERS_CONFIGURED
    }
}

int HandlerPool::getTransportRequestFlowHandlerChain (HandlerChain** ppChain,
    string &sSessionId, AXIS_PROTOCOL_TYPE Protocol)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HandlerPool", "getTransportRequestFlowHandlerChain", this, 3,
					TRACETYPE_POINTER, sizeof(HandlerChain*), ((void*)&ppChain),
					TRACETYPE_STLSTRING, 0, ((void*)&sSessionId),
					TRACETYPE_DATA, sizeof(AXIS_PROTOCOL_TYPE), ((void*)&Protocol));	  /* AUTOINSERTED TRACE */
	#endif

    const WSDDHandlerList* pHandlerList =
        g_pWSDDDeployment->getTransportRequestFlowHandlers (Protocol);
    if (pHandlerList)
    {
        	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (getHandlerChain (sSessionId, ppChain, pHandlerList));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HandlerPool", "getTransportRequestFlowHandlerChain", this, 1,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return getHandlerChain (sSessionId, ppChain, pHandlerList);
		#endif
	}

    }
    else
    {
        *ppChain = NULL;
        	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HandlerPool", "getTransportRequestFlowHandlerChain", this, 2,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}
  // NO_HANDLERS_CONFIGURED
    }
}

int HandlerPool::getTransportResponseFlowHandlerChain (HandlerChain** ppChain,
    string & sSessionId, AXIS_PROTOCOL_TYPE Protocol)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HandlerPool", "getTransportResponseFlowHandlerChain", this, 3,
					TRACETYPE_POINTER, sizeof(HandlerChain*), ((void*)&ppChain),
					TRACETYPE_STLSTRING, 0, ((void*)&sSessionId),
					TRACETYPE_DATA, sizeof(AXIS_PROTOCOL_TYPE), ((void*)&Protocol));	  /* AUTOINSERTED TRACE */
	#endif

    const WSDDHandlerList* pHandlerList =
        g_pWSDDDeployment->getTransportResponseFlowHandlers (Protocol);
    if (pHandlerList)
    {
        	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (getHandlerChain (sSessionId, ppChain, pHandlerList));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HandlerPool", "getTransportResponseFlowHandlerChain", this, 1,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return getHandlerChain (sSessionId, ppChain, pHandlerList);
		#endif
	}

    }
    else
    {
        *ppChain = NULL;
        	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HandlerPool", "getTransportResponseFlowHandlerChain", this, 2,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}
  // NO_HANDLERS_CONFIGURED
    }
}

int HandlerPool::getRequestFlowHandlerChain (HandlerChain** ppChain,
    string &sSessionId, const WSDDService* pService)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HandlerPool", "getRequestFlowHandlerChain", this, 3,
					TRACETYPE_POINTER, sizeof(HandlerChain*), ((void*)&ppChain),
					TRACETYPE_STLSTRING, 0, ((void*)&sSessionId),
					TRACETYPE_POINTER, sizeof(WSDDService), ((void*)&pService));	  /* AUTOINSERTED TRACE */
	#endif

    const WSDDHandlerList* pHandlerList = pService->getRequestFlowHandlers ();
    if (pHandlerList)
    {
        	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (getHandlerChain (sSessionId, ppChain, pHandlerList));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HandlerPool", "getRequestFlowHandlerChain", this, 1,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return getHandlerChain (sSessionId, ppChain, pHandlerList);
		#endif
	}

    }
    else
    {
        *ppChain = NULL;
        /* AXISTRACE1("No handlers configured", INFO); */
        	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HandlerPool", "getRequestFlowHandlerChain", this, 2,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}
  // NO_HANDLERS_CONFIGURED
    }
}

int HandlerPool::getResponseFlowHandlerChain (HandlerChain** ppChain,
    string &sSessionId, const WSDDService* pService)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HandlerPool", "getResponseFlowHandlerChain", this, 3,
					TRACETYPE_POINTER, sizeof(HandlerChain*), ((void*)&ppChain),
					TRACETYPE_STLSTRING, 0, ((void*)&sSessionId),
					TRACETYPE_POINTER, sizeof(WSDDService), ((void*)&pService));	  /* AUTOINSERTED TRACE */
	#endif

    const WSDDHandlerList* pHandlerList = pService->getResponseFlowHandlers ();
    if (pHandlerList)
    {
        	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (getHandlerChain (sSessionId, ppChain, pHandlerList));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HandlerPool", "getResponseFlowHandlerChain", this, 1,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return getHandlerChain (sSessionId, ppChain, pHandlerList);
		#endif
	}

    }
    else
    {
        *ppChain = NULL;
        	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HandlerPool", "getResponseFlowHandlerChain", this, 2,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}
  // NO_HANDLERS_CONFIGURED
    }
}

int HandlerPool::getHandlerChain (string &sSessionId, HandlerChain** ppChain,
    const WSDDHandlerList* pHandlerList)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HandlerPool", "getHandlerChain", this, 3,
					TRACETYPE_STLSTRING, 0, ((void*)&sSessionId),
					TRACETYPE_POINTER, sizeof(HandlerChain*), ((void*)&ppChain),
					TRACETYPE_POINTER, sizeof(WSDDHandlerList), ((void*)&pHandlerList));	  /* AUTOINSERTED TRACE */
	#endif

    //lock ();
	Lock l(this);
    *ppChain = NULL;
    HandlerChain* pChain = NULL;
    // check m_ChainStore to get a HandlerChain
    if (!m_ChainStore.empty ())
    {
        pChain = m_ChainStore.front ();
        m_ChainStore.pop_front ();
        pChain->init ();
    }
    else
    {
        pChain = new HandlerChain ();
    }
    WSDDHandler* pWSDDH = NULL;
    BasicHandler* pBH = NULL;
    int Status = AXIS_SUCCESS;
    for (WSDDHandlerList::const_iterator it = pHandlerList->begin ();
         it != pHandlerList->end (); it++)
    {
        pWSDDH = (*it);
        if ((Status = getHandler (&pBH, sSessionId, pWSDDH->getScope (),
            pWSDDH->getLibId ())) == AXIS_SUCCESS)
        {
            /*
             * if (NORMAL_HANDLER == pBH->_functions->GetType(pBH->_object))
             * {                
             *     ((Handler*)
	     *     (pBH->_object))->SetOptionList(pWSDDH->GetParameterList());
	     *     pChain->AddHandler(pBH, pWSDDH->GetScope(), pWSDDH->GetLibId());                
             * }
             * else
             * {
             *     Status = SERVER_ENGINE_WRONGHANDLERTYPE;
             *     break;
             * }
             */
            if (0 != pBH->_functions)
            {
                /* C Handler */
            }
            else if (0 != pBH->_object)
            {
                if (NORMAL_HANDLER == ((Handler*) (pBH->_object))->getType ())
                {
                    ((Handler*) (pBH->_object))->
                        setOptionList (pWSDDH->getParameterList ());
                    pChain->addHandler (pBH, pWSDDH->getScope (),
                        pWSDDH->getLibId ());
                }
                else
                {
                    Status = SERVER_ENGINE_WRONG_HANDLER_TYPE;
                    break;
                }
            }
        }
        else
        {
            break;
        }
    }
    if (Status != AXIS_SUCCESS) //some failure so undo whatever done here
    {
        AXISTRACE1 ("Handler failure", WARN);
        string nosession = SESSIONLESSHANDLERS;
        for (pChain->m_itCurrHandler = pChain->m_HandlerList.begin ();
             pChain->m_itCurrHandler != pChain->m_HandlerList.end ();
               pChain->m_itCurrHandler++)
        {
            if ((*pChain->m_itCurrHandler).m_pHandler != NULL)
            {
                poolHandler (nosession, (*pChain->m_itCurrHandler).m_pHandler,
                    (*pChain->m_itCurrHandler).m_nScope,
                    (*pChain->m_itCurrHandler).m_nLibId);
            }
        }
        pChain->fini ();
        m_ChainStore.push_back (pChain);
        //unlock ();
        	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (Status);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HandlerPool", "getHandlerChain", this, 1,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return Status;
		#endif
	}

    }
    else
    {
        *ppChain = pChain;
        //unlock ();
        	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (Status);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HandlerPool", "getHandlerChain", this, 2,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return Status;
		#endif
	}

    }
}

void HandlerPool::poolHandlerChain (HandlerChain* pChain, string &sSessionId)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HandlerPool", "poolHandlerChain", this, 2,
					TRACETYPE_POINTER, sizeof(HandlerChain), ((void*)&pChain),
					TRACETYPE_STLSTRING, 0, ((void*)&sSessionId));	  /* AUTOINSERTED TRACE */
	#endif

    for (pChain->m_itCurrHandler = pChain->m_HandlerList.begin ();
        pChain->m_itCurrHandler != pChain->m_HandlerList.end ();
        pChain->m_itCurrHandler++)
    {
        if ((*pChain->m_itCurrHandler).m_pHandler != NULL)
        {
            poolHandler (sSessionId, (*pChain->m_itCurrHandler).m_pHandler,
                (*pChain->m_itCurrHandler).m_nScope, 
                (*pChain->m_itCurrHandler).m_nLibId);
        }
    }
    pChain->fini ();
    //lock ();
	Lock l(this);
    m_ChainStore.push_back (pChain);
    //unlock ();
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HandlerPool", "poolHandlerChain", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

int HandlerPool::getWebService( BasicHandler ** ppHandler, string & sSessionId, const WSDDHandler * pService)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HandlerPool", "getWebService", this, 3,
					TRACETYPE_POINTER, sizeof(BasicHandler*), ((void*)&ppHandler),
					TRACETYPE_STLSTRING, 0, ((void*)&sSessionId),
					TRACETYPE_POINTER, sizeof(WSDDHandler), ((void*)&pService));	  /* AUTOINSERTED TRACE */
	#endif

    int Status = getHandler( ppHandler, sSessionId, pService->getScope(), pService->getLibId());
    if( Status == AXIS_SUCCESS)
    {
        if( NULL != (*ppHandler)->_functions)
        // C web service 
        {
            /*if (AXIS_SUCCESS !=
                (Status =
                (*ppHandler)->_functions->init ((*ppHandler)->_object)))
            {
                (*ppHandler)->_functions->fini ((*ppHandler)->_object);
                poolHandler (sSessionId, *ppHandler, pService->getScope (),
                    pService->getLibId ());
                *ppHandler = NULL;
            }*/
        }
        else if( NULL == (*ppHandler)->_object)
		{
            Status = AXIS_FAIL;
		}
        else
        // C++ web service 
        {
            /*if (AXIS_SUCCESS !=
                (Status =
                ((HandlerBase *) (*ppHandler)->_object)->init ()))
            {
                ((HandlerBase *) (*ppHandler)->_object)->fini ();
                poolHandler (sSessionId, *ppHandler, pService->getScope (),
                    pService->getLibId ());
                *ppHandler = NULL;
            }*/
        }
    }
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (Status);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HandlerPool", "getWebService", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return Status;
		#endif
	}

}

void HandlerPool::poolWebService (string &sSessionId, BasicHandler* pHandler,
    const WSDDHandler * pHandlerInfo)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HandlerPool", "poolWebService", this, 3,
					TRACETYPE_STLSTRING, 0, ((void*)&sSessionId),
					TRACETYPE_POINTER, sizeof(BasicHandler), ((void*)&pHandler),
					TRACETYPE_POINTER, sizeof(WSDDHandler), ((void*)&pHandlerInfo));	  /* AUTOINSERTED TRACE */
	#endif

    poolHandler (sSessionId, pHandler, pHandlerInfo->getScope (), pHandlerInfo->getLibId (), true);
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HandlerPool", "poolWebService", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

AXIS_CPP_NAMESPACE_END

