#ifdef ENABLE_AXISTRACE
/* TRACE ADDED BY THE TRACE INSTRUMENTOR TOOL */
#include "../common/AxisTrace.h"
#endif
/*
 *   Copyright 2003-2004 The Apache Software Foundation.
// (c) Copyright IBM Corp. 2004, 2005 All Rights Reserved
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 */

/**
 * This class 
 *
 * @author Susantha Kumara (skumara@virtusa.com, susantha@opensource.lk)
 *
 */

#include "../platforms/PlatformAutoSense.hpp"

#include "SOAPTransportFactory.h"

#include "../transport/SOAPTransport.h"

#include <stdio.h>

#include "../common/AxisConfig.h"

#include "AxisEngineException.h"

#include "../common/AxisTrace.h"
                                                                                                                             
extern AXIS_CPP_NAMESPACE_PREFIX AxisConfig* g_pConfig;

AXIS_CPP_NAMESPACE_START

const char* SOAPTransportFactory::m_pcLibraryPath = 0;

DLHandler SOAPTransportFactory::m_LibHandler = 0;

CREATE_OBJECT1 SOAPTransportFactory::m_Create = 0;

DELETE_OBJECT1 SOAPTransportFactory::m_Delete = 0;

//void(*SOAPTransportFactory::m_startEventLoop)(void) = NULL;

//void(*SOAPTransportFactory::m_stopEventLoop)(void) = NULL;

SOAPTransportFactory::SOAPTransportFactory()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SOAPTransportFactory", "SOAPTransportFactory", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

	m_LibHandler = 0;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SOAPTransportFactory", "SOAPTransportFactory", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

SOAPTransportFactory::~SOAPTransportFactory()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SOAPTransportFactory", "~SOAPTransportFactory", this, 0);	  /* AUTOINSERTED TRACE */
	#endif


	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SOAPTransportFactory", "~SOAPTransportFactory", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

int SOAPTransportFactory::initialize()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SOAPTransportFactory", "initialize", NULL, 0);	  /* AUTOINSERTED TRACE */
	#endif

	m_pcLibraryPath = g_pConfig->getAxisConfProperty(AXCONF_TRANSPORTHTTP);
	if (!loadLib())
	{
        m_Create = (CREATE_OBJECT1) PLATFORM_GETPROCADDR(m_LibHandler, CREATE_FUNCTION1);
        m_Delete = (DELETE_OBJECT1) PLATFORM_GETPROCADDR(m_LibHandler, DELETE_FUNCTION1);
        if (!m_Create || !m_Delete)
        {
            unloadLib();
            AXISTRACE1("SERVER_ENGINE_LOADING_TRANSPORT_FAILED", CRITICAL);
            char *s = new char[strlen(m_pcLibraryPath)+1];
            strcpy(s,m_pcLibraryPath);
            throw AxisEngineException(SERVER_ENGINE_LOADING_TRANSPORT_FAILED,  s);
        }
#ifdef ENABLE_AXISTRACE
        // Load function to do lib level inits
        void (*initializeLibrary) (AxisTraceEntrypoints*);
        initializeLibrary = (void (*)(AxisTraceEntrypoints*))PLATFORM_GETPROCADDR(m_LibHandler, INIT_FUNCTION);
        if (initializeLibrary)
            (*initializeLibrary)(AxisTrace::getTraceEntrypoints());
#endif
		void (*preloadChannels) (char*, char*);
		preloadChannels = (void (*)(char*, char*))PLATFORM_GETPROCADDR(m_LibHandler, "preloadChannels");
		if (preloadChannels)
			(*preloadChannels)(g_pConfig->getAxisConfProperty( AXCONF_CHANNEL_HTTP), 
				g_pConfig->getAxisConfProperty( AXCONF_SSLCHANNEL_HTTP));
	}
	else
	{
        AXISTRACE1("SERVER_ENGINE_LOADING_TRANSPORT_FAILED", CRITICAL);
        char *s = new char[strlen(m_pcLibraryPath)+1];
        strcpy(s,m_pcLibraryPath);
        throw AxisEngineException(SERVER_ENGINE_LOADING_TRANSPORT_FAILED,  s);
	}
		{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SOAPTransportFactory", "initialize", NULL, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}

}

int SOAPTransportFactory::uninitialize()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SOAPTransportFactory", "uninitialize", NULL, 0);	  /* AUTOINSERTED TRACE */
	#endif

   void (*uninitializeLibrary) (void);
   uninitializeLibrary = (void (*)(void))PLATFORM_GETPROCADDR(m_LibHandler, UNINIT_FUNCTION);
   if (uninitializeLibrary)
        (*uninitializeLibrary)();
		{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (unloadLib());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SOAPTransportFactory", "uninitialize", NULL, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return unloadLib();
		#endif
	}

}

/**
 * Should create an instance of transport of type given by eProtocol
 */

SOAPTransport* SOAPTransportFactory::getTransportObject(AXIS_PROTOCOL_TYPE eProtocol)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SOAPTransportFactory", "getTransportObject", NULL, 1,
					TRACETYPE_DATA, sizeof(AXIS_PROTOCOL_TYPE), ((void*)&eProtocol));	  /* AUTOINSERTED TRACE */
	#endif

	SOAPTransport* pTpt = NULL;
	m_Create(&pTpt);
        pTpt->setProtocol(eProtocol);
		{
		#ifdef ENABLE_AXISTRACE
			SOAPTransport* traceRet = (pTpt);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SOAPTransportFactory", "getTransportObject", NULL, 0,
					TRACETYPE_POINTER, sizeof(SOAPTransport), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return pTpt;
		#endif
	}

}

void SOAPTransportFactory::destroyTransportObject(SOAPTransport* pObject)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SOAPTransportFactory", "destroyTransportObject", NULL, 1,
					TRACETYPE_POINTER, sizeof(SOAPTransport), ((void*)&pObject));	  /* AUTOINSERTED TRACE */
	#endif

	m_Delete(pObject);
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SOAPTransportFactory", "destroyTransportObject", NULL, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

int SOAPTransportFactory::loadLib()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SOAPTransportFactory", "loadLib", NULL, 0);	  /* AUTOINSERTED TRACE */
	#endif

    m_LibHandler = PLATFORM_LOADLIB(m_pcLibraryPath);
    if (!m_LibHandler)
    {
        AXISTRACE1("SERVER_ENGINE_LOADING_TRANSPORT_FAILED", CRITICAL);
        long dwError = GETLASTERROR
        string *    message = PLATFORM_GET_ERROR_MESSAGE( dwError);
        char        fullMessage[1024];
        sprintf(fullMessage,
                "Failed to load transport within server engine: \n \
                Error Message='%s'\
                Error Code='%d'\n \
                Load lib error='%s' \n",
                message->c_str(), (int) dwError, PLATFORM_LOADLIB_ERROR);
        delete( message);
        throw AxisEngineException(SERVER_ENGINE_LOADING_TRANSPORT_FAILED, fullMessage);
    }
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SOAPTransportFactory", "loadLib", NULL, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}

}

int SOAPTransportFactory::unloadLib()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SOAPTransportFactory", "unloadLib", NULL, 0);	  /* AUTOINSERTED TRACE */
	#endif

    PLATFORM_UNLOADLIB(m_LibHandler);
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SOAPTransportFactory", "unloadLib", NULL, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}

}

/*
void SOAPTransportFactory::startEventLoop()
{
    if (m_startEventLoop)
        (*m_startEventLoop)();
}
void SOAPTransportFactory::stopEventLoop()
{
    if (m_stopEventLoop)
        (*m_stopEventLoop)();
}
*/

AXIS_CPP_NAMESPACE_END

