#ifdef ENABLE_AXISTRACE
/* TRACE ADDED BY THE TRACE INSTRUMENTOR TOOL */
#include "../../common/AxisTrace.h"
#endif
/*
 *   Copyright 2003-2004 The Apache Software Foundation.
// (c) Copyright IBM Corp. 2004, 2005 All Rights Reserved
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 *
 * @author Samisa Abeysinghe (sabeysinghe@virtusa.com)
 * @author Roshan Weerasuriya (roshan@opensource.lk, roshanw@jkcsworld.com)
 */

#include <axis/client/Stub.hpp>

#include <stdio.h>

#include "../../transport/SOAPTransport.h"

#include <axis/AxisWrapperAPI.hpp>

#include "../../soap/SoapSerializer.h"

#include "../../common/AxisUtils.h"

#include "../../soap/apr_base64.h"

#include "../../common/AxisConfig.h"

#include "../../common/AxisGenException.h"

extern AXIS_CPP_NAMESPACE_PREFIX AxisConfig* g_pConfig;

AXIS_CPP_NAMESPACE_USE
    Stub::Stub (const char *pcEndPointUri, AXIS_PROTOCOL_TYPE eProtocol):
m_pcUsername (NULL),
m_pcPassword (NULL),
m_proxyUsername (NULL),
m_proxyPassword (NULL)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Stub", "Stub", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pcEndPointUri),
					TRACETYPE_DATA, sizeof(AXIS_PROTOCOL_TYPE), ((void*)&eProtocol));	  /* AUTOINSERTED TRACE */
	#endif

    m_pCall = new Call ();
    m_pCall->setProtocol (eProtocol);
    m_pTransport = m_pCall->getTransport ();
    m_pTransport->setEndpointUri( pcEndPointUri);
    
    // SSL channel related initilizations
    char * pcSSLChannelInfo = g_pConfig->getAxisConfProperty( AXCONF_SECUREINFO);
    if( pcSSLChannelInfo && strlen( pcSSLChannelInfo) > 0)
    {
        char * pszArgPtr = NULL;
        int iArgIndex = 0;
        pszArgPtr = strtok( pcSSLChannelInfo, ",");
        while( pszArgPtr != NULL && iArgIndex < 8)
        {
            m_sArguments[iArgIndex] = pszArgPtr;
            iArgIndex++;
            pszArgPtr = strtok( NULL, ",");
            while( pszArgPtr != NULL && *pszArgPtr == ' ' && *pszArgPtr != '\0')
            {
                pszArgPtr++;
            }
        }
    }
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Stub", "Stub", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

Stub::~Stub ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Stub", "~Stub", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    delete m_pCall;
    m_pCall = NULL;
    unsigned int j = 0;
    for (j = 0; j < m_vSOAPHeaderBlocks.size (); j++)
    {
	delete m_vSOAPHeaderBlocks[j];
	m_vSOAPHeaderBlocks[j] = NULL;
    }
    if (m_pcUsername)
	delete[]m_pcUsername;
    if (m_pcPassword)
	delete[]m_pcPassword;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Stub", "~Stub", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void
Stub::setEndPoint (const char *pcEndPoint)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Stub", "setEndPoint", this, 1,
					TRACETYPE_STRING, 0, ((void*)&pcEndPoint));	  /* AUTOINSERTED TRACE */
	#endif

    if (m_pTransport)
    {
        m_pTransport->setEndpointUri( pcEndPoint);
    }
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Stub", "setEndPoint", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void
Stub::setTransportProperty (const char *pcKey, const char *pcValue)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Stub", "setTransportProperty", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pcKey),
					TRACETYPE_STRING, 0, ((void*)&pcValue));	  /* AUTOINSERTED TRACE */
	#endif

    if (m_pTransport)
    {
		if(!pcKey)
		{
			m_pTransport=NULL; //this should be set to null. otherwise the previous setting can be still there.
				{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Stub", "setTransportProperty", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
// if the pcKey is set to null, the method returns
		}
		else if (strcmp (pcKey, "SOAPAction") == 0)	//Samisa: Setting SOAPAction, 
	    //may need to add '"' to value - AXISCPP-531
		{
	    // both "" missing
			if (pcValue
			&& (pcValue[0] != '"'
		    && pcValue[strlen (pcValue) - 1] != '"'))
			{
				char *tempvalue = new char[strlen (pcValue) + 3];
				sprintf (tempvalue, "\"%s\"", pcValue);
				m_pTransport->setTransportProperty (pcKey, tempvalue);
				delete[]tempvalue;
			}
			else if (pcValue && (pcValue[0] != '"'))	//starting '"' missing
			{
				char *tempvalue = new char[strlen (pcValue) + 2];
				sprintf (tempvalue, "\"%s", pcValue);
				m_pTransport->setTransportProperty (pcKey, tempvalue);
				delete[]tempvalue;
			}
			else if (pcValue && (pcValue[strlen (pcValue) - 1] != '"'))	// ending '"' missing
			{
				char *tempvalue = new char[strlen (pcValue) + 3];
				sprintf (tempvalue, "\"%s\"", pcValue);
				m_pTransport->setTransportProperty (pcKey, tempvalue);
				delete[]tempvalue;
			}
			else		// both "" present
				m_pTransport->setTransportProperty (pcKey, pcValue);
		}
	
		else
		{
			 m_pTransport->setTransportProperty (pcKey, pcValue);
		}
    }
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Stub", "setTransportProperty", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

const char *
Stub::getTransportProperty (const char *key, bool response)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Stub", "getTransportProperty", this, 2,
					TRACETYPE_STRING, 0, ((void*)&key),
					TRACETYPE_BOOL, 0, ((void*)&response));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (m_pCall->getTransportProperty (key, response));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Stub", "getTransportProperty", this, 0,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pCall->getTransportProperty (key, response);
		#endif
	}

}

const char *
Stub::getFirstTransportPropertyKey (bool response)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Stub", "getFirstTransportPropertyKey", this, 1,
					TRACETYPE_BOOL, 0, ((void*)&response));	  /* AUTOINSERTED TRACE */
	#endif

    if (m_pTransport)
		{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (m_pTransport->getFirstTransportPropertyKey (response));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Stub", "getFirstTransportPropertyKey", this, 1,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pTransport->getFirstTransportPropertyKey (response);
		#endif
	}

    else
		{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Stub", "getFirstTransportPropertyKey", this, 2,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

}

const char *
Stub::getNextTransportPropertyKey (bool response)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Stub", "getNextTransportPropertyKey", this, 1,
					TRACETYPE_BOOL, 0, ((void*)&response));	  /* AUTOINSERTED TRACE */
	#endif

    if (m_pTransport)
		{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (m_pTransport->getNextTransportPropertyKey (response));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Stub", "getNextTransportPropertyKey", this, 1,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pTransport->getNextTransportPropertyKey (response);
		#endif
	}

    else
		{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Stub", "getNextTransportPropertyKey", this, 2,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

}

const char *
Stub::getCurrentTransportPropertyKey (bool response)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Stub", "getCurrentTransportPropertyKey", this, 1,
					TRACETYPE_BOOL, 0, ((void*)&response));	  /* AUTOINSERTED TRACE */
	#endif

    if (m_pTransport)
	   	{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (m_pTransport->getCurrentTransportPropertyKey (response));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Stub", "getCurrentTransportPropertyKey", this, 1,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pTransport->getCurrentTransportPropertyKey (response);
		#endif
	}

    else
	   	{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Stub", "getCurrentTransportPropertyKey", this, 2,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

}

const char *
Stub::getCurrentTransportPropertyValue (bool response)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Stub", "getCurrentTransportPropertyValue", this, 1,
					TRACETYPE_BOOL, 0, ((void*)&response));	  /* AUTOINSERTED TRACE */
	#endif

    if (m_pTransport)
		{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (m_pTransport->getCurrentTransportPropertyValue (response));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Stub", "getCurrentTransportPropertyValue", this, 1,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pTransport->getCurrentTransportPropertyValue (response);
		#endif
	}

    else
		{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Stub", "getCurrentTransportPropertyValue", this, 2,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

}

void
Stub::deleteCurrentTransportProperty (bool response)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Stub", "deleteCurrentTransportProperty", this, 1,
					TRACETYPE_BOOL, 0, ((void*)&response));	  /* AUTOINSERTED TRACE */
	#endif

    if (m_pTransport)
	m_pTransport->deleteCurrentTransportProperty (response);
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Stub", "deleteCurrentTransportProperty", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void
Stub::deleteTransportProperty (char *pcKey, unsigned int uiOccurance)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Stub", "deleteTransportProperty", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pcKey),
					TRACETYPE_UINT, 0, ((void*)&uiOccurance));	  /* AUTOINSERTED TRACE */
	#endif

    if (m_pTransport)
	    m_pTransport->deleteTransportProperty (pcKey, uiOccurance);
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Stub", "deleteTransportProperty", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void
Stub::setHandlerProperty (AxisChar * name, void *value, int len)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Stub", "setHandlerProperty", this, 3,
					TRACETYPE_STRING, 0, ((void*)&name),
					TRACETYPE_POINTER, 1, ((void*)&value),
					TRACETYPE_INT, 0, ((void*)&len));	  /* AUTOINSERTED TRACE */
	#endif

    m_pCall->setHandlerProperty (name, value, len);
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Stub", "setHandlerProperty", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

IHeaderBlock *
Stub::createSOAPHeaderBlock (AxisChar * pachLocalName, AxisChar * pachUri)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Stub", "createSOAPHeaderBlock", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pachLocalName),
					TRACETYPE_STRING, 0, ((void*)&pachUri));	  /* AUTOINSERTED TRACE */
	#endif

    if (pachLocalName && pachUri)
    {
	if ((AxisUtils::isStringOnlyWithSpaces (pachLocalName)) ||
	    (AxisUtils::isStringOnlyWithSpaces (pachUri)))
	{
	    	{
		#ifdef ENABLE_AXISTRACE
			IHeaderBlock* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Stub", "createSOAPHeaderBlock", this, 1,
					TRACETYPE_POINTER, sizeof(IHeaderBlock), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

	}
	else
	{
	    IHeaderBlock *pNewSoapheader =
		m_pCall->createHeaderBlock (pachLocalName, pachUri);
	    m_vSOAPHeaderBlocks.push_back (pNewSoapheader);
	    	{
		#ifdef ENABLE_AXISTRACE
			IHeaderBlock* traceRet = (pNewSoapheader);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Stub", "createSOAPHeaderBlock", this, 2,
					TRACETYPE_POINTER, sizeof(IHeaderBlock), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return pNewSoapheader;
		#endif
	}

	}
    }
    else
    {
		{
		#ifdef ENABLE_AXISTRACE
			IHeaderBlock* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Stub", "createSOAPHeaderBlock", this, 3,
					TRACETYPE_POINTER, sizeof(IHeaderBlock), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

    }
}

IHeaderBlock *
Stub::getFirstSOAPHeaderBlock ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Stub", "getFirstSOAPHeaderBlock", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    m_viCurrentSOAPHeaderBlock = m_vSOAPHeaderBlocks.begin ();
    if (m_viCurrentSOAPHeaderBlock == m_vSOAPHeaderBlocks.end ())
		{
		#ifdef ENABLE_AXISTRACE
			IHeaderBlock* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Stub", "getFirstSOAPHeaderBlock", this, 1,
					TRACETYPE_POINTER, sizeof(IHeaderBlock), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

    else
		{
		#ifdef ENABLE_AXISTRACE
			IHeaderBlock* traceRet = ((*m_viCurrentSOAPHeaderBlock));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Stub", "getFirstSOAPHeaderBlock", this, 2,
					TRACETYPE_POINTER, sizeof(IHeaderBlock), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return (*m_viCurrentSOAPHeaderBlock);
		#endif
	}

}

IHeaderBlock *
Stub::getNextSOAPHeaderBlock ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Stub", "getNextSOAPHeaderBlock", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    //already at the end?
    if (m_viCurrentSOAPHeaderBlock == m_vSOAPHeaderBlocks.end ())
		{
		#ifdef ENABLE_AXISTRACE
			IHeaderBlock* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Stub", "getNextSOAPHeaderBlock", this, 1,
					TRACETYPE_POINTER, sizeof(IHeaderBlock), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

    m_viCurrentSOAPHeaderBlock++;
    if (m_viCurrentSOAPHeaderBlock == m_vSOAPHeaderBlocks.end ())
		{
		#ifdef ENABLE_AXISTRACE
			IHeaderBlock* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Stub", "getNextSOAPHeaderBlock", this, 2,
					TRACETYPE_POINTER, sizeof(IHeaderBlock), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

    else
		{
		#ifdef ENABLE_AXISTRACE
			IHeaderBlock* traceRet = ((*m_viCurrentSOAPHeaderBlock));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Stub", "getNextSOAPHeaderBlock", this, 3,
					TRACETYPE_POINTER, sizeof(IHeaderBlock), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return (*m_viCurrentSOAPHeaderBlock);
		#endif
	}

}

IHeaderBlock *
Stub::getCurrentSOAPHeaderBlock ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Stub", "getCurrentSOAPHeaderBlock", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    if (m_viCurrentSOAPHeaderBlock != m_vSOAPHeaderBlocks.end ())
    {
		{
		#ifdef ENABLE_AXISTRACE
			IHeaderBlock* traceRet = ((*m_viCurrentSOAPHeaderBlock));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Stub", "getCurrentSOAPHeaderBlock", this, 1,
					TRACETYPE_POINTER, sizeof(IHeaderBlock), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return (*m_viCurrentSOAPHeaderBlock);
		#endif
	}

    }
    	{
		#ifdef ENABLE_AXISTRACE
			IHeaderBlock* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Stub", "getCurrentSOAPHeaderBlock", this, 2,
					TRACETYPE_POINTER, sizeof(IHeaderBlock), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

}

void
Stub::deleteCurrentSOAPHeaderBlock ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Stub", "deleteCurrentSOAPHeaderBlock", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    if (m_viCurrentSOAPHeaderBlock != m_vSOAPHeaderBlocks.end ())
    {
	delete (*m_viCurrentSOAPHeaderBlock);
	m_vSOAPHeaderBlocks.erase (m_viCurrentSOAPHeaderBlock);
    }
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Stub", "deleteCurrentSOAPHeaderBlock", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void
Stub::deleteSOAPHeaderBlock (IHeaderBlock * pHeaderBlock)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Stub", "deleteSOAPHeaderBlock", this, 1,
					TRACETYPE_POINTER, sizeof(IHeaderBlock), ((void*)&pHeaderBlock));	  /* AUTOINSERTED TRACE */
	#endif

    vector < IHeaderBlock * >::iterator currentSOAPHeaderBlock =
	m_vSOAPHeaderBlocks.begin ();
    bool bDone = false;
    while (!bDone && currentSOAPHeaderBlock != m_vSOAPHeaderBlocks.end ())
    {
	if (pHeaderBlock == *currentSOAPHeaderBlock)
	{
	    delete (*currentSOAPHeaderBlock);
	    m_vSOAPHeaderBlocks.erase (currentSOAPHeaderBlock);
	    bDone = true;
	}
	currentSOAPHeaderBlock++;
    }
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Stub", "deleteSOAPHeaderBlock", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void
Stub::setSOAPHeaders ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Stub", "setSOAPHeaders", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    SoapSerializer *pSerializer = NULL;
    if (m_pCall)
	pSerializer = m_pCall->getSOAPSerializer ();
    if (pSerializer)
    {
	for (unsigned int i = 0; i < m_vSOAPHeaderBlocks.size (); i++)
	{
	    pSerializer->addHeaderBlock (m_vSOAPHeaderBlocks[i]);
	}
    }
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Stub", "setSOAPHeaders", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void
Stub::applyUserPreferences ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Stub", "applyUserPreferences", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    setSOAPHeaders ();
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Stub", "applyUserPreferences", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void
Stub::setProxy (const char *pcProxyHost, unsigned int uiProxyPort)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Stub", "setProxy", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pcProxyHost),
					TRACETYPE_UINT, 0, ((void*)&uiProxyPort));	  /* AUTOINSERTED TRACE */
	#endif

    if (m_pTransport)
    {
        m_pTransport->setProxy (pcProxyHost, uiProxyPort);
    }
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Stub", "setProxy", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void
Stub::setTransportTimeout (long lSeconds)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Stub", "setTransportTimeout", this, 1,
					TRACETYPE_LONG, 0, ((void*)&lSeconds));	  /* AUTOINSERTED TRACE */
	#endif

    if (m_pTransport)
    {
        m_pTransport->setTimeout (lSeconds);
    }
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Stub", "setTransportTimeout", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

int
Stub::getStatus ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Stub", "getStatus", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    if (m_pCall == NULL)
		{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Stub", "getStatus", this, 1,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}

    else
		{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (m_pCall->getStatus ());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Stub", "getStatus", this, 2,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pCall->getStatus ();
		#endif
	}

}

void
Stub::setMaintainSession (bool bSession)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Stub", "setMaintainSession", this, 1,
					TRACETYPE_BOOL, 0, ((void*)&bSession));	  /* AUTOINSERTED TRACE */
	#endif

    if (m_pTransport)
    {
	m_pTransport->setMaintainSession (bSession);
    }
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Stub", "setMaintainSession", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void
Stub::setTransportProtocol (AXIS_PROTOCOL_TYPE eProtocol)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Stub", "setTransportProtocol", this, 1,
					TRACETYPE_DATA, sizeof(AXIS_PROTOCOL_TYPE), ((void*)&eProtocol));	  /* AUTOINSERTED TRACE */
	#endif

    if (m_pCall)
	m_pCall->setProtocol (eProtocol);
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Stub", "setTransportProtocol", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

AXIS_PROTOCOL_TYPE
Stub::getTransportProtocol ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Stub", "getTransportProtocol", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    if (m_pCall)
		{
		#ifdef ENABLE_AXISTRACE
			AXIS_PROTOCOL_TYPE traceRet = (m_pCall->getProtocol ());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Stub", "getTransportProtocol", this, 1,
					TRACETYPE_DATA, sizeof(AXIS_PROTOCOL_TYPE), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pCall->getProtocol ();
		#endif
	}

    else
		{
		#ifdef ENABLE_AXISTRACE
			AXIS_PROTOCOL_TYPE traceRet = (APT_UNKNOWN);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Stub", "getTransportProtocol", this, 2,
					TRACETYPE_DATA, sizeof(AXIS_PROTOCOL_TYPE), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return APT_UNKNOWN;
		#endif
	}

}

IHeaderBlock *
Stub::createSOAPHeaderBlock (AxisChar * pachLocalName, AxisChar * pachUri,
			     AxisChar * pachPrefix)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Stub", "createSOAPHeaderBlock", this, 3,
					TRACETYPE_STRING, 0, ((void*)&pachLocalName),
					TRACETYPE_STRING, 0, ((void*)&pachUri),
					TRACETYPE_STRING, 0, ((void*)&pachPrefix));	  /* AUTOINSERTED TRACE */
	#endif

    if (pachLocalName && pachUri && pachPrefix)
    {
	if (AxisUtils::isStringOnlyWithSpaces (pachLocalName) ||
	    AxisUtils::isStringOnlyWithSpaces (pachUri))
	{
	    	{
		#ifdef ENABLE_AXISTRACE
			IHeaderBlock* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Stub", "createSOAPHeaderBlock", this, 1,
					TRACETYPE_POINTER, sizeof(IHeaderBlock), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

	}
	else
	{
	    IHeaderBlock *pNewSoapheader =
		m_pCall->createHeaderBlock (pachLocalName, pachUri,
					    pachPrefix);
	    m_vSOAPHeaderBlocks.push_back (pNewSoapheader);
	    	{
		#ifdef ENABLE_AXISTRACE
			IHeaderBlock* traceRet = (pNewSoapheader);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Stub", "createSOAPHeaderBlock", this, 2,
					TRACETYPE_POINTER, sizeof(IHeaderBlock), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return pNewSoapheader;
		#endif
	}

	}
    }
    else
    {
		{
		#ifdef ENABLE_AXISTRACE
			IHeaderBlock* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Stub", "createSOAPHeaderBlock", this, 3,
					TRACETYPE_POINTER, sizeof(IHeaderBlock), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

    }
}

void
Stub::setUsername (const char *pcUsername)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Stub", "setUsername", this, 1,
					TRACETYPE_STRING, 0, ((void*)&pcUsername));	  /* AUTOINSERTED TRACE */
	#endif

    if (m_pcUsername)
    {
	delete[]m_pcUsername;
	m_pcUsername = NULL;
    }
    if (!pcUsername)
		{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Stub", "setUsername", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}

    m_pcUsername = new char[strlen (pcUsername) + 1];
    strcpy (m_pcUsername, pcUsername);
    if (m_pcPassword)
    {
	setAuthorizationHeader ();
    }
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Stub", "setUsername", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void
Stub::setProxyUsername (const char *pcProxyUsername)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Stub", "setProxyUsername", this, 1,
					TRACETYPE_STRING, 0, ((void*)&pcProxyUsername));	  /* AUTOINSERTED TRACE */
	#endif

    if (m_proxyUsername)
    {
	delete[]m_proxyUsername;
	m_proxyUsername = NULL;
    }
    if (!pcProxyUsername)
		{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Stub", "setProxyUsername", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}

    m_proxyUsername = new char[strlen (pcProxyUsername) + 1];
    strcpy (m_proxyUsername, pcProxyUsername);
    if (m_proxyPassword)
    {
	setProxyAuthorizationHeader ();
    }
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Stub", "setProxyUsername", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void
Stub::setPassword (const char *pcPassword)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Stub", "setPassword", this, 1,
					TRACETYPE_STRING, 0, ((void*)&pcPassword));	  /* AUTOINSERTED TRACE */
	#endif

    if (m_pcPassword)
    {
	delete[]m_pcPassword;
	m_pcPassword = NULL;
    }
    if (!pcPassword)
		{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Stub", "setPassword", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}

    m_pcPassword = new char[strlen (pcPassword) + 1];
    strcpy (m_pcPassword, pcPassword);
    if (m_pcUsername)
    {
	setAuthorizationHeader ();
    }
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Stub", "setPassword", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void
Stub::setProxyPassword (const char *pcProxyPassword)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Stub", "setProxyPassword", this, 1,
					TRACETYPE_STRING, 0, ((void*)&pcProxyPassword));	  /* AUTOINSERTED TRACE */
	#endif

    if (m_proxyPassword)
    {
	delete[]m_proxyPassword;
	m_proxyPassword = NULL;
    }
    if (!pcProxyPassword)
		{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Stub", "setProxyPassword", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}

    m_proxyPassword = new char[strlen (pcProxyPassword) + 1];
    strcpy (m_proxyPassword, pcProxyPassword);
    if (m_proxyUsername)
    {
	setProxyAuthorizationHeader ();
    }
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Stub", "setProxyPassword", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

const char *
Stub::getUsername ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Stub", "getUsername", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (m_pcUsername);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Stub", "getUsername", this, 0,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pcUsername;
		#endif
	}

}

const char *
Stub::getProxyUsername ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Stub", "getProxyUsername", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (m_proxyUsername);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Stub", "getProxyUsername", this, 0,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_proxyUsername;
		#endif
	}

}

const char *
Stub::getPassword ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Stub", "getPassword", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (m_pcPassword);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Stub", "getPassword", this, 0,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pcPassword;
		#endif
	}

}

const char *
Stub::getProxyPassword ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Stub", "getProxyPassword", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (m_proxyPassword);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Stub", "getProxyPassword", this, 0,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_proxyPassword;
		#endif
	}

}

void
Stub::setAuthorizationHeader ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Stub", "setAuthorizationHeader", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    char *cpUsernamePassword =
	new char[strlen (m_pcUsername) + strlen (m_pcPassword) + 2];
    strcpy (cpUsernamePassword, m_pcUsername);
    strcat (cpUsernamePassword, ":");
    strcat (cpUsernamePassword, m_pcPassword);
    int len = apr_base64_encode_len (strlen (cpUsernamePassword));
    AxisChar *base64Value = new AxisChar[len + 1];
    len =
	apr_base64_encode_binary (base64Value,
				  (const unsigned char *) cpUsernamePassword,
				  strlen (cpUsernamePassword));
    std::string strValue = "Basic ";
    strValue += base64Value;
    if (m_pTransport)
	m_pTransport->setTransportProperty ("Authorization",
					    strValue.c_str ());
    delete[]cpUsernamePassword;
    delete[]base64Value;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Stub", "setAuthorizationHeader", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void 
Stub::setProxyAuthorizationHeader ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Stub", "setProxyAuthorizationHeader", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    char* cpUsernamePassword = new char[strlen( m_proxyUsername) + strlen( m_proxyPassword ) + 2];
    strcpy( cpUsernamePassword, m_proxyUsername );
    strcat( cpUsernamePassword, ":" );
    strcat( cpUsernamePassword, m_proxyPassword );
    int len = apr_base64_encode_len (strlen(cpUsernamePassword));
     
	AxisChar* base64Value = new AxisChar[len + 1];
    len = apr_base64_encode_binary ( base64Value, (const unsigned char*)cpUsernamePassword, strlen
                                                                                   (cpUsernamePassword));
  
	std::string strValue = "Basic ";
    strValue += base64Value;
  
	this->setTransportProperty( "Proxy-Authorization", strValue.c_str());
    delete [] cpUsernamePassword;
    delete [] base64Value;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Stub", "setProxyAuthorizationHeader", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

ISoapAttachment* Stub::createSoapAttachment()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Stub", "createSoapAttachment", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

		{
		#ifdef ENABLE_AXISTRACE
			ISoapAttachment* traceRet = (m_pCall->createSoapAttachment());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Stub", "createSoapAttachment", this, 0,
					TRACETYPE_POINTER, sizeof(ISoapAttachment), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pCall->createSoapAttachment();
		#endif
	}

}

/*Methods for supporting SecureChannel*/

void Stub::SetSecure( char * pszArguments, ...)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Stub", "SetSecure", this, 1,
					TRACETYPE_STRING, 0, ((void*)&pszArguments) );	  /* AUTOINSERTED TRACE */
	#endif

    int iArgIndex = 0;
    va_list args;
    char *  pszArg = NULL;
    
    va_start( args, pszArguments);
    
    if( (pszArg = pszArguments) != NULL)
    {
        do
        {
            if( pszArg == (char *) 1)
            {
                m_sArguments[iArgIndex] = "true";
            }
            else
            {
                m_sArguments[iArgIndex] = pszArg;
            }
            iArgIndex++;
        } while( (pszArg = va_arg( args, char *)) != NULL && iArgIndex < 8);
        if( iArgIndex == 6)
        {
            m_sArguments[iArgIndex] = "false";
        }
    }
    va_end( args);
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Stub", "SetSecure", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void Stub::includeSecure()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Stub", "includeSecure", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    m_pCall->setTransportProperty( SECURE_PROPERTIES, (const char *) &m_sArguments);
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Stub", "includeSecure", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

