#ifdef ENABLE_AXISTRACE
/* TRACE ADDED BY THE TRACE INSTRUMENTOR TOOL */
#include "../../common/AxisTrace.h"
#endif
/*
 *   Copyright 2003-2004 The Apache Software Foundation.
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 *
 *
 * @author Susantha Kumara (susantha@opensource.lk, skumara@virtusa.com)
 *
 */

#include "Apache2Transport.h"

#define AXIS_URI_EXTENSION "/axis"

/* Following is the character that should be used to separate the method name in
 * the SOAPAction header value. Ex: "http://localhost:80/axis/Calculator#Add"
 */

#define SOAPACTION_METHODNAME_SEPARATOR "#"

Apache2Transport::Apache2Transport(void* pContext)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Apache2Transport", "Apache2Transport", this, 1,
					TRACETYPE_POINTER, 1, ((void*)&pContext));	  /* AUTOINSERTED TRACE */
	#endif

    m_eProtocolType = APTHTTP1_1;
	m_pContext = pContext;
#ifndef CHUNCKED_DATA_SUPPORTED
    m_pBuffers = new BufferInfo[NO_OF_SERIALIZE_BUFFERS];
	memset((void*)m_pBuffers, 0, sizeof(BufferInfo)*NO_OF_SERIALIZE_BUFFERS);
#endif
	
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Apache2Transport", "Apache2Transport", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

Apache2Transport::~Apache2Transport()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Apache2Transport", "~Apache2Transport", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

	if(m_pBuffers)
	{
		delete [] m_pBuffers;
		m_pBuffers = NULL;
	}
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Apache2Transport", "~Apache2Transport", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

AXIS_TRANSPORT_STATUS Apache2Transport::sendBytes(const char* pcSendBuffer, const void* pBufferId)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Apache2Transport", "sendBytes", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pcSendBuffer),
					TRACETYPE_POINTER, 1, ((void*)&pBufferId));	  /* AUTOINSERTED TRACE */
	#endif

#ifndef CHUNCKED_DATA_SUPPORTED
    int index;
#endif
    if (!pBufferId) /* temporary buffers should always sent immediately */
    {
		ap_rputs(pcSendBuffer, (request_rec*)m_pContext);
        	{
		#ifdef ENABLE_AXISTRACE
			AXIS_TRANSPORT_STATUS traceRet = (TRANSPORT_FINISHED);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Apache2Transport", "sendBytes", this, 1,
					TRACETYPE_DATA, sizeof(AXIS_TRANSPORT_STATUS), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return TRANSPORT_FINISHED;
		#endif
	}

    }
#ifdef CHUNCKED_DATA_SUPPORTED
    // Do we need to send any headers and length of this chunk ? 
	ap_rputs(pcSendBuffer, (request_rec*)m_pContext);
    /* Do we need to send any indication to mark the end of this 
     * chunk ?
     */ 
    	{
		#ifdef ENABLE_AXISTRACE
			AXIS_TRANSPORT_STATUS traceRet = (TRANSPORT_FINISHED);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Apache2Transport", "sendBytes", this, 2,
					TRACETYPE_DATA, sizeof(AXIS_TRANSPORT_STATUS), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return TRANSPORT_FINISHED;
		#endif
	}

#else
    for (index = 0; index < NO_OF_SERIALIZE_BUFFERS; index++)
    {
        if (!m_pBuffers[index].pcBuffer)
        {
            m_pBuffers[index].pcBuffer = pcSendBuffer;
            m_pBuffers[index].pBufferId = pBufferId;
            break;
        }
    }
    	{
		#ifdef ENABLE_AXISTRACE
			AXIS_TRANSPORT_STATUS traceRet = (TRANSPORT_IN_PROGRESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Apache2Transport", "sendBytes", this, 3,
					TRACETYPE_DATA, sizeof(AXIS_TRANSPORT_STATUS), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return TRANSPORT_IN_PROGRESS;
		#endif
	}

#endif	
}

int Apache2Transport::setTransportProperty(AXIS_TRANSPORT_INFORMATION_TYPE type, const char* value)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Apache2Transport", "setTransportProperty", this, 2,
					TRACETYPE_DATA, sizeof(AXIS_TRANSPORT_INFORMATION_TYPE), ((void*)&type),
					TRACETYPE_STRING, 0, ((void*)&value));	  /* AUTOINSERTED TRACE */
	#endif

    const char* key = NULL;
    switch (type)
    {
        case SOAPACTION_HEADER:  /* needed only in the client side ? */
            break;
        case SERVICE_URI:        /* need to set ? */
            break;
        case OPERATION_NAME:     /* need to set ? */
            break;
        case SOAP_MESSAGE_LENGTH:
		/* This is apache module and transport is http so the key */ 
            key = "Content-Length"; 
            break;
/*	case CONTENT_TYPE:
	    ((request_rec*)m_pContext)->content_type = value;
	    break; */
        default:;
    }
    if (key)
    {
		ap_table_set(((request_rec*)m_pContext)->headers_out, key, value);
#ifdef CHUNCKED_DATA_SUPPORTED
		ap_send_http_header((request_rec*)m_pContext);
		/* Should we remove the sent headers ? */
#endif
	}
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (0);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Apache2Transport", "setTransportProperty", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return 0;
		#endif
	}

}

AXIS_TRANSPORT_STATUS Apache2Transport::flushOutput()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Apache2Transport", "flushOutput", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

#ifndef CHUNCKED_DATA_SUPPORTED
    int contentLength = 0;
    int index;
    char strtonum[8];
#endif
#ifdef CHUNCKED_DATA_SUPPORTED
    /* headers have already been sent. see set_transport_information
     * http body too have been sent
     * Do we need to send any indication to mark end of chuncked 
     * data ?  
     */
#else
    /* Calculate Content-Length and set header */
    for (index = 0; index < NO_OF_SERIALIZE_BUFFERS; index++)
    {
        if (!m_pBuffers[index].pcBuffer)
            break;
        contentLength += strlen(m_pBuffers[index].pcBuffer);
    }
    if (contentLength != 0)     /* do only if the http body is not empty. */
    {
        sprintf(strtonum, "%d", contentLength);
        setTransportProperty(SOAP_MESSAGE_LENGTH, strtonum);
        ap_send_http_header((request_rec*)m_pContext);
        /* Send all buffers */
        for (index = 0; index < NO_OF_SERIALIZE_BUFFERS; index++)
        {
            if (!m_pBuffers[index].pcBuffer)
                break;
            ap_rputs(m_pBuffers[index].pcBuffer, (request_rec*)m_pContext);
            /* Let Axis know that the buffer is no longer in use */
            m_pReleaseBufferCallback(m_pBuffers[index].pcBuffer, 
                m_pBuffers[index].pBufferId);
        }
    }
#endif
		{
		#ifdef ENABLE_AXISTRACE
			AXIS_TRANSPORT_STATUS traceRet = (TRANSPORT_FINISHED);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Apache2Transport", "flushOutput", this, 0,
					TRACETYPE_DATA, sizeof(AXIS_TRANSPORT_STATUS), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return TRANSPORT_FINISHED;
		#endif
	}

}

AXIS_TRANSPORT_STATUS Apache2Transport::getBytes(char* pBuffer, int* piSize)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Apache2Transport", "getBytes", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pBuffer),
					TRACETYPE_POINTER, sizeof(int), ((void*)&piSize));	  /* AUTOINSERTED TRACE */
	#endif

	
	int nBufSize = *piSize;
    int len_read;
    len_read = ap_get_client_block((request_rec*) m_pContext, pBuffer, *piSize);
	
 	if (strstr(pBuffer, "Content-Id")) {
		pAttachmentHelper = new AttachmentHelper();
		//char *pAttachBuffer = (char*)malloc(1000);
		//char *mimeBoundary = (char*)malloc(1000);		
		pAttachmentHelper->extract_Attachment(pBuffer);		
		pAttachmentHelper->extract_SOAPMimeHeaders(pBuffer);	
		pAttachmentHelper->extract_Soap(pBuffer);
				
	}
    *piSize = len_read;
    if (len_read < nBufSize)
    {
        pBuffer[len_read] = '\0';
        	{
		#ifdef ENABLE_AXISTRACE
			AXIS_TRANSPORT_STATUS traceRet = (TRANSPORT_FINISHED);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Apache2Transport", "getBytes", this, 1,
					TRACETYPE_DATA, sizeof(AXIS_TRANSPORT_STATUS), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return TRANSPORT_FINISHED;
		#endif
	}

    }
    else
        	{
		#ifdef ENABLE_AXISTRACE
			AXIS_TRANSPORT_STATUS traceRet = (TRANSPORT_IN_PROGRESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Apache2Transport", "getBytes", this, 2,
					TRACETYPE_DATA, sizeof(AXIS_TRANSPORT_STATUS), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return TRANSPORT_IN_PROGRESS;
		#endif
	}

}

const char* Apache2Transport::getTransportProperty(AXIS_TRANSPORT_INFORMATION_TYPE eType)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Apache2Transport", "getTransportProperty", this, 1,
					TRACETYPE_DATA, sizeof(AXIS_TRANSPORT_INFORMATION_TYPE), ((void*)&eType));	  /* AUTOINSERTED TRACE */
	#endif

    const char* pcValue;
	/* the member, "path" of "parsed_uri" contains the uri of the
       request (i.e "/abc/xyz" part of http://somehost/abc/xyz) */
    const char* pcUriPath =  ((request_rec*)m_pContext)->parsed_uri.path;
    switch (eType)
    {
        case SOAPACTION_HEADER:
				{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (getTransportProperty("SOAPAction"));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Apache2Transport", "getTransportProperty", this, 1,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return getTransportProperty("SOAPAction");
		#endif
	}

        case SERVICE_URI:
            if (strstr(pcUriPath, AXIS_URI_EXTENSION))
            {
                	{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (strstr(pcUriPath, AXIS_URI_EXTENSION) +
                    strlen(AXIS_URI_EXTENSION) + 1);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Apache2Transport", "getTransportProperty", this, 2,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return strstr(pcUriPath, AXIS_URI_EXTENSION) +
                    strlen(AXIS_URI_EXTENSION) + 1;
		#endif
	}

            }
            else
            {
                	{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (pcUriPath);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Apache2Transport", "getTransportProperty", this, 3,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return pcUriPath;
		#endif
	}

            }
        case OPERATION_NAME:
            pcValue = getTransportProperty("SOAPAction");
            if (pcValue)
            {
                if (strstr(pcValue, SOAPACTION_METHODNAME_SEPARATOR))
                {
                    	{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (strstr(pcValue, SOAPACTION_METHODNAME_SEPARATOR) +
                          strlen(SOAPACTION_METHODNAME_SEPARATOR));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Apache2Transport", "getTransportProperty", this, 4,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return strstr(pcValue, SOAPACTION_METHODNAME_SEPARATOR) +
                          strlen(SOAPACTION_METHODNAME_SEPARATOR);
		#endif
	}

                }
                else
                {
                    	{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (pcValue);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Apache2Transport", "getTransportProperty", this, 5,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return pcValue;
		#endif
	}

                }
            }
        case SOAP_MESSAGE_LENGTH:
				{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (getTransportProperty("Content-Length"));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Apache2Transport", "getTransportProperty", this, 6,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return getTransportProperty("Content-Length");
		#endif
	}

	    /* this is apache module and transport is http so the key */
        default:;
    }
    	{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Apache2Transport", "getTransportProperty", this, 7,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

}

int Apache2Transport::setTransportProperty(const char* pcKey, const char* pcValue)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Apache2Transport", "setTransportProperty", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pcKey),
					TRACETYPE_STRING, 0, ((void*)&pcValue));	  /* AUTOINSERTED TRACE */
	#endif

	/*ap_table_set(((request_rec*)m_pContext)->headers_out, pcKey, pcValue);*/
	//ap_send_http_header((request_rec*)m_pContext);
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (0);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Apache2Transport", "setTransportProperty", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return 0;
		#endif
	}

}

const char* Apache2Transport::getTransportProperty(const char* pcKey, bool response)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Apache2Transport", "getTransportProperty", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pcKey),
					TRACETYPE_BOOL, 0, ((void*)&response));	  /* AUTOINSERTED TRACE */
	#endif

    /* ap_table_elts returns an array_header struct. The nelts element of that 
     * struct contains the number of input header elements. Finally assigns that
     * to the axis soap data structure. */
    const apr_array_header_t* arr;
	int headercount = ap_table_elts(((request_rec*)m_pContext)->headers_in)->nelts;
	apr_table_entry_t* pHeaders = 0;
    /* casting req_rec->headers_in to axis header struct and assigning that to 
     * the axis soap structure. Hope this is ok 
     */
    /* obtain the array_header from the headers_in table and assign it to the 
     * axis soap structure 
     */
    arr = ap_table_elts(((request_rec*)m_pContext)->headers_in);
	pHeaders = (apr_table_entry_t*)arr->elts;
	for (int ix=0; ix<headercount; ix++)
	{
		if (!strcmp(pHeaders->key, pcKey))
				{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (pHeaders->val);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Apache2Transport", "getTransportProperty", this, 1,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return pHeaders->val;
		#endif
	}

		pHeaders++;
	}
		{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (0);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Apache2Transport", "getTransportProperty", this, 2,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return 0;
		#endif
	}

}

void Apache2Transport::setSessionId(const char* pcSessionId)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Apache2Transport", "setSessionId", this, 1,
					TRACETYPE_STRING, 0, ((void*)&pcSessionId));	  /* AUTOINSERTED TRACE */
	#endif


	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Apache2Transport", "setSessionId", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

const char* Apache2Transport::getSessionId()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Apache2Transport", "getSessionId", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

		{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = ("this is temporary session id");
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Apache2Transport", "getSessionId", this, 0,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return "this is temporary session id";
		#endif
	}
 //TODO
}

const char* Apache2Transport::getServiceName()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Apache2Transport", "getServiceName", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

		{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (0);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Apache2Transport", "getServiceName", this, 0,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return 0;
		#endif
	}
 //TODO
}

AXIS_PROTOCOL_TYPE Apache2Transport::getProtocol()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Apache2Transport", "getProtocol", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

		{
		#ifdef ENABLE_AXISTRACE
			AXIS_PROTOCOL_TYPE traceRet = (m_eProtocolType);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Apache2Transport", "getProtocol", this, 0,
					TRACETYPE_DATA, sizeof(AXIS_PROTOCOL_TYPE), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_eProtocolType;
		#endif
	}

}
 
int Apache2Transport::setProtocol(AXIS_PROTOCOL_TYPE eProtocol)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Apache2Transport", "setProtocol", this, 1,
					TRACETYPE_DATA, sizeof(AXIS_PROTOCOL_TYPE), ((void*)&eProtocol));	  /* AUTOINSERTED TRACE */
	#endif

    if( eProtocol == APTHTTP1_1 || eProtocol == APTHTTP1_0 )
    {
       m_eProtocolType = eProtocol;
       	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Apache2Transport", "setProtocol", this, 1,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}

    }
    else
        	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_FAIL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Apache2Transport", "setProtocol", this, 2,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_FAIL;
		#endif
	}

}

int Apache2Transport::getSubProtocol()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Apache2Transport", "getSubProtocol", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    /*Determine the http method and assign it to the axis soap structure */
    switch (((request_rec*)m_pContext)->method_number)
    {
        case M_GET:
            	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_HTTP_GET);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Apache2Transport", "getSubProtocol", this, 1,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_HTTP_GET;
		#endif
	}

        case M_POST:
            	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_HTTP_POST);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Apache2Transport", "getSubProtocol", this, 2,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_HTTP_POST;
		#endif
	}

        default:
            	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_HTTP_UNSUPPORTED);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Apache2Transport", "getSubProtocol", this, 3,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_HTTP_UNSUPPORTED;
		#endif
	}

    }
}

ISoapAttachment*  Apache2Transport::getAttachment(const char* pcAttachmentid)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Apache2Transport", "getAttachment", this, 1,
					TRACETYPE_STRING, 0, ((void*)&pcAttachmentid));	  /* AUTOINSERTED TRACE */
	#endif
		
	ISoapAttachment* pAttch = pAttachmentHelper->getAttachment(pcAttachmentid);
		{
		#ifdef ENABLE_AXISTRACE
			ISoapAttachment* traceRet = (pAttch);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Apache2Transport", "getAttachment", this, 0,
					TRACETYPE_POINTER, sizeof(ISoapAttachment), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return pAttch;
		#endif
	}

//	return NULL;
};

char* Apache2Transport::getIncomingSOAPMimeHeaders()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Apache2Transport", "getIncomingSOAPMimeHeaders", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

		{
		#ifdef ENABLE_AXISTRACE
			char* traceRet = (pAttachmentHelper->getIncomingSOAPMimeHeaders());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Apache2Transport", "getIncomingSOAPMimeHeaders", this, 0,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return pAttachmentHelper->getIncomingSOAPMimeHeaders();
		#endif
	}

//	return NULL;
}

ISoapAttachment**  Apache2Transport::getAllAttachments(int *pAttchArraySize)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("Apache2Transport", "getAllAttachments", this, 1,
					TRACETYPE_POINTER, sizeof(int), ((void*)&pAttchArraySize));	  /* AUTOINSERTED TRACE */
	#endif
		
	ISoapAttachment** pAttachments = pAttachmentHelper->getAllAttachments(pAttchArraySize);
		{
		#ifdef ENABLE_AXISTRACE
			ISoapAttachment** traceRet = (pAttachments);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("Apache2Transport", "getAllAttachments", this, 0,
					TRACETYPE_POINTER, sizeof(ISoapAttachment*), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return pAttachments;
		#endif
	}

//	return NULL;
};

