#ifdef ENABLE_AXISTRACE
/* TRACE ADDED BY THE TRACE INSTRUMENTOR TOOL */
#include "../../common/AxisTrace.h"
#endif
/*
 *   Copyright 2003-2004 The Apache Software Foundation.
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 *
 *
 * @author Samisa Abeysinghe (sabeysinghe@virtusa.com)
 *
 */

#include "SimpleAxisTransport.h"

#define AXIS_URI_EXTENSION "/axis"

/* Following is the character that should be used to separate the method name in
 * the SOAPAction header value. Ex: "http://localhost:80/axis/Calculator#Add"
 */

#define SOAPACTION_METHODNAME_SEPARATOR "#"

SimpleAxisTransport::SimpleAxisTransport( int iSocket)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SimpleAxisTransport", "SimpleAxisTransport", this, 1,
					TRACETYPE_INT, 0, ((void*)&iSocket));	  /* AUTOINSERTED TRACE */
	#endif

    m_eProtocolType = APTHTTP1_1;
    m_iSocket = iSocket;
    m_bReadPastHeaders = false;
    m_strPayloadToSend = "";
    m_strServiceURI = "base";
    m_strReceived = "";
    m_strSOAPAction = "";
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SimpleAxisTransport", "SimpleAxisTransport", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

SimpleAxisTransport::~SimpleAxisTransport()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SimpleAxisTransport", "~SimpleAxisTransport", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

#ifdef WIN32
    shutdown( m_iSocket, SD_BOTH);
	closesocket( m_iSocket);
#else //Linux
    shutdown( m_iSocket, SHUT_RDWR);	
    close( m_iSocket);
#endif
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SimpleAxisTransport", "~SimpleAxisTransport", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

AXIS_TRANSPORT_STATUS SimpleAxisTransport::sendBytes( const char *pcSendBuffer, const void *pBufferId)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SimpleAxisTransport", "sendBytes", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pcSendBuffer),
					TRACETYPE_POINTER, 1, ((void*)&pBufferId));	  /* AUTOINSERTED TRACE */
	#endif

    m_strPayloadToSend += pcSendBuffer;
    	{
		#ifdef ENABLE_AXISTRACE
			AXIS_TRANSPORT_STATUS traceRet = (TRANSPORT_FINISHED);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SimpleAxisTransport", "sendBytes", this, 0,
					TRACETYPE_DATA, sizeof(AXIS_TRANSPORT_STATUS), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return TRANSPORT_FINISHED;
		#endif
	}

/*
    if (!pBufferId)		// temporary buffers should always sent immediately 
    {
	if ((send (m_iSocket, pcSendBuffer, (int) strlen( pcSendBuffer), MSG_DONTROUTE)) == -1)
	    return TRANSPORT_FAILED;
	return TRANSPORT_FINISHED;
    }
    for (int index = 0; index < NO_OF_SERIALIZE_BUFFERS; index++)
    {
	if (!m_pBuffers[index].pcBuffer)
	{
	    m_pBuffers[index].pcBuffer = pcSendBuffer;
	    m_pBuffers[index].pBufferId = pBufferId;
	    break;
	}
    }
    return TRANSPORT_IN_PROGRESS;
*/
}

int SimpleAxisTransport::setTransportProperty( AXIS_TRANSPORT_INFORMATION_TYPE type, const char * value)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SimpleAxisTransport", "setTransportProperty", this, 2,
					TRACETYPE_DATA, sizeof(AXIS_TRANSPORT_INFORMATION_TYPE), ((void*)&type),
					TRACETYPE_STRING, 0, ((void*)&value));	  /* AUTOINSERTED TRACE */
	#endif

	const char *	key = NULL;
	int				iSuccess = AXIS_FAIL;
	switch( type)
	{
		case SOAPACTION_HEADER:	/* needed only in the client side ? */
			break;
	
		case SERVICE_URI:		/* need to set ? */
			break;
		case OPERATION_NAME:	/* need to set ? */
			break;
		case SOAP_MESSAGE_LENGTH: /* This is apache module and transport is http so the key */
			key = "Content-Length";
			break;
		default:
			;
	}
	if( key)
	{
		iSuccess = setTransportProperty( key, value);
	}
		{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SimpleAxisTransport", "setTransportProperty", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}

}

AXIS_TRANSPORT_STATUS SimpleAxisTransport::flushOutput()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SimpleAxisTransport", "flushOutput", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    int		contentLength = (int) m_strPayloadToSend.length();
    char	strtonum[8];
	if( contentLength != 0)	// do only if the http body is not empty. 
    {
		sprintf( strtonum, "%d", contentLength); // This assumes that the content length will be less that 8 digits (9,999,999 bytes).
		setTransportProperty( SOAP_MESSAGE_LENGTH, strtonum);
		m_strHTTPHeaders = std::string( "HTTP/1.1 200 OK\r\n") +
						   std::string( "Server: Apache Axis C++/1.6.a\r\n") +
						   std::string( "Connection: close\r\n") +
						   m_strHTTPHeaders +
						   std::string( "Content-Type: text/xml\r\n\r\n");
		if( (send( m_iSocket,
				   m_strHTTPHeaders.c_str(),
				   (int) m_strHTTPHeaders.size(),
				   MSG_DONTROUTE)) == -1)
		{
				{
		#ifdef ENABLE_AXISTRACE
			AXIS_TRANSPORT_STATUS traceRet = (TRANSPORT_FAILED);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SimpleAxisTransport", "flushOutput", this, 1,
					TRACETYPE_DATA, sizeof(AXIS_TRANSPORT_STATUS), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return TRANSPORT_FAILED;
		#endif
	}

		}
        if( (send( m_iSocket,
				   m_strPayloadToSend.c_str(),
				   (int) m_strPayloadToSend.length(),
				   MSG_DONTROUTE)) == -1)
		{
				{
		#ifdef ENABLE_AXISTRACE
			AXIS_TRANSPORT_STATUS traceRet = (TRANSPORT_FAILED);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SimpleAxisTransport", "flushOutput", this, 2,
					TRACETYPE_DATA, sizeof(AXIS_TRANSPORT_STATUS), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return TRANSPORT_FAILED;
		#endif
	}

		}
    }
		{
		#ifdef ENABLE_AXISTRACE
			AXIS_TRANSPORT_STATUS traceRet = (TRANSPORT_FINISHED);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SimpleAxisTransport", "flushOutput", this, 3,
					TRACETYPE_DATA, sizeof(AXIS_TRANSPORT_STATUS), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return TRANSPORT_FINISHED;
		#endif
	}

}

AXIS_TRANSPORT_STATUS SimpleAxisTransport::getBytes( char * pBuffer, int * piSize)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SimpleAxisTransport", "getBytes", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pBuffer),
					TRACETYPE_POINTER, sizeof(int), ((void*)&piSize));	  /* AUTOINSERTED TRACE */
	#endif

    if( m_strReceived.length () <= 0)
    {
		pBuffer[0] = '\0';
		*piSize = 0;
			{
		#ifdef ENABLE_AXISTRACE
			AXIS_TRANSPORT_STATUS traceRet = (TRANSPORT_FINISHED);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SimpleAxisTransport", "getBytes", this, 1,
					TRACETYPE_DATA, sizeof(AXIS_TRANSPORT_STATUS), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return TRANSPORT_FINISHED;
		#endif
	}

    }
    if( (int) m_strReceived.length() < *piSize)
    {
		pBuffer[0] = '\0';
		strcat( pBuffer, m_strReceived.c_str());
		*piSize = (int) m_strReceived.length();
		m_strReceived = "";
			{
		#ifdef ENABLE_AXISTRACE
			AXIS_TRANSPORT_STATUS traceRet = (TRANSPORT_FINISHED);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SimpleAxisTransport", "getBytes", this, 2,
					TRACETYPE_DATA, sizeof(AXIS_TRANSPORT_STATUS), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return TRANSPORT_FINISHED;
		#endif
	}

    }
    else
    {
		pBuffer[0] = '\0';
		std::string strToSend = m_strReceived.substr( *piSize - 1);
		strcat( pBuffer, strToSend.c_str());
		*piSize = (int) strToSend.length();
		m_strReceived = m_strReceived.substr( (*piSize),
											  m_strReceived.length() - *piSize);
			{
		#ifdef ENABLE_AXISTRACE
			AXIS_TRANSPORT_STATUS traceRet = (TRANSPORT_IN_PROGRESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SimpleAxisTransport", "getBytes", this, 3,
					TRACETYPE_DATA, sizeof(AXIS_TRANSPORT_STATUS), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return TRANSPORT_IN_PROGRESS;
		#endif
	}

    }
}

void SimpleAxisTransport::readFromClient()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SimpleAxisTransport", "readFromClient", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    int					len_read = 0;
    const unsigned int	BUF_SIZE = 48000;
    char				arr[BUF_SIZE];
    unsigned int		iStartPos = (int) std::string::npos;
    std::string strReceived = "";
    do
    {
		len_read = recv( m_iSocket, arr, BUF_SIZE - 1, 0);
		if( len_read < 0)
		{
				{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SimpleAxisTransport", "readFromClient", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}

		}
		arr[len_read] = '\0';
		strReceived += arr;
		iStartPos = (int) strReceived.find( "\r\n\r\n");
	} while( iStartPos == std::string::npos);
    iStartPos += 4;
	unsigned int uiServiceStartPos = (unsigned int) (strReceived.find( "POST ") + strlen( "POST "));
    unsigned int uiServiceEndPos = (unsigned int) strReceived.find( " ", uiServiceStartPos);
    m_strServiceURI = strReceived.substr( uiServiceStartPos, uiServiceEndPos - uiServiceStartPos);
	unsigned int uiActionStartPos =(unsigned int) (strReceived.find( "SOAPAction: \"") + strlen( "SOAPAction: \""));
    unsigned int uiActionEndPos = (unsigned int) strReceived.find( "\"", uiActionStartPos);
	m_strSOAPAction = strReceived.substr (uiActionStartPos, uiActionEndPos - uiActionStartPos);
    unsigned int uiEnvelopePos = (unsigned int) strReceived.find( "Envelope");
	// Find the beginning of the SOAP message by finding the opening 'Envelope' tag.
	while( uiEnvelopePos == std::string::npos)
	{
		len_read = recv (m_iSocket, arr, BUF_SIZE - 1, 0);
		arr[len_read] = '\0';
		strReceived += arr;
		uiEnvelopePos = (unsigned int) strReceived.find( "Envelope");
	}
    uiEnvelopePos += (unsigned int) strlen( "Envelope");
    unsigned int uiEnvelopeEndPos = (unsigned int) strReceived.find( "Envelope", uiEnvelopePos);
	// Find the end of the SOAP message by finding the matching closing 'Envelope' tag.
    while( uiEnvelopeEndPos == std::string::npos)
    {
		len_read = recv( m_iSocket, arr, BUF_SIZE - 1, 0);
		arr[len_read] = '\0';
		strReceived += arr;
		uiEnvelopeEndPos = (unsigned int) strReceived.find( "Envelope", uiEnvelopePos);
    }
    m_strReceived = strReceived.substr( iStartPos, strReceived.length () - iStartPos);
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SimpleAxisTransport", "readFromClient", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

const char * SimpleAxisTransport::getTransportProperty( AXIS_TRANSPORT_INFORMATION_TYPE eType)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SimpleAxisTransport", "getTransportProperty", this, 1,
					TRACETYPE_DATA, sizeof(AXIS_TRANSPORT_INFORMATION_TYPE), ((void*)&eType));	  /* AUTOINSERTED TRACE */
	#endif

	switch( eType)
	{
		case SOAPACTION_HEADER:
		{
				{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (getTransportProperty( "SOAPAction"));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SimpleAxisTransport", "getTransportProperty", this, 1,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return getTransportProperty( "SOAPAction");
		#endif
	}

		}
		case SERVICE_URI:
		{
			readFromClient();
			if( m_strServiceURI.find( AXIS_URI_EXTENSION) != std::string::npos)
			{
				unsigned int uiStartPos = (unsigned int) (m_strServiceURI.find( AXIS_URI_EXTENSION) +
														  strlen( AXIS_URI_EXTENSION) +
														  1);
				m_strTransportPropertyURI = m_strServiceURI.substr( uiStartPos,
																	m_strServiceURI.length() - uiStartPos);
					{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (m_strTransportPropertyURI.c_str());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SimpleAxisTransport", "getTransportProperty", this, 2,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_strTransportPropertyURI.c_str();
		#endif
	}

			}
				{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (m_strServiceURI.c_str());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SimpleAxisTransport", "getTransportProperty", this, 3,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_strServiceURI.c_str();
		#endif
	}

		}
		case OPERATION_NAME:
		{
			unsigned int uiOpStart = (unsigned int) m_strSOAPAction.find( SOAPACTION_METHODNAME_SEPARATOR);
			if( uiOpStart != std::string::npos)
			{
				uiOpStart += (unsigned int) strlen( SOAPACTION_METHODNAME_SEPARATOR);
				m_strTransportPropertyOperation = m_strSOAPAction.substr( uiOpStart,
																		  m_strSOAPAction.length() - uiOpStart);
					{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (m_strTransportPropertyOperation.c_str());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SimpleAxisTransport", "getTransportProperty", this, 4,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_strTransportPropertyOperation.c_str();
		#endif
	}

			}
				{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (m_strSOAPAction.c_str());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SimpleAxisTransport", "getTransportProperty", this, 5,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_strSOAPAction.c_str();
		#endif
	}

		}
		case SOAP_MESSAGE_LENGTH:
		{
				{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (getTransportProperty( "Content-Length"));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SimpleAxisTransport", "getTransportProperty", this, 6,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return getTransportProperty( "Content-Length");
		#endif
	}

		}
		default:
		{
			;
		}
	}
    
		{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SimpleAxisTransport", "getTransportProperty", this, 7,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

}

int SimpleAxisTransport::setTransportProperty( const char * pcKey, const char * pcValue)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SimpleAxisTransport", "setTransportProperty", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pcKey),
					TRACETYPE_STRING, 0, ((void*)&pcValue));	  /* AUTOINSERTED TRACE */
	#endif

    m_strHTTPHeaders += std::string( pcKey) +
						std::string( ": ") +
						std::string( pcValue) +
						std::string( "\r\n");
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (0);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SimpleAxisTransport", "setTransportProperty", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return 0;
		#endif
	}

}

const char * SimpleAxisTransport::getTransportProperty( const char * pcKey, bool response)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SimpleAxisTransport", "getTransportProperty", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pcKey),
					TRACETYPE_BOOL, 0, ((void*)&response));	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (0);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SimpleAxisTransport", "getTransportProperty", this, 0,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return 0;
		#endif
	}

}

void SimpleAxisTransport::setSessionId( const char * pcSessionId)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SimpleAxisTransport", "setSessionId", this, 1,
					TRACETYPE_STRING, 0, ((void*)&pcSessionId));	  /* AUTOINSERTED TRACE */
	#endif


	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SimpleAxisTransport", "setSessionId", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

const char * SimpleAxisTransport::getSessionId()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SimpleAxisTransport", "getSessionId", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = ("this is temporary session id");
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SimpleAxisTransport", "getSessionId", this, 0,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return "this is temporary session id";
		#endif
	}
	//TODO
}

const char * SimpleAxisTransport::getServiceName()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SimpleAxisTransport", "getServiceName", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (0);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SimpleAxisTransport", "getServiceName", this, 0,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return 0;
		#endif
	}
			//TODO
}

AXIS_PROTOCOL_TYPE SimpleAxisTransport::getProtocol()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SimpleAxisTransport", "getProtocol", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			AXIS_PROTOCOL_TYPE traceRet = (APTHTTP1_1);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SimpleAxisTransport", "getProtocol", this, 0,
					TRACETYPE_DATA, sizeof(AXIS_PROTOCOL_TYPE), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return APTHTTP1_1;
		#endif
	}

}

int SimpleAxisTransport::setProtocol(AXIS_PROTOCOL_TYPE eProtocol)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SimpleAxisTransport", "setProtocol", this, 1,
					TRACETYPE_DATA, sizeof(AXIS_PROTOCOL_TYPE), ((void*)&eProtocol));	  /* AUTOINSERTED TRACE */
	#endif

    if( eProtocol == APTHTTP1_1 || eProtocol == APTHTTP1_0)
    {
       m_eProtocolType = eProtocol;
       	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SimpleAxisTransport", "setProtocol", this, 1,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}

    }
		{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_FAIL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SimpleAxisTransport", "setProtocol", this, 2,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_FAIL;
		#endif
	}

}

int SimpleAxisTransport::getSubProtocol()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SimpleAxisTransport", "getSubProtocol", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_HTTP_POST);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SimpleAxisTransport", "getSubProtocol", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_HTTP_POST;
		#endif
	}

}

