#ifdef ENABLE_AXISTRACE
/* TRACE ADDED BY THE TRACE INSTRUMENTOR TOOL */
#include "../common/AxisTrace.h"
#endif
/*
 *   Copyright 2003-2004 The Apache Software Foundation.
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 *
 * @author Rangika Mendis (rangika@opensource.lk)
 * @author Nithyakala Thangarajah (nithya@opensource.lk)
 * @author Roshan Weerasuriya (roshan@opensource.lk, roshan@jkcsworld.com)
 */

// SoapAttachment.cpp: implementation of the SoapAttachment class.

#include "SoapAttachment.hpp"

#include "SoapSerializer.h"

AXIS_CPP_NAMESPACE_START

//////////////////////////////////////////////////////////////////////

// Construction/Destruction

//////////////////////////////////////////////////////////////////////

SoapAttachment::SoapAttachment(ContentIdSet *pContentIdSet)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapAttachment", "SoapAttachment", this, 1,
					TRACETYPE_POINTER, sizeof(ContentIdSet), ((void*)&pContentIdSet));	  /* AUTOINSERTED TRACE */
	#endif
	
	m_AttachmentHeaders = new SoapAttachmentHeaders();
	m_AttachmentBody = 0;
	m_pContentIdSet = pContentIdSet;
	if (NULL!=m_pContentIdSet)
		m_AttachmentHeaders->addHeader(AXIS_CONTENT_ID, pContentIdSet->generateId());
	// Set up some sensible default values for mandatory headers.
	m_AttachmentHeaders->addHeader(AXIS_CONTENT_TYPE,"text/plain");
	m_AttachmentHeaders->addHeader(AXIS_CONTENT_TRANSFER_ENCODING,"base64");
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapAttachment", "SoapAttachment", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

SoapAttachment::~SoapAttachment()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapAttachment", "~SoapAttachment", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

	delete m_AttachmentHeaders;
	m_AttachmentHeaders =0;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapAttachment", "~SoapAttachment", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void SoapAttachment::addHeader(const char* pchName, const char* pchValue)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapAttachment", "addHeader", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pchName),
					TRACETYPE_STRING, 0, ((void*)&pchValue));	  /* AUTOINSERTED TRACE */
	#endif

	// Registering the contentid here ensures that it is unique within the mime message.
	if (NULL!=m_pContentIdSet && 0==strcmp(AXIS_CONTENT_ID,pchName)) 
		m_pContentIdSet->registerId(pchValue);
	m_AttachmentHeaders->addHeader(pchName, pchValue);
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapAttachment", "addHeader", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void SoapAttachment::addBody(xsd__base64Binary* objBody)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapAttachment", "addBody", this, 1,
					TRACETYPE_POINTER, sizeof(xsd__base64Binary), ((void*)&objBody));	  /* AUTOINSERTED TRACE */
	#endif

    iEncodingStyle = AXIS_BASE64;
	m_AttachmentBody = objBody;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapAttachment", "addBody", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void SoapAttachment::addBody(char* pchBinaryBody)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapAttachment", "addBody", this, 1,
					TRACETYPE_STRING, 0, ((void*)&pchBinaryBody));	  /* AUTOINSERTED TRACE */
	#endif
 
    iEncodingStyle = AXIS_BINARY;
    m_binaryBody = new char[strlen(pchBinaryBody) + 1];
    strcpy(m_binaryBody,pchBinaryBody);
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapAttachment", "addBody", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void SoapAttachment::serialize(SoapSerializer &pSZ)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapAttachment", "serialize", this, 1,
					TRACETYPE_DATA, sizeof(SoapSerializer&), ((void*)&pSZ));	  /* AUTOINSERTED TRACE */
	#endif

	/* Serialize the Attachment Headers */
	pSZ.serialize("\r\n", NULL);
	m_AttachmentHeaders->serialize(pSZ);
	/* Serialize the Attachment Body */
    if (iEncodingStyle == AXIS_BASE64)
    {
	    if (m_AttachmentBody) {
		    pSZ.serialize("\r\n", NULL);
		    pSZ.serializeAsChardata(m_AttachmentBody, XSD_BASE64BINARY);
	    }
    } 
    else if (iEncodingStyle == AXIS_BINARY)
    {
        pSZ.serialize("\r\n", NULL);
        
        
        string attachBody = m_binaryBody;
       
        long boundary = attachBody.find("------=_Part_0_6349096.11054186923",0);
        if (boundary != -1)
        {
            string attachment = attachBody.substr(0,(boundary-1));
            char* binary = new char[attachment.length() + 1];
            strcpy(binary, attachment.c_str());        
             m_binaryBody = binary;            
            pSZ.serialize(m_binaryBody, NULL);
        }
        else              
        pSZ.serialize(m_binaryBody, NULL);
    }
	pSZ.serialize("\r\n", NULL);
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapAttachment", "serialize", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

xsd__base64Binary* SoapAttachment::getBody()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapAttachment", "getBody", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

		{
		#ifdef ENABLE_AXISTRACE
			xsd__base64Binary* traceRet = (m_AttachmentBody);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapAttachment", "getBody", this, 0,
					TRACETYPE_POINTER, sizeof(xsd__base64Binary), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_AttachmentBody;
		#endif
	}

}

const char* SoapAttachment::getHeader(const char *pchName)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapAttachment", "getHeader", this, 1,
					TRACETYPE_STRING, 0, ((void*)&pchName));	  /* AUTOINSERTED TRACE */
	#endif

		{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (m_AttachmentHeaders->getHeader(pchName));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapAttachment", "getHeader", this, 0,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_AttachmentHeaders->getHeader(pchName);
		#endif
	}

}

const char* SoapAttachment::getAttachmentId()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("SoapAttachment", "getAttachmentId", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

   	{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (getHeader(AXIS_CONTENT_ID));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("SoapAttachment", "getAttachmentId", this, 0,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return getHeader(AXIS_CONTENT_ID);
		#endif
	}

}

AXIS_CPP_NAMESPACE_END

