#ifdef ENABLE_AXISTRACE
/* TRACE ADDED BY THE TRACE INSTRUMENTOR TOOL */
#include "../../common/AxisTrace.h"
#endif
/*
*   Copyright 2003-2004 The Apache Software Foundation.
// (c) Copyright IBM Corp. 2004, 2005 All Rights Reserved
*
*   Licensed under the Apache License, Version 2.0 (the "License");
*   you may not use this file except in compliance with the License.
*   You may obtain a copy of the License at
*
*       http://www.apache.org/licenses/LICENSE-2.0
*
*   Unless required by applicable law or agreed to in writing, software
*   distributed under the License is distributed on an "AS IS" BASIS,
*   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*   See the License for the specific language governing permissions and
*   limitations under the License.
*/

/*
 * @author Samisa Abeysinghe (sabeysinghe@virtusa.com)
 *
 */

#ifdef WIN32

#pragma warning (disable : 4786)

#pragma warning (disable : 4101)

#endif

#include "HTTPTransport.hpp"

#include "../../platforms/PlatformAutoSense.hpp"

#include <stdio.h>

// You can uncommment thisl line if you want any debug putting to stdio but PLEASE ensure you comment it out again before releasing.

// #include <iostream>

// =================================================================

// In order to parse the HTTP protocol data on an ebcdic system, we

// need to ensure that the various tokens we are looking for to distinguish

// between the HTTP headers and payload are in ASCII.  So the following 

// defines for carriage return-line feed, etc. are ensured to be in ascii

// by using ascii hexadecimal representation of the tokens.

// =================================================================

// Ascii character defines                   

#define ASCII_C_EQUAL         '\x3D'    // '='  

#define ASCII_C_DASH          '\x2D'    // '-'  

#define ASCII_C_LF            '\x0A'    // '\n'

#define ASCII_C_CR            '\x0D'    // '\r' 

#define ASCII_C_SPACE         '\x20'    // ' ' 

#define ASCII_C_SEMI          '\x3B'    // ';' 

#define ASCII_C_LOWERCASEA    '\x61'    // 'a'

#define ASCII_C_LOWERCASEF    '\x66'    // 'f'

#define ASCII_C_UPPERCASEA    '\x41'    // 'A'

#define ASCII_C_UPPERCASEF    '\x46'    // 'F'

#define ASCII_C_ZERO          '\x30'    // '0'

#define ASCII_C_NINE          '\x39'    // '9'

// Ascii string defines

#define ASCII_S_LF             "\x0a"                 // "\n"     

#define ASCII_S_CRLF           "\x0d\x0a"             // "\r\n"     

#define ASCII_S_CRLFCRLF       "\x0d\x0a\x0d\x0a"     // "\r\n\r\n"  

#define ASCII_S_HTTP           "\x48\x54\x54\x50"     // "HTTP"

#define ASCII_S_CONTENT_LENGTH "\x43\x6F\x6E\x74\x65\x6E\x74\x2D\x4c\x65\x6e\x67\x74\x68\x3a\x20" // "Content-Length: " 

#define ASCII_S_TRANSFERENCODING_CHUNKED "\x54\x72\x61\x6E\x73\x66\x65\x72\x2D\x45\x6E\x63\x6F\x64\x69\x6E\x67\x3A\x20\x63\x68\x75\x6e\x6b\x65\x64" // "Transfer-Encoding: chunked"

#define ASCII_S_LEFTPAREN      "\x28"   // "("

#define ASCII_S_COLON          "\x3A"   // ":" 

/*
 * HTTPTransport constuctor
 */

HTTPTransport::HTTPTransport ():m_bReopenConnection (false),
m_strHTTPProtocol ("HTTP/1.1"),
m_strHTTPMethod ("POST"),
m_strProxyHost (""), m_uiProxyPort (0), m_bUseProxy (false),
m_bMaintainSession (false)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HTTPTransport", "HTTPTransport", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    m_pcEndpointUri = NULL;
    m_pReleaseBufferCallback = 0;
    m_eProtocolType = APTHTTP1_1;
    m_strBytesToSend = "";
    m_strHeaderBytesToSend = "";
    m_iBytesLeft = 0;
    m_iContentLength = 0;
	m_bChannelSecure = false;
    m_pNormalChannel = 0;
    m_pSecureChannel = 0;
	m_pActiveChannel = 0;
    m_pChannelFactory = new ChannelFactory();
    m_bMimeTrue = false;
    m_viCurrentHeader = m_vHTTPHeaders.begin();
    m_viCurrentResponseHeader = m_vResponseHTTPHeaders.begin();
	m_pszRxBuffer = new char [BUF_SIZE];
#ifdef WIN32
	m_lChannelTimeout = 10;
#else
	m_lChannelTimeout = 0;
#endif
	m_pNormalChannel = m_pChannelFactory->createChannel(UnsecureChannel);
	m_pSecureChannel = m_pChannelFactory->createChannel(SecureChannel);
	m_GetBytesState = eWaitingForHTTPHeader;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "HTTPTransport", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

/*
 * HTTPTransport destructor
 */

HTTPTransport::~HTTPTransport()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HTTPTransport", "~HTTPTransport", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    if( m_pcEndpointUri)
    {
		delete[] m_pcEndpointUri;
		m_pcEndpointUri = NULL;
    }
    if( m_pChannelFactory)
	{
		delete m_pChannelFactory;
		m_pChannelFactory = NULL;
	}
	if( m_pszRxBuffer)
	{
		delete [] m_pszRxBuffer;
		m_pszRxBuffer = NULL;
	}
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "~HTTPTransport", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

/*
 * HTTPTransport::setEndpointUri( EndpointURI) sets the URI for the message.
 * Everytime the endpoint changes then currently connected channel is closed
 * and a new channel connection is opened.
 *
 * @param	EndpointURI - char * to a null terminated string that holds the
 *			new URI. 
 */

void HTTPTransport::setEndpointUri( const char * pcEndpointUri) throw (HTTPTransportException)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HTTPTransport", "setEndpointUri", this, 1,
					TRACETYPE_STRING, 0, ((void*)&pcEndpointUri));	  /* AUTOINSERTED TRACE */
	#endif

    bool bUpdateURL = false;
    if (!pcEndpointUri) // We have NULL so cannot proceed
        	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "setEndpointUri", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}

    // Samisa:We have to have at lest "http://" in the URL
    // We also need to handle passing " " as URL which 
    // is done by some auto generated code
    if ( strlen(pcEndpointUri) < strlen("http://") ) 
        	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "setEndpointUri", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}

                                                  
    // Get the current channel URI
    if( m_pActiveChannel != NULL && m_pActiveChannel->getURL())
    {
		// Does the new URI equal the existing channel URI?
		if( strcmp (m_pActiveChannel->getURL (), pcEndpointUri) != 0)
		{
	    	// There is a new URI.
			bUpdateURL = true;
		}
    }
    else
    {
		bUpdateURL = true;
    }
    // If there is a new URI, then this flag will be set.  Depending on whether
    // there is an SSL implementation available, if the new URI is a secure
	// connection, a secure channel will be opened.  If an SSL implementation
	// is not available and the URL requires a secure connection then an
	// exception will be thrown.
    if( bUpdateURL)
    {
		if( m_pActiveChannel == NULL)
		{
			m_pActiveChannel = m_pNormalChannel;
		}
		m_pActiveChannel->setURL( pcEndpointUri);
		m_bReopenConnection = true;
		// Check if the new URI requires SSL (denoted by the https prefix).
		if( (m_pActiveChannel->getURLObject()).getProtocol() == URL::https)
		{
			if( m_pSecureChannel != NULL)
			{
				m_pNormalChannel->close();
				m_pActiveChannel = m_pSecureChannel;
				m_pActiveChannel->setURL( pcEndpointUri);
				m_bChannelSecure = true;
			}
			if( !m_bChannelSecure)
			{
				throw HTTPTransportException( CLIENT_TRANSPORT_HAS_NO_SECURE_TRANSPORT_LAYER);
			}
		}
		else
		{
	    	// URI does not require a secure channel.  Delete the existing
	    	// channel if it is secure and create a new unsecure
	    	// channel.
			if (m_bChannelSecure)
			{
				if( m_pNormalChannel != NULL)
				{
					m_pSecureChannel->close();
					m_pActiveChannel = m_pNormalChannel;
					m_pActiveChannel->setURL( pcEndpointUri);
					m_bChannelSecure = false;
				}
				if( m_bChannelSecure)
				{
					throw HTTPTransportException( CLIENT_TRANSPORT_HAS_NO_UNSECURE_TRANSPORT_LAYER);
				}
			}
		}
    }
	// Need this code to set the channel timeout.  If the timeout was changed
	// before the channel was created, then it may not have the correct timeout.
	// By setting it here, the channel is sure to have the correct timeout value
	// next time the channel is read.
	if( m_pActiveChannel != NULL)
	{
		m_pActiveChannel->setTimeout( m_lChannelTimeout);
	}
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "setEndpointUri", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

/*
 * HTTPTransport::openConnection().
 */

int HTTPTransport::openConnection()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HTTPTransport", "openConnection", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    if( m_bReopenConnection)
    {
		m_bReopenConnection = false;
		if( m_pActiveChannel->open() != AXIS_SUCCESS)
		{
		    int	iStringLength = m_pActiveChannel->GetLastErrorMsg().length() + 1;
			const char * pszLastError = new char[iStringLength];
		    memcpy( (void *) pszLastError,
					m_pActiveChannel->GetLastErrorMsg().c_str(),
					iStringLength);
		    throw HTTPTransportException( CLIENT_TRANSPORT_OPEN_CONNECTION_FAILED,
										  (char *) pszLastError);
		}
	}
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "openConnection", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}

}

/*
 * HTTPTransport::closeConnection().
 */

void HTTPTransport::closeConnection()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HTTPTransport", "closeConnection", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    // get ready for a new message.
	m_GetBytesState = eWaitingForHTTPHeader;
    //clear the message buffer in preperation of the next read.
    m_strReceived = "";
    m_iBytesLeft = 0;
    m_iContentLength = 0;
    // Samisa : closing the connection is done in setEndpointUri
    // no need to close here
    // Fix for AXISCPP-481
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "closeConnection", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

/*
 * HTTPTransport::flushOutput() Is called when the message construction is
 * complete.  The message is ready to be 'flushed out' onto the network.  
 * Check if the URI has changed.  If it has, then need to open a new Channel
 * instance before transmitting the message.
 *
 * @return AXIS_TRANSPORT_STATUS If the method completes successfully, then
 * this will be set to TRANSPORT_FINISHED.  Otherwise, an exception will have
 * been thrown.
 */

AXIS_TRANSPORT_STATUS HTTPTransport::flushOutput() throw (AxisException, HTTPTransportException)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HTTPTransport", "flushOutput", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

	char *utf8Buf = NULL; // buffer for ebcdic/utf8 conversions.
	
    // In preperation for sending the message, calculate the size of the message
    // by using the string length method.
    char buff[24];
	sprintf( buff, "%d", m_strBytesToSend.length ());
    this->setTransportProperty ("Content-Length", buff);
    // The header is now complete.  The message header and message can now be
    // transmitted.
	try
	{
#ifndef __OS400__
		*m_pActiveChannel << this->getHTTPHeaders ();
		*m_pActiveChannel << this->m_strBytesToSend.c_str ();
#else
        // Ebcdic (OS/400) systems need to convert the data to UTF-8. Note that free() is 
        // correctly used and should not be changed to delete().		
        const char *buf = this->getHTTPHeaders ();
        utf8Buf = toUTF8((char *)buf, strlen(buf)+1);
		*m_pActiveChannel << utf8Buf;
        free(utf8Buf);
        utf8Buf = NULL;
        utf8Buf = toUTF8((char *)this->m_strBytesToSend.c_str(), this->m_strBytesToSend.length()+1);
		*m_pActiveChannel << utf8Buf;
        free(utf8Buf);
        utf8Buf = NULL;
#endif
	}
	catch( HTTPTransportException & e)
	{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceCatch("HTTPTransport", "flushOutput", this, 1,
					TRACETYPE_DATA, sizeof(HTTPTransportException&), ((void*)&e));	  /* AUTOINSERTED TRACE */
	#endif

		if (utf8Buf) free(utf8Buf);
        m_strBytesToSend = "";
        m_strHeaderBytesToSend = "";
		throw;
	}
	catch( AxisException & e)
	{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceCatch("HTTPTransport", "flushOutput", this, 2,
					TRACETYPE_AXISEXCEPTION, sizeof(AxisException&), ((void*)&e));	  /* AUTOINSERTED TRACE */
	#endif

		if (utf8Buf) free(utf8Buf);
        m_strBytesToSend = "";
        m_strHeaderBytesToSend = "";
		throw;
	}
	catch(...)
	{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceCatch("HTTPTransport", "flushOutput", this, 3 );	  /* AUTOINSERTED TRACE */
	#endif

		if (utf8Buf) free(utf8Buf);
        m_strBytesToSend = "";
        m_strHeaderBytesToSend = "";
		throw;
	}
    // Empty the bytes to send string.
	m_strBytesToSend = "";
	m_strHeaderBytesToSend = "";
 
    // Also empty the response headers as there aren't any yet until the response comes back !
    m_vResponseHTTPHeaders.clear();
    // TODO: Possible memory leak here - does the clear op clean out the memory too?
    
		{
		#ifdef ENABLE_AXISTRACE
			AXIS_TRANSPORT_STATUS traceRet = (TRANSPORT_FINISHED);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "flushOutput", this, 0,
					TRACETYPE_DATA, sizeof(AXIS_TRANSPORT_STATUS), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return TRANSPORT_FINISHED;
		#endif
	}

}

/* HTTPTransport::getHTTPHeaders() Called to retrieve the current HTTP header
 * information block that will preceed the SOAP message.
 *
 * @return const char* Pointer to a NULL terminated character string containing
 * the HTTP header block of information.
 */

const char * HTTPTransport::getHTTPHeaders()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HTTPTransport", "getHTTPHeaders", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    URL &           url = m_pActiveChannel->getURLObject(); 
    unsigned short	uiPort;
    char			buff[32];
    m_strHeaderBytesToSend = m_strHTTPMethod + " ";
    if (m_bUseProxy)
    {
        m_strHeaderBytesToSend += url.getURL ();
        m_strHeaderBytesToSend += " ";
        m_strHeaderBytesToSend += m_strHTTPProtocol;
        m_strHeaderBytesToSend += "\r\nHost: ";
        m_strHeaderBytesToSend += m_strProxyHost;
        uiPort = m_uiProxyPort;
    }
    else
    {
		m_strHeaderBytesToSend += url.getResource ();
        m_strHeaderBytesToSend += " ";
        m_strHeaderBytesToSend += m_strHTTPProtocol;
		m_strHeaderBytesToSend += "\r\nHost: ";
		m_strHeaderBytesToSend += url.getHostName ();
        uiPort = url.getPort();
    }  
    sprintf(buff, ":%u\r\n", uiPort);
    m_strHeaderBytesToSend += buff; 
	bool foundCT = false;
    for (unsigned int j = 0; j < m_vHTTPHeaders.size (); j++)
    {
		if (0==strcmp(AXIS_CONTENT_TYPE,m_vHTTPHeaders[j].first.c_str())) foundCT = true;
    }
	// The Content-Type must be set, but it may already be set in m_strHeaderBytesToSend
	// if we're using attachments, for example.
	if (!foundCT)
		m_strHeaderBytesToSend += AXIS_CONTENT_TYPE ": text/xml; charset=UTF-8\r\n";
    // Set other HTTP headers but not cookies as they are put in afterwards.
    for (unsigned int i = 0; i < m_vHTTPHeaders.size (); i++)
    {
        if( strcmp(m_vHTTPHeaders[i].first.c_str(), "Cookie")!=0)
        {
          m_strHeaderBytesToSend += m_vHTTPHeaders[i].first;
		  m_strHeaderBytesToSend += ": ";
		  m_strHeaderBytesToSend += m_vHTTPHeaders[i].second;
		  m_strHeaderBytesToSend += "\r\n";
        }
    }
    // Set cookies
    if (m_bMaintainSession && (m_vCookies.size () > 0))
    {
        string cookieHeader="";
        
        // Add in all the cookies ar the last one because that shouldn't have a ';' on it
        for (unsigned int var = 0; var < m_vCookies.size()-1; var++) 
        {
            cookieHeader += m_vCookies[var].first;
            cookieHeader += "=";
            cookieHeader += m_vCookies[var].second;
            cookieHeader += ";";
        }
        // add on the last cookie
        cookieHeader += m_vCookies[m_vCookies.size()-1].first;
        cookieHeader += "=";
        cookieHeader += m_vCookies[m_vCookies.size()-1].second;
        
        
        m_strHeaderBytesToSend += "Cookie: ";
        m_strHeaderBytesToSend += cookieHeader;
        m_strHeaderBytesToSend += "\r\n";
        
        // Now add this header in to the list of sent headers
        // if it's not already been set ! If it has been then override it
        bool b_keyFound=false;
        for (unsigned int i = 0; i < m_vHTTPHeaders.size(); i++)
        {
            if (m_vHTTPHeaders[i].first == (string)"Cookie")
            {
                m_vHTTPHeaders[i].second = (string) cookieHeader;
                b_keyFound = true;
                break;
            }
        }
        if(!b_keyFound)
        {
            m_vHTTPHeaders.push_back( std::make_pair( (string) "Cookie", (string) cookieHeader));
        }
    }
    m_strHeaderBytesToSend += "\r\n";
    	{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (m_strHeaderBytesToSend.c_str ());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "getHTTPHeaders", this, 0,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_strHeaderBytesToSend.c_str ();
		#endif
	}

}

/* HTTPTransport::getHTTPMethod() Is a public method that gets the HTTP method
 * (i.e. GET or POST) that will be part of the HTTP header block.
 *
 * @return const char* Pointer to a NULL terminated character string containing
 * the HTTP method.
 */

const char * HTTPTransport::getHTTPMethod()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HTTPTransport", "getHTTPMethod", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (m_strHTTPMethod.c_str ());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "getHTTPMethod", this, 0,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_strHTTPMethod.c_str ();
		#endif
	}

}

/* HTTPTransport::setHTTPMethod( Method) Is a public method that sets the HTTP
 * method (i.e. POST or GET) that will be part of the HTTP header block.
 *
 * @param const char* Pointer to a NULL terminated character string containing
 * the new HTTP method.
 */

void HTTPTransport::setHTTPMethod( const char *cpMethod)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HTTPTransport", "setHTTPMethod", this, 1,
					TRACETYPE_STRING, 0, ((void*)&cpMethod));	  /* AUTOINSERTED TRACE */
	#endif

    m_strHTTPMethod = std::string( cpMethod);
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "setHTTPMethod", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

/* HTTPTransport::sendBytes( SendBuffer, BufferId) Is a public method that
 * concatinates the new send buffer to the bytes to send string.  This message
 * will only be sent when a flush buffer is received.
 *
 * @param const char* SendBufer - Pointer to a NULL terminated character string
 * containing all or some of the transmission message.
 * @param const void* BufferId - Pointer.  This parameter is ignored.
 *
 * @return AXIS_TRANSPORT_STATUS Value to a status value (currently it will
 * always be TRANSPORT_IN_PROGRESS).
 */

AXIS_TRANSPORT_STATUS HTTPTransport::sendBytes( const char *pcSendBuffer, const void *pBufferId)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HTTPTransport", "sendBytes", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pcSendBuffer),
					TRACETYPE_POINTER, 1, ((void*)&pBufferId));	  /* AUTOINSERTED TRACE */
	#endif

    m_strBytesToSend += std::string (pcSendBuffer);
    	{
		#ifdef ENABLE_AXISTRACE
			AXIS_TRANSPORT_STATUS traceRet = (TRANSPORT_IN_PROGRESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "sendBytes", this, 0,
					TRACETYPE_DATA, sizeof(AXIS_TRANSPORT_STATUS), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return TRANSPORT_IN_PROGRESS;
		#endif
	}

}

/* HTTPTransport::getBytes( ReceiveBuffer, Size) Is a public method that will
 * receive the synchronous reply to the sent message.
 *
 * @param const char* ReceiveBuffer - Pointer to a character string that on
 * return will containing all or part of the received message.
 * @param int* Size - Pointer to an integer value that on return will contain
 * the length of the received message.
 *
 * @return AXIS_TRANSPORT_STATUS Value to the status o message reception
 * (TRANSPORT_FINISHED or TRANSPORT_IN_PROGRESS).
 */

AXIS_TRANSPORT_STATUS HTTPTransport::getBytes( char * pcBuffer, int * piSize) throw (AxisException, HTTPTransportException)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HTTPTransport", "getBytes", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pcBuffer),
					TRACETYPE_POINTER, sizeof(int), ((void*)&piSize));	  /* AUTOINSERTED TRACE */
	#endif

	std::string	nextChunk = "";
// It is assumed that the constructor has initialised the following variables:-
// Type			Label			Initial Value		Description
// enum			m_eMsgState		eHTTP_Header		Message decode state.
// int			m_iBytesLeft	0					Size of string held in m_strReceived.
// char *		m_pszRxBuffer	new char [BUF_SIZE]	Local buffer with message read from channel.
// std::string	m_strReceived   <uninitialised>		Contains the concatination of all unprocessed message string parts.
// The method getBytes has three distinct states.  These are defined as
// follows:-
//   eWaitingForHTTPHeader    - Waiting for a 'valid' HTTP header.  A header is
//                              is valid when a 'HTTP' string is found that is
//                              subsequently followed by a CRLFCRLF sequence.
//                              The HTTP header is then read and if the message
//                              number is incorrect an exception is thrown.
//                              The header also contains information on whether
//                              the accompanying message is or is not chunked.
//   eSOAPMessageIsChunked    - When a message is chunked, it has been broken
//                              into a number of blocks.  Each block is
//                              preceeded by a hex number.  The number is the
//                              chunk length.  A zero length chunk represents
//                              the end of the message.
//                              NB: because AXIS uses a 'pull' parser, the last
//                                  chunk size should never be read as the
//                                  parser will return once it has encountered
//                                  '</envelope>'.  If it does read the last
//                                  '0', this could indicate that there is a
//                                  problem in the message.
//   eSOAPMessageIsNotChunked - The message is read until the number of message
//                              bytes read equals the HTTP header content
//                              length value.
	switch( m_GetBytesState)
	{
		case eWaitingForHTTPHeader:
		{
			// If there is still data to be sent, then drop out of the switch statement and send more data!
			if( m_iBytesLeft > 0)
			{
				break;
			}
		// Wait for a HTTP header to be located on the input stream.
			do
			{
		// From the input stream, wait for a 'valid' HTTP header.
				readHTTPHeader();
		// From the header,extract such things as chunking, message length, etc.
				processHTTPHeader();
			} while( m_iResponseHTTPStatusCode == 100);
		// Check that the HTTP status code is valid.
			checkHTTPStatusCode();
		// Done with HTTP headers, get SOAP message.
			int iHTTPStart = m_strReceived.find( ASCII_S_HTTP);
			int iHTTPEnd = m_strReceived.find( ASCII_S_CRLFCRLF, iHTTPStart);
			m_strReceived = m_strReceived.substr( iHTTPEnd + strlen( ASCII_S_CRLFCRLF));
			m_iBytesLeft = m_strReceived.length();
		// This bit of code should not be necessary, but just in case...
			if( m_GetBytesState == eWaitingForHTTPHeader)
			{
				break;
			}
		}
		// At this point it is assumed that m_strReceived contains the block of
		// unprocessed data.  m_iBytesLeft is the length of text/data in m_strReceived
		// is a 'char *' type copy of the m_strReceived string.
		// NB: It is assumed that all of these variables ARE in sync at this point.
		case eSOAPMessageIsChunked:
			{
				if( m_GetBytesState == eSOAPMessageIsChunked)
				{
					if( m_iBytesLeft == 0)
					{
						getNextDataPacket( "No data available for next chunk size.");
					}
					m_iContentLength = getChunkSize();
		// If the chunk size is larger than the available data, then read in more data
		// until all of the chunk has been read.
					while( m_iContentLength > m_iBytesLeft)
					{
						getNextDataPacket( "No data available for next chunk.");
					}
		// If data read is longer than chunk size, then copy the extra data to a
		// temporary variable and process data just belonging to this chunk.
					if( m_iBytesLeft > m_iContentLength)
					{
						if( m_strReceived.length() > (m_iContentLength + strlen( ASCII_S_CRLF) + 1))
						{
							nextChunk = m_strReceived.substr( m_iContentLength + strlen( ASCII_S_CRLF));
						}
						else
						{
							nextChunk = "";
						}
						m_strReceived = m_strReceived.substr( 0, m_iContentLength);
						m_iBytesLeft = m_iContentLength;
		// Check to see if the next chunk size is zero.  If it is then change the state.
						if( peekChunkLength( nextChunk) == 0)
						{
							m_GetBytesState = eWaitingForHTTPHeader;
						}
					}
					else
					{
						nextChunk = "";
					}
		// Now have at least chunk size worth of data.  The chunk may contain Mime data
		// (this depends on information in the HTTP header).  If Mime data is expected,
		// process it first.
					if( m_bMimeTrue)
					{
						processRootMimeBody();
						m_iBytesLeft = m_strReceived.length();
					}
					break;
				}
			}
		case eSOAPMessageIsNotChunked:
		{
		// Check that there is more message to read.  Or, if the message has no
		// length defined, keep reading until the connection is closed by the
		// server.
			if( m_iContentLength > 0 ||
				(m_bReopenConnection && m_pActiveChannel->getSocket()))
			{
				if( m_iContentLength > 0)
				{
					getNextDataPacket( "No data available for message.");
				}
				else
				{
					getNextDataPacket( "Expecting server connection to close.");
				}
		// Check for Mime header
				if( m_bMimeTrue)
				{
					processRootMimeBody();
					m_iBytesLeft = m_strReceived.length();
				}
		// Subtract message length (so far) from expcted content length.
				m_iContentLength -= m_iBytesLeft;
		// If all of the message has been received, then reset the process state.
				if( m_iContentLength <= 0)
				{
		// The content length is <= 0, BUT the m_bReopenConnection is true
		// (because a 'Connection: close' has been found) the socket has been
		// closed by the server, then wait for the next HTTP message, otherwise
		// keep reading!
					if( m_bReopenConnection && m_pActiveChannel->getSocket())
					{
						m_iContentLength = 0;
					}
					else
					{
						m_GetBytesState = eWaitingForHTTPHeader;
					}
				}
			}
			else
			{
		// Reset the process state.
				m_GetBytesState = eWaitingForHTTPHeader;
			}
		break;
		}
	}
// Copy as much of the message to the parser buffer as possible.
	if( copyDataToParserBuffer( pcBuffer, piSize, m_iBytesLeft))
	{
		m_strReceived += nextChunk;
		m_iBytesLeft = m_strReceived.length();
			{
		#ifdef ENABLE_AXISTRACE
			AXIS_TRANSPORT_STATUS traceRet = (TRANSPORT_IN_PROGRESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "getBytes", this, 1,
					TRACETYPE_DATA, sizeof(AXIS_TRANSPORT_STATUS), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return TRANSPORT_IN_PROGRESS;
		#endif
	}

	}
		{
		#ifdef ENABLE_AXISTRACE
			AXIS_TRANSPORT_STATUS traceRet = (TRANSPORT_FINISHED);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "getBytes", this, 2,
					TRACETYPE_DATA, sizeof(AXIS_TRANSPORT_STATUS), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return TRANSPORT_FINISHED;
		#endif
	}

}

/* HTTPTransport::setTransportProperty( Type, Value) Is an overloaded public
 * method used to set a HTTP transport or SSL implementation property.
 *
 * @param AXIS_TRANSPORT_INFORMATION_TYPE Type is an enumerated type containing
 * the type of information to be stored in either the HTTP Header or SSL
 * implementation settings.
 * @param const char* Value is a NULL terminated character string containing
 * the value associated with the type.
 */

int HTTPTransport::setTransportProperty( AXIS_TRANSPORT_INFORMATION_TYPE type, const char *value) throw (HTTPTransportException)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HTTPTransport", "setTransportProperty", NULL, 2,
					TRACETYPE_DATA, sizeof(AXIS_TRANSPORT_INFORMATION_TYPE), ((void*)&type),
					TRACETYPE_STRING, 0, ((void*)&value));	  /* AUTOINSERTED TRACE */
	#endif

    const char *key = NULL;
	int			iSuccess = AXIS_SUCCESS;
    switch (type)
    {
	    case SOAPACTION_HEADER:
		{
			key = "SOAPAction";
		    break;
		}
		case SERVICE_URI:		// need to set ?
		{
			break;
		}
		case OPERATION_NAME:	// need to set ?
		{
			break;
		}
		case SOAP_MESSAGE_LENGTH:
		{
			key = "Content-Length";	// this Axis transport handles only HTTP
			break;
		}
		case TRANSPORT_PROPERTIES:
		{
			if( m_pActiveChannel != NULL)
			{
				m_pActiveChannel->setTransportProperty( type, value);
			}
			break;
		}
		case SECURE_PROPERTIES:
		{
			if( m_pActiveChannel != NULL)
			{
				iSuccess = m_pActiveChannel->setSecureProperties( value);
			}
			break;
		}
		case CHANNEL_HTTP_DLL_NAME:
		{
			if( m_pChannelFactory != NULL)
			{
				m_pNormalChannel = m_pChannelFactory->LoadChannelLibrary( UnsecureChannel, value);
			}
			break;
		}
		case CHANNEL_HTTP_SSL_DLL_NAME:
		{
			if( m_pChannelFactory != NULL)
			{
				m_pSecureChannel = m_pChannelFactory->LoadChannelLibrary( SecureChannel, value);
			}
			break;
		}
		default:
		{
			break;
		}
    }
    if( key)
    {
		setTransportProperty( key, value);
    }
		{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (iSuccess);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "setTransportProperty", NULL, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return iSuccess;
		#endif
	}

}

/* HTTPTransport::setTransportProperty( Key, Value) Is an overloaded public
 * method used to set a HTTP transport or SSL implementation property.
 *
 * @param const char* Key is a NULL terminated character string containing
 * the type of information to be stored in either the HTTP Header or SSL
 * implementation settings.
 * @param const char* Value is a NULL terminated character string containing
 * the value associated with the type.
 */

int HTTPTransport::setTransportProperty( const char *pcKey, const char *pcValue) throw (HTTPTransportException)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HTTPTransport", "setTransportProperty", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pcKey),
					TRACETYPE_STRING, 0, ((void*)&pcValue));	  /* AUTOINSERTED TRACE */
	#endif

    if( !pcKey || !pcValue)	// Samisa - fix for AXISCPP-295. We must check for valid values here.
	{
			{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "setTransportProperty", this, 1,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}

	}
    bool b_KeyFound = false;
    // Check for well known headers that we add on in every iteration
    if( strcmp( pcKey, "SOAPAction") == 0 || strcmp( pcKey, "Content-Length") == 0)
    {
		std::string strKeyToFind = std::string( pcKey);
		for (unsigned int i = 0; i < m_vHTTPHeaders.size(); i++)
		{
		    if (m_vHTTPHeaders[i].first == strKeyToFind)
			{
				m_vHTTPHeaders[i].second = (string) pcValue;
				b_KeyFound = true;
				break;
		    }
		}
    }
    else
    {
        if(strcmp(pcKey, "Cookie")==0)
        {
        	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (addCookie(pcValue));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "setTransportProperty", this, 2,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return addCookie(pcValue);
		#endif
	}

        }
    }
    if( !b_KeyFound)
    {
  		m_vHTTPHeaders.push_back( std::make_pair( (string) pcKey, (string) pcValue));
    }
		{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "setTransportProperty", this, 3,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}

}

/* HTTPTransport::getTransportProperty( Type) Is a public method that will
 * return the HTTP Header/SSL implementation value associated with type.
 *
 * @param AXIS_TRANSPORT_INFORMATION_TYPE Type is an enumerated type containing
 * the type of information to be retrieved in either the HTTP Header or SSL
 * implementation settings.
 *
 * @return const char* Value is a NULL terminated character string containing
 * the value associated with the type.
 */

const char * HTTPTransport::getTransportProperty( AXIS_TRANSPORT_INFORMATION_TYPE eType) throw (HTTPTransportException)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HTTPTransport", "getTransportProperty", NULL, 1,
					TRACETYPE_DATA, sizeof(AXIS_TRANSPORT_INFORMATION_TYPE), ((void*)&eType));	  /* AUTOINSERTED TRACE */
	#endif

    const char *pszPropValue = NULL;
    switch( eType)
    {
	    case SOAPACTION_HEADER:
		{
			int iIndex = FindTransportPropertyIndex( "SOAPAction");
		    if (iIndex > -1)
		    {
				pszPropValue = m_vHTTPHeaders[iIndex].second.c_str();
		    }
		    break;
		}
	    case SERVICE_URI:
		{
			break;
		}
	    case OPERATION_NAME:
		{
			break;
		}
	    case SOAP_MESSAGE_LENGTH:
		{
			int iIndex = FindTransportPropertyIndex( "Content-Length");
		    if (iIndex > -1)
		    {
				pszPropValue = m_vHTTPHeaders[iIndex].second.c_str();
		    }
		    break;
		}
	case TRANSPORT_PROPERTIES:
    case SECURE_PROPERTIES:
		{
			pszPropValue = m_pActiveChannel->getTransportProperty( eType);
			break;
		}
	case CHANNEL_HTTP_SSL_DLL_NAME:
	case CHANNEL_HTTP_DLL_NAME:
    case CONTENT_TYPE:
		{
			break;
		}
    }
    	{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (pszPropValue);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "getTransportProperty", NULL, 0,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return pszPropValue;
		#endif
	}

}

/* HTTPTransport::FindTransportPropertyIndex( Key) Is a private method that will
 * return the HTTP Header index associated with Key.
 *
 * @param AXIS_TRANSPORT_INFORMATION_TYPE Key is an enumerated type containing
 * the type of information to be retrieved in either the HTTP Header settings.
 *
 * @return int Index is an index to the key within the HTTP Header list.  If
 * the return value is -1, then the key was not found.
 */

int HTTPTransport::FindTransportPropertyIndex( string sKey)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HTTPTransport", "FindTransportPropertyIndex", NULL, 1,
					TRACETYPE_STLSTRING, 0, ((void*)&sKey));	  /* AUTOINSERTED TRACE */
	#endif

    bool	bKeyFound = false;
    int		iIndex = 0;
    while( (unsigned int) iIndex < m_vHTTPHeaders.size() && !bKeyFound)
    {
		if (!m_vHTTPHeaders[iIndex].first.compare( sKey))
		{
			bKeyFound = true;
		}
		else
		{
			iIndex++;
		}
    } 
    if( !bKeyFound)
    {
		iIndex = -1;
    }
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (iIndex);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "FindTransportPropertyIndex", NULL, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return iIndex;
		#endif
	}

}

/* HTTPTransport::getServiceName() Is a public method to return the HTTP
 * Header service name.
 *
 * @return const char* Value is a NULL terminated character string containing
 * the value associated with the service name.
 */

const char * HTTPTransport::getServiceName()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HTTPTransport", "getServiceName", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    //Assume SOAPAction header to contain service name
    int		iIndex = FindTransportPropertyIndex( "SOAPAction");
    if (iIndex > -1)
    {
			{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (m_vHTTPHeaders[iIndex].second.c_str());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "getServiceName", this, 1,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_vHTTPHeaders[iIndex].second.c_str();
		#endif
	}

    }
    	{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "getServiceName", this, 2,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

}

AXIS_PROTOCOL_TYPE HTTPTransport::getProtocol()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HTTPTransport", "getProtocol", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			AXIS_PROTOCOL_TYPE traceRet = (m_eProtocolType);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "getProtocol", this, 0,
					TRACETYPE_DATA, sizeof(AXIS_PROTOCOL_TYPE), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_eProtocolType;
		#endif
	}

}

int HTTPTransport::setProtocol( AXIS_PROTOCOL_TYPE eProtocol)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HTTPTransport", "setProtocol", this, 1,
					TRACETYPE_DATA, sizeof(AXIS_PROTOCOL_TYPE), ((void*)&eProtocol));	  /* AUTOINSERTED TRACE */
	#endif

    if (eProtocol == APTHTTP1_1 || eProtocol == APTHTTP1_0)
    {
		m_eProtocolType = eProtocol;
		m_strHTTPProtocol = (m_eProtocolType == APTHTTP1_1) ? "HTTP/1.1" : "HTTP/1.0";
			{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "setProtocol", this, 1,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}

    }
    else
	{
			{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_FAIL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "setProtocol", this, 2,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_FAIL;
		#endif
	}

	}
}

/**
 * HTTPTransport::getSubProtocol() is a public method that is supposed to
 * return the sub protocol (currently this method always return 0).
 * This method is supposed to return whether it is http GET or POST
 */

int HTTPTransport::getSubProtocol()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HTTPTransport", "getSubProtocol", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    //TODO
    // for SimpleAxisServer assume POST
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_HTTP_POST);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "getSubProtocol", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_HTTP_POST;
		#endif
	}

    //return 0;
}

/* HTTPTransport::setProxy( Host, Port) Is a public method for setting or
 * updating the proxy for the connection.
 *
 * @param const char* Host is a NULL terminated character string containing the new
 * proxy host.
 * @param unsigned int Port is the new proxy port number.
 */

void HTTPTransport::setProxy( const char *pcProxyHost, unsigned int uiProxyPort)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HTTPTransport", "setProxy", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pcProxyHost),
					TRACETYPE_UINT, 0, ((void*)&uiProxyPort));	  /* AUTOINSERTED TRACE */
	#endif

    m_pActiveChannel->setProxy(pcProxyHost,uiProxyPort);
	m_strProxyHost = pcProxyHost;
    m_uiProxyPort = uiProxyPort;
    m_bUseProxy = true;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "setProxy", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

/* HTTPTransport::setTimeout( Timeout) Is a public method for setting the
 * current maximum timeout period between that can elapse between receiving
 * message parts.
 *
 * @param long Timeout is a long value in seconds.
 */

void HTTPTransport::setTimeout( long lSeconds)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HTTPTransport", "setTimeout", this, 1,
					TRACETYPE_LONG, 0, ((void*)&lSeconds));	  /* AUTOINSERTED TRACE */
	#endif

	if( m_pActiveChannel != NULL)
	{
		m_pActiveChannel->setTimeout( lSeconds);
	}
	m_lChannelTimeout = lSeconds;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "setTimeout", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

/* HTTPTransport::getHTTPProtocol() Is a public method for retrieving the
 * current HTTP protocol settings.
 *
 * @return const char* HTTPProtocol is a NULL terminated character string
 * containing the HTTP protocol.
 */

const char * HTTPTransport::getHTTPProtocol()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HTTPTransport", "getHTTPProtocol", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (m_strHTTPProtocol.c_str ());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "getHTTPProtocol", this, 0,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_strHTTPProtocol.c_str ();
		#endif
	}

}

/* axtoi( Hex) Is a private method to convert an ascii hex string to an integer.
 */

int axtoi( char *pcHexString)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, "axtoi", NULL, 1,
					TRACETYPE_STRING, 0, ((void*)&pcHexString));	  /* AUTOINSERTED TRACE */
	#endif

    int		iN = 0;			// position in string
    int		iM = 0;			// position in digit[] to shift
    int		iCount;			// loop index
    int		intValue = 0;	// integer value of hex string
    int		iDigit[32];		// hold values to convert
/*
	char *	pcHexString = pcHexStringIn;
	// Remove whitespace characters from string before processing.
	while( *pcHexString == ' ' || *pcHexString == '\r' || *pcHexString == '\n' || *pcHexString == '\t') 
	{
		pcHexString++;
	}
*/
    while( iN < 32)
    {
		if( pcHexString[iN] == '\0')
		{
			break;
		}
	
		if( pcHexString[iN] >= ASCII_C_ZERO && 
			pcHexString[iN] <= ASCII_C_NINE)
		{
			iDigit[iN] = pcHexString[iN] & 0x0f;	//convert to int
		}
		else if ((pcHexString[iN] >= ASCII_C_LOWERCASEA && 
		          pcHexString[iN] <= ASCII_C_LOWERCASEF) ||
		         (pcHexString[iN] >= ASCII_C_UPPERCASEA && 
		          pcHexString[iN] <= ASCII_C_UPPERCASEF))
		{
			iDigit[iN] = (pcHexString[iN] & 0x0f) + 9;	//convert to int
		}
		else
		{
			break;
		}
		iN++;
    }
    iCount = iN;
    iM = iN - 1;
    iN = 0;
    while( iN < iCount)
    {
		// digit[n] is value of hex digit at position n
		// (m << 2) is the number of positions to shift
		// OR the bits into return value
		intValue = intValue | (iDigit[iN] << (iM << 2));
		iM--;			// adjust the position to set
		iN++;			// next digit to process
    }
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (intValue);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, "axtoi", NULL, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return intValue;
		#endif
	}

}

/* HTTPTransport::processResponseHTTPHeaders() Is a public method used to
 * parse the HTTP header of the response message.
 */

void HTTPTransport::processResponseHTTPHeaders() throw (HTTPTransportException)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HTTPTransport", "processResponseHTTPHeaders", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    unsigned long iPosition = std::string::npos;
    unsigned long iStartPosition = iPosition;
    if( (iPosition = m_strResponseHTTPHeaders.find( ASCII_S_HTTP )) != std::string::npos)
    {
		m_strResponseHTTPProtocol = m_strResponseHTTPHeaders.substr( iPosition, strlen( "HTTP/1.x"));
		iPosition += strlen( "HTTP/1.x");
		while( m_strResponseHTTPHeaders.substr()[iPosition] == ASCII_C_SPACE)
		{
			iPosition++;
		}
		iStartPosition = iPosition;
		while( m_strResponseHTTPHeaders.substr()[iPosition] != ASCII_C_SPACE )
		{
			iPosition++;
		}
		std::string strResponseHTTPStatusCode = 
               m_strResponseHTTPHeaders.substr( iStartPosition,iPosition - iStartPosition);
		m_iResponseHTTPStatusCode = atoi( PLATFORM_ASCTOSTR(strResponseHTTPStatusCode.c_str()));
		iStartPosition = ++iPosition;
		iPosition = m_strResponseHTTPHeaders.find( ASCII_S_LF );
		m_strResponseHTTPStatusMessage = 
              m_strResponseHTTPHeaders.substr( iStartPosition,iPosition - iStartPosition - 1);
        PLATFORM_ASCTOSTR(m_strResponseHTTPStatusMessage.c_str());
		// reached the end of the first line
		iStartPosition = m_strResponseHTTPHeaders.find( ASCII_S_LF );
		iStartPosition++;
		// read header fields and add to vector
		do
		{
		    m_strResponseHTTPHeaders = m_strResponseHTTPHeaders.substr( iStartPosition);
			iPosition = m_strResponseHTTPHeaders.find( ASCII_S_LF );
		    if( iPosition == std::string::npos)
		    {
				break;
			}
			std::string		strHeaderLine = m_strResponseHTTPHeaders.substr( 0, iPosition);
			unsigned long	iSeperator = strHeaderLine.find( ASCII_S_COLON );
		    if( iSeperator == std::string::npos)
			{
				break;
			}
			iStartPosition = iPosition + 1;
			string	key = strHeaderLine.substr( 0, iSeperator);
			string	value = strHeaderLine.substr( iSeperator + 1,
												  strHeaderLine.length () - iSeperator - 1 - 1);
            PLATFORM_ASCTOSTR(key.c_str());
            PLATFORM_ASCTOSTR(value.c_str());
			m_vResponseHTTPHeaders.push_back( std::make_pair( key, value));
	   	 	// if HTTP/1.0 we have to always close the connection by default
		    if( m_eProtocolType == APTHTTP1_0)
			{
				m_bReopenConnection = true;
			}
	    	// if HTTP/1.1 we have to assume persistant connection by default
	    	// We need to close the connection and open a new one if we have 'Connection: close'
			if( key == "Connection" && (value == " close" || value == " Close"))
			{
				m_bReopenConnection = true;
				m_pActiveChannel->closeQuietly( true);
			}
            // We need to close the connection and open a new one if we have 'Proxy-Connection: close'
            if (key == "Proxy-Connection" && (value == " close" || value == " Close"))
			{
                m_bReopenConnection = true;
			}
	    	// For both HTTP/1.0 and HTTP/1.1,
	    	// We need to keep the connection if we have 'Connection: Keep-Alive'
			if( key == "Connection" && value == " Keep-Alive")
			{
				m_bReopenConnection = false;
			}
	    	// Look for cookies
			if( m_bMaintainSession )
		    {
				if( key == "Set-Cookie")
                  addCookie(value);
		    }
	    	/* If Content-Type: Multipart/Related; boundary=<MIME_boundary>; type=text/xml;
	       start="<content id>" */
			if( key == "Content-Type")
			{
				m_strContentType = value;
		
				unsigned long	ulMimePos = m_strContentType.find( ";");
				std::string		strTypePart;
				if( ulMimePos != std::string::npos)
				{
					strTypePart = m_strContentType.substr( 1, ulMimePos - 1);
				}
		
				if( "Multipart/Related" == strTypePart)
				{
					m_bMimeTrue = true;
					m_strContentType = m_strContentType.substr( ulMimePos + 1, m_strContentType.length());
				    ulMimePos = m_strContentType.find( "boundary=");
					m_strMimeBoundary = m_strContentType.substr( ulMimePos);
					ulMimePos = m_strMimeBoundary.find( ";");
					m_strMimeBoundary = m_strMimeBoundary.substr( 9, ulMimePos - 9);
                    PLATFORM_STRTOASC(m_strMimeBoundary.c_str());
					ulMimePos = m_strContentType.find( "type=");
					m_strMimeType = m_strContentType.substr( ulMimePos);
					ulMimePos = m_strMimeType.find( ";");
					m_strMimeType = m_strMimeType.substr( 5, ulMimePos - 5);
				    ulMimePos = m_strContentType.find( "start=");
					m_strMimeStart = m_strContentType.substr( ulMimePos);
					ulMimePos = m_strMimeStart.find( ";");
					m_strMimeStart = m_strMimeStart.substr( 6, ulMimePos - 6);
				}
			}
		} while( iPosition != std::string::npos);
    }
    else
    {
		throw HTTPTransportException( SERVER_TRANSPORT_UNKNOWN_HTTP_RESPONSE,
									  "Protocol is not HTTP.");
    }
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "processResponseHTTPHeaders", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

/* HTTPTransport::processRootMimeBody() Is a public method used to
 * parse the mime attachments.
 */

void HTTPTransport::processRootMimeBody()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HTTPTransport", "processRootMimeBody", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    if( false == m_bReadPastRootMimeHeader)
    {
		do
		{
			if( m_strReceived.find( ASCII_S_CRLFCRLF ) == std::string::npos)
			{
				m_pszRxBuffer [0] = '\0';
				*m_pActiveChannel >> m_pszRxBuffer;
				m_strReceived += m_pszRxBuffer;
			}
		} while( m_strReceived.find( ASCII_S_CRLFCRLF ) == std::string::npos);
		//now we have found the end of root mime header
		m_bReadPastRootMimeHeader = true;
		//processMimeHeader(); For the time being we don't process this
		// Done with root mime body headers, get rest of the payload 
		// which contain the soap message
		m_strReceived = m_strReceived.substr( m_strReceived.find( ASCII_S_CRLFCRLF ) + 4);
	
		unsigned long intMimeTemp = m_strReceived.find( m_strMimeBoundary);
		if (intMimeTemp != std::string::npos)
		{
			m_strReceived = m_strReceived.substr( 0, intMimeTemp);
			m_strMimeReceived = m_strReceived.substr( intMimeTemp);
            PLATFORM_ASCTOSTR(m_strMimeReceived.c_str());
	    	// Using m_strMimeReceived will be continued when getAttachment is called.
			m_bMimeTrue = false;
		}
    }
    else
    {
		unsigned long intMimeTemp = m_strReceived.find( m_strMimeBoundary);
		
		if( intMimeTemp != std::string::npos)
		{
			m_strReceived = m_strReceived.substr( 0, intMimeTemp);
			m_strMimeReceived = m_strReceived.substr( intMimeTemp);
            PLATFORM_ASCTOSTR(m_strMimeReceived.c_str());
	    	// Using m_strMimeReceived will be continued when getAttachment is called.
			m_bMimeTrue = false;
		}
		{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "processRootMimeBody", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}

    }
}

/* HTTPTransport::processMimeHeaders() Is a public method used to
 * parse the Mime headers of the response message.
 */

void HTTPTransport::processMimeHeader()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HTTPTransport", "processMimeHeader", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    unsigned long pos = 0;
    unsigned long temppos = 0;
    // Look for content lenght
    if( (pos = m_strMimeReceived.find( "Content-Type: ")) != std::string::npos)
	{
		m_strMimeContentType = m_strMimeReceived.substr( pos + strlen( "Content-Type: "),
														 m_strMimeReceived.find( "\n", pos));
		pos = m_strMimeContentType.find( ";");
		temppos = m_strMimeContentType.find( "\r\n");
		if( pos < temppos)
		{
			m_strMimeContentType = m_strMimeContentType.substr( 0, pos);
		}
		else
		{
			m_strMimeContentType = m_strMimeContentType.substr( 0, temppos);
		}
    }
    // Look for mime root body's content transfer encoding
    if( (pos = m_strMimeReceived.find( "Content-Transfer-Encoding: ")) != std::string::npos)
	{
		m_strMimeContentTransferEncoding = m_strMimeReceived.substr( pos + strlen( "Content-Transfer-Encoding: "),
																     m_strMimeReceived.find( "\n", pos));
		temppos = m_strMimeContentTransferEncoding.find( "\r\n");
		m_strMimeContentTransferEncoding = m_strMimeContentTransferEncoding.substr( 0, temppos);
    }
    // Look for mime root body's content id
    if( (pos = m_strMimeReceived.find( "Content-ID: ")) != std::string::npos)
	{
		m_strMimeContentID = m_strMimeReceived.substr( pos + strlen( "Content-ID: "),
													   m_strMimeReceived.find( "\n", pos));
		temppos = m_strMimeContentID.find( "\r\n");
		m_strMimeContentID = m_strMimeContentID.substr( 0, temppos);
    }
    // Look for mime root body's content location
    if( (pos = m_strMimeReceived.find( "Content-Location: ")) != std::string::npos)
    {
		m_strMimeContentLocation = atoi( m_strMimeReceived.substr( pos + strlen( "Content-Location: "),
																   m_strMimeReceived.find( "\n", pos)).c_str());
		temppos = m_strMimeContentLocation.find( "\r\n");
		m_strMimeContentLocation = m_strMimeContentLocation.substr( 0, temppos);
    }
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "processMimeHeader", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void HTTPTransport::processMimeBody ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HTTPTransport", "processMimeBody", this, 0);	  /* AUTOINSERTED TRACE */
	#endif


	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "processMimeBody", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void HTTPTransport::getAttachment( char * pStrAttachment, int * pIntSize, int intAttachmentId)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HTTPTransport", "getAttachment", this, 3,
					TRACETYPE_STRING, 0, ((void*)&pStrAttachment),
					TRACETYPE_POINTER, sizeof(int), ((void*)&pIntSize),
					TRACETYPE_INT, 0, ((void*)&intAttachmentId));	  /* AUTOINSERTED TRACE */
	#endif

	m_pszRxBuffer [0] = '\0';
	*m_pActiveChannel >> m_pszRxBuffer;
    m_strMimeReceived += m_pszRxBuffer;
    do
    {
		if( m_strMimeReceived.find( "\r\n\r\n") == std::string::npos)
		{
			m_pszRxBuffer [0] = '\0';
			*m_pActiveChannel >> m_pszRxBuffer;
		    m_strMimeReceived += m_pszRxBuffer;
		}
    } while( m_strMimeReceived.find( "\r\n\r\n") == std::string::npos);
    //now we have found the end of next mime header
    processMimeHeader();
    m_strMimeReceived = m_strMimeReceived.substr( m_strMimeReceived.find( "\r\n\r\n"));
    processMimeBody();
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "getAttachment", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void HTTPTransport::setSocket( unsigned int uiNewSocket)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HTTPTransport", "setSocket", this, 1,
					TRACETYPE_UINT, 0, ((void*)&uiNewSocket));	  /* AUTOINSERTED TRACE */
	#endif

    m_pActiveChannel->setSocket( uiNewSocket);
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "setSocket", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

const char * HTTPTransport::getTransportProperty( const char * pcKey, bool response) throw (HTTPTransportException)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HTTPTransport", "getTransportProperty", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pcKey),
					TRACETYPE_BOOL, 0, ((void*)&response));	  /* AUTOINSERTED TRACE */
	#endif

    std::string strKeyToFind = std::string( pcKey);
	std::vector < std::pair < std::string, std::string > > *hdrs=NULL;
	if (response) hdrs = &m_vResponseHTTPHeaders;
	else hdrs = &m_vHTTPHeaders;
    for( unsigned int i = 0; i < hdrs->size(); i++)
	{
		if( (*hdrs)[i].first == strKeyToFind)
		{
				{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = ((*hdrs)[i].second.c_str());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "getTransportProperty", this, 1,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return (*hdrs)[i].second.c_str();
		#endif
	}

		}
    }
    	{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "getTransportProperty", this, 2,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

}

const char * HTTPTransport::getFirstTransportPropertyKey(bool response)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HTTPTransport", "getFirstTransportPropertyKey", this, 1,
					TRACETYPE_BOOL, 0, ((void*)&response));	  /* AUTOINSERTED TRACE */
	#endif

    if(response)
    {
        m_viCurrentResponseHeader = m_vResponseHTTPHeaders.begin ();
    
        if( m_viCurrentResponseHeader == m_vResponseHTTPHeaders.end())
        {
         	{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "getFirstTransportPropertyKey", this, 1,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

        }
        else
        {
         	{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = ((*m_viCurrentResponseHeader).first.c_str());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "getFirstTransportPropertyKey", this, 2,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return (*m_viCurrentResponseHeader).first.c_str();
		#endif
	}

        }
    }
    else
    {
        m_viCurrentHeader = m_vHTTPHeaders.begin ();
    
        if( m_viCurrentHeader == m_vHTTPHeaders.end())
    	{
    			{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "getFirstTransportPropertyKey", this, 3,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

    	}
        else
    	{
    			{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = ((*m_viCurrentHeader).first.c_str());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "getFirstTransportPropertyKey", this, 4,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return (*m_viCurrentHeader).first.c_str();
		#endif
	}

    	}
    }
}

const char * HTTPTransport::getNextTransportPropertyKey(bool response)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HTTPTransport", "getNextTransportPropertyKey", this, 1,
					TRACETYPE_BOOL, 0, ((void*)&response));	  /* AUTOINSERTED TRACE */
	#endif

    if(response)
    {
        //already at the end?
        if( m_viCurrentResponseHeader == m_vResponseHTTPHeaders.end())
        {
            	{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "getNextTransportPropertyKey", this, 1,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

        }
        m_viCurrentResponseHeader++;
        if( m_viCurrentResponseHeader == m_vResponseHTTPHeaders.end())
        {
            	{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "getNextTransportPropertyKey", this, 2,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

        }
        else
        {
            	{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = ((*m_viCurrentResponseHeader).first.c_str());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "getNextTransportPropertyKey", this, 3,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return (*m_viCurrentResponseHeader).first.c_str();
		#endif
	}

        }
    }
    else
    {
    
        //already at the end?
        if( m_viCurrentHeader == m_vResponseHTTPHeaders.end())
    	{
    			{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "getNextTransportPropertyKey", this, 4,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

    	}
    
        m_viCurrentHeader++;
    
        if( m_viCurrentHeader == m_vHTTPHeaders.end())
    	{
    			{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "getNextTransportPropertyKey", this, 5,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

    	}
        else
    	{
    			{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = ((*m_viCurrentHeader).first.c_str());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "getNextTransportPropertyKey", this, 6,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return (*m_viCurrentHeader).first.c_str();
		#endif
	}

    	}
    }
}

const char * HTTPTransport::getCurrentTransportPropertyKey(bool response)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HTTPTransport", "getCurrentTransportPropertyKey", this, 1,
					TRACETYPE_BOOL, 0, ((void*)&response));	  /* AUTOINSERTED TRACE */
	#endif

    if (response)
    {
        if( m_viCurrentResponseHeader == m_vResponseHTTPHeaders.end())
        {
            	{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "getCurrentTransportPropertyKey", this, 1,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

        }
        else
        {
            	{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = ((*m_viCurrentResponseHeader).first.c_str());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "getCurrentTransportPropertyKey", this, 2,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return (*m_viCurrentResponseHeader).first.c_str();
		#endif
	}

        }
    }
    else
    {
        if( m_viCurrentHeader == m_vHTTPHeaders.end())
        {
            	{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "getCurrentTransportPropertyKey", this, 3,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

        }
        else
        {
            	{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = ((*m_viCurrentHeader).first.c_str());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "getCurrentTransportPropertyKey", this, 4,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return (*m_viCurrentHeader).first.c_str();
		#endif
	}

        }
    }
}

const char * HTTPTransport::getCurrentTransportPropertyValue(bool response)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HTTPTransport", "getCurrentTransportPropertyValue", this, 1,
					TRACETYPE_BOOL, 0, ((void*)&response));	  /* AUTOINSERTED TRACE */
	#endif

    if(response)
    {
        if( m_viCurrentResponseHeader == m_vResponseHTTPHeaders.end())
        {
            	{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "getCurrentTransportPropertyValue", this, 1,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

        }
        else
        {
            	{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = ((*m_viCurrentResponseHeader).second.c_str());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "getCurrentTransportPropertyValue", this, 2,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return (*m_viCurrentResponseHeader).second.c_str();
		#endif
	}

        }
    }
    else
    {
        if( m_viCurrentHeader == m_vHTTPHeaders.end())
        {
            	{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "getCurrentTransportPropertyValue", this, 3,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

        }
        else
        {
            	{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = ((*m_viCurrentHeader).second.c_str());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "getCurrentTransportPropertyValue", this, 4,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return (*m_viCurrentHeader).second.c_str();
		#endif
	}

        }
    }
}

void HTTPTransport::deleteCurrentTransportProperty(bool response)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HTTPTransport", "deleteCurrentTransportProperty", this, 1,
					TRACETYPE_BOOL, 0, ((void*)&response));	  /* AUTOINSERTED TRACE */
	#endif

    // response=true by default
    std::vector < std::pair < std::string, std::string > >* headers = &m_vResponseHTTPHeaders;
    vector <std::pair < std::string, std::string > >::iterator* currentHeader = &m_viCurrentResponseHeader;
    if(!response)
    {
        headers = &m_vHTTPHeaders;
        currentHeader = &m_viCurrentHeader;
    }
    if( *currentHeader != headers->end())
    {
       headers->erase( *currentHeader);
    }
//    if( m_viCurrentHeader != m_vHTTPHeaders.end())
//    {
//		m_vHTTPHeaders.erase( m_viCurrentHeader);
//    }
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "deleteCurrentTransportProperty", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void HTTPTransport::deleteTransportProperty (char *pcKey, unsigned int uiOccurance)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HTTPTransport", "deleteTransportProperty", this, 2,
					TRACETYPE_STRING, 0, ((void*)&pcKey),
					TRACETYPE_UINT, 0, ((void*)&uiOccurance));	  /* AUTOINSERTED TRACE */
	#endif

    vector < std::pair < std::string,
	std::string > >::iterator currentHeader = m_vHTTPHeaders.begin();
    unsigned int uiCount = 1;
    bool found=false;
    while( currentHeader != m_vHTTPHeaders.end() && uiCount <= uiOccurance)
    {
		if( strcmp( pcKey, (*currentHeader).first.c_str()) == 0)
		{
			if( uiCount == uiOccurance)
			{
				m_vHTTPHeaders.erase( currentHeader);
                // if this is the special case of cookies then delete them all
                if(strcmp(pcKey, "Cookie")==0)
                {
                 removeAllCookies();
                }
                found=true;
				break;
			}
	    
			uiCount++;
		}
	
		currentHeader++;
    }
    // if the property has not been found then it might be a cookie
    if(!found)
    {
        removeCookie(pcKey);
    }
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "deleteTransportProperty", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void HTTPTransport::setMaintainSession( bool bSession)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HTTPTransport", "setMaintainSession", this, 1,
					TRACETYPE_BOOL, 0, ((void*)&bSession));	  /* AUTOINSERTED TRACE */
	#endif

    m_bMaintainSession = bSession;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "setMaintainSession", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void HTTPTransport::setSessionId( const char * pcSessionId)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HTTPTransport", "setSessionId", this, 1,
					TRACETYPE_STRING, 0, ((void*)&pcSessionId));	  /* AUTOINSERTED TRACE */
	#endif

    m_strSessionKey = std::string (pcSessionId);
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "setSessionId", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

const char * HTTPTransport::getSessionId()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HTTPTransport", "getSessionId", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    	{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (m_strSessionKey.c_str());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "getSessionId", this, 0,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_strSessionKey.c_str();
		#endif
	}

}

const char * HTTPTransport::getLastChannelError()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HTTPTransport", "getLastChannelError", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

	if( m_pActiveChannel != NULL)
	{
			{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (m_pActiveChannel->GetLastErrorMsg().c_str());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "getLastChannelError", this, 1,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_pActiveChannel->GetLastErrorMsg().c_str();
		#endif
	}

	}
		{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "getLastChannelError", this, 2,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

}

void HTTPTransport::readHTTPHeader()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HTTPTransport", "readHTTPHeader", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

// The parser is expecting a SOAP message.  Thus, the HTTP header must have
// been read and processed before control is returned to the parser.  It can
// not be assumed that the HTTP header will be read in one block, thus there
// must be processing that first identifies the beginning of the HTTP header
// block (i.e. looks for 'HTTP') and then additional processing that identifies
// the end of the HTTP header block (i.e. looks for CR LF CR LF).  To stop the
// search becoming 'stuck' because of an incomplete, corrupt or unexpected
// message an iteration count has been added (this could become configurable if
// the user needs to remove this feature if the server is particularily slow,
// etc.).
	bool	bHTTPHeaderFound = false;
	int		iIterationCount = 100;
	m_strReceived = "";
	m_pActiveChannel->closeQuietly( false);
	do
	{
// Read whatever part of the response message that has arrived at the active
// channel socket.
		m_pszRxBuffer[0] = '\0';
		*m_pActiveChannel >> m_pszRxBuffer;
// Add the new message part to the received string.
		m_strReceived += m_pszRxBuffer;
// Do iteration processing.
		if( strlen( m_pszRxBuffer) > 0)
		{
			iIterationCount = 100;
		}
		else
		{
			iIterationCount--;
		}
// Check for beginning and end of HTTP header.
		if( m_strReceived.find( ASCII_S_HTTP) != std::string::npos &&
			m_strReceived.find( ASCII_S_CRLFCRLF) != std::string::npos)
		{
			bHTTPHeaderFound = true;
		}
	} while( !bHTTPHeaderFound && iIterationCount > 0);
// If the HTTP header was not found in the given number of iterations then
// throw an exception.
	if( iIterationCount == 0)
	{
		throw HTTPTransportException( SERVER_TRANSPORT_INPUT_STREAMING_ERROR,
									  "Timed out waiting for HTTP header message.");
	}
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "readHTTPHeader", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void HTTPTransport::processHTTPHeader()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HTTPTransport", "processHTTPHeader", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

// At this point the HTTP header has been found.  It now needs to be processed.
	int	iPosContentLength = m_strReceived.find( ASCII_S_CONTENT_LENGTH);
	if( iPosContentLength != std::string::npos)
	{
		int	iEOL = m_strReceived.find( ASCII_S_LF, iPosContentLength);
		iPosContentLength += strlen( ASCII_S_CONTENT_LENGTH);
		m_iContentLength = atoi( m_strReceived.substr( iPosContentLength, iEOL).c_str());
		m_GetBytesState = eSOAPMessageIsNotChunked;
	}
// Check if the message is chunked
	int	iPosChunked = m_strReceived.find( ASCII_S_TRANSFERENCODING_CHUNKED);
	int iHTTPStart = m_strReceived.find( ASCII_S_HTTP);
	int iHTTPEnd = m_strReceived.find( ASCII_S_CRLFCRLF);
	if( iPosChunked != std::string::npos)
	{
//Chunked data looks like ->
//      Chunked-Body   = *chunk
//                       "0" CRLF
//                       footer
//                       CRLF
//
//      chunk          = chunk-size [ chunk-ext ] CRLF
//                         chunk-data CRLF
//
//      hex-no-zero    = <HEX excluding "0">
//
//      chunk-size     = hex-no-zero *HEX
//      chunk-ext      = *( ";" chunk-ext-name [ "=" chunk-ext-value ] )
//      chunk-ext-name = token
//      chunk-ext-val  = token | quoted-string
//      chunk-data     = chunk-size(OCTET)
//
//      footer         = *entity-header
		m_GetBytesState = eSOAPMessageIsChunked;
	}
	else
	{
		if( (m_eProtocolType == APTHTTP1_0) || (m_eProtocolType == APTHTTP1_1) )
		{
			m_GetBytesState = eSOAPMessageIsNotChunked;
			m_iBytesLeft = m_strReceived.substr( iHTTPEnd + 2).length();
// Check if all the message has already been received.  If not, then subtract
// that bit that has been from the total length.  If so, then make the content
// length equal to zero.
			if( m_iContentLength >= m_iBytesLeft)
			{
				m_iContentLength -= m_iBytesLeft;
			}
			else
			{
				m_iContentLength = 0;
			}
		}
		else
		{
			m_GetBytesState = eWaitingForHTTPHeader;
			throw HTTPTransportException( SERVER_TRANSPORT_INPUT_STREAMING_ERROR,
										  "HTTP header message must be chunked or have a content length.");
		}
	}
// Extract HTTP header and process it
	m_strResponseHTTPHeaders = m_strReceived.substr( iHTTPStart, iHTTPEnd + 2 - iHTTPStart);
	processResponseHTTPHeaders();
// If the HTTP message is a 'continue' message then remove the HTTP header and
// then repeat HTTP header processing.
	if( m_iResponseHTTPStatusCode == 100)
	{
		m_strReceived = m_strReceived.substr( iHTTPEnd + 2);
	}
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "processHTTPHeader", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void HTTPTransport::checkHTTPStatusCode()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HTTPTransport", "checkHTTPStatusCode", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

// Now have a valid HTTP header that is not 100.
	if ( m_iResponseHTTPStatusCode != 500 &&
		 (m_iResponseHTTPStatusCode < 200 ||
		  m_iResponseHTTPStatusCode >= 300))
	{
		m_GetBytesState = eWaitingForHTTPHeader;
		m_strResponseHTTPStatusMessage = std::string( "\n Server sent HTTP error: '") +
										 m_strResponseHTTPStatusMessage +
										 std::string("'\n");
		throw HTTPTransportException( SERVER_TRANSPORT_HTTP_EXCEPTION,
								  const_cast <char *> (m_strResponseHTTPStatusMessage.c_str()));
	}
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "checkHTTPStatusCode", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

bool HTTPTransport::getNextDataPacket( const char * pcszExceptionMessage)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HTTPTransport", "getNextDataPacket", this, 1,
					TRACETYPE_STRING, 0, ((void*)&pcszExceptionMessage));	  /* AUTOINSERTED TRACE */
	#endif

	int		iIterationCount = 100;
	bool	bDataRead = false;
	do
	{
// Read whatever part of the response message that has arrived at the active
// channel socket.
		m_pszRxBuffer[0] = '\0';
		*m_pActiveChannel >> m_pszRxBuffer;
// Do iteration processing.
		if( strlen( m_pszRxBuffer) == 0)
		{
			iIterationCount--;
		}
		else
		{
			bDataRead = true;
		}
	} while( !bDataRead && iIterationCount > 0);
	if( bDataRead)
	{
		m_strReceived += m_pszRxBuffer;
		m_iBytesLeft = m_strReceived.length();
	}
	else
	{
		if( m_strReceived.length() == 0)
		{
			m_GetBytesState = eWaitingForHTTPHeader;
			if( pcszExceptionMessage != NULL && strlen( pcszExceptionMessage) > 0)
			{
				int	iStringLength = strlen( pcszExceptionMessage) + 1;
				const char * pszLastError = new char[iStringLength];
				memcpy( (void *) pszLastError, pcszExceptionMessage, iStringLength);
				throw HTTPTransportException( SERVER_TRANSPORT_HTTP_EXCEPTION,
											  (char *) pszLastError);
			}
		}
	}
		{
		#ifdef ENABLE_AXISTRACE
			bool traceRet = (bDataRead);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "getNextDataPacket", this, 0,
					TRACETYPE_BOOL, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return bDataRead;
		#endif
	}

}

int HTTPTransport::getChunkSize()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HTTPTransport", "getChunkSize", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

	int	iChunkSize;
	while( m_strReceived.find( ASCII_S_CRLF) == std::string::npos)
	{
		getNextDataPacket( "Could not find delimiter for end of chunk size.");
	}
	int iEndOfChunkData = m_strReceived.find( ASCII_S_CRLF) + strlen( ASCII_S_CRLF);
	int iEndOfChunkSize = m_strReceived.find( ASCII_S_CRLF);
// Now get the size of the chunk from the data.  Look to see if there are any
// extensions - these are put in brackets so look for those.
	if( m_strReceived.substr( 0, iEndOfChunkSize).find( ASCII_S_LEFTPAREN) != string::npos)
	{
		iEndOfChunkSize = m_strReceived.find( ASCII_S_LEFTPAREN);
	}
// Convert the hex string into the length of the chunk.
	iChunkSize = axtoi( (char *) m_strReceived.substr( 0, iEndOfChunkSize).c_str());
	m_strReceived = m_strReceived.substr( iEndOfChunkData);
	m_iBytesLeft = m_strReceived.length();
		{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (iChunkSize);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "getChunkSize", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return iChunkSize;
		#endif
	}

}

bool HTTPTransport::copyDataToParserBuffer( char * pcBuffer, int * piSize, int iBytesToCopy)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HTTPTransport", "copyDataToParserBuffer", this, 3,
					TRACETYPE_STRING, 0, ((void*)&pcBuffer),
					TRACETYPE_POINTER, sizeof(int), ((void*)&piSize),
					TRACETYPE_INT, 0, ((void*)&iBytesToCopy));	  /* AUTOINSERTED TRACE */
	#endif

	bool	bTransportInProgress = false;
	if( iBytesToCopy > 0)
	{
		int iToCopy = (*piSize < iBytesToCopy) ? *piSize : iBytesToCopy;
		strncpy( pcBuffer, m_strReceived.c_str(), iToCopy);
		m_iBytesLeft -= iToCopy;
		*piSize = iToCopy;
		if( m_iBytesLeft > 0)
		{
			m_strReceived = m_strReceived.substr( iToCopy);
		}
		else
		{
			m_strReceived = "";
		}
		bTransportInProgress = true;
	}
		{
		#ifdef ENABLE_AXISTRACE
			bool traceRet = (bTransportInProgress);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "copyDataToParserBuffer", this, 0,
					TRACETYPE_BOOL, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return bTransportInProgress;
		#endif
	}

}

int HTTPTransport::addCookie(const string name, const string value)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HTTPTransport", "addCookie", this, 2,
					TRACETYPE_STLSTRING, 0, ((void*)&name),
					TRACETYPE_STLSTRING, 0, ((void*)&value));	  /* AUTOINSERTED TRACE */
	#endif

    // trim the name
    string theName(name);
    trim(theName);
    // Make sure that the cookie is not duplicated.
    // This cookie might be replacing one that's already there
    bool b_keyFound=false;
    for (unsigned int i = 0; i < m_vCookies.size(); i++)
    {
        if (m_vCookies[i].first == theName)
        {
            m_vCookies[i].second = (string) value;
            b_keyFound = true;
            break;
        }
     }
 
 
    if(!b_keyFound)
    {
     // cookie has not already been found
     m_vCookies.push_back( std::make_pair( theName, value));
    }
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (true);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "addCookie", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return true;
		#endif
	}

}

int HTTPTransport::addCookie(const string nameValuePair)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HTTPTransport", "addCookie", this, 1,
					TRACETYPE_STLSTRING, 0, ((void*)&nameValuePair));	  /* AUTOINSERTED TRACE */
	#endif

    // Spec syntax : Set-Cookie: NAME=VALUE; expires=DATE; path=PATH; domain=DOMAIN_NAME; secure
    // This code assumes it to be : Set-Cookie: NAME=VALUE; Anything_else
    // And discards stuff after first ';'
    // This is the same assumption used in Axis Java
    
    unsigned long ulKeyEndsAt = nameValuePair.find( ";");
           
    string nameValue;
    if( ulKeyEndsAt != std::string::npos)
    {
       nameValue = nameValuePair.substr( 0, ulKeyEndsAt);
    }
    // Now split the nameValue up
    unsigned long nameEndsAt = nameValue.find("=");
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (addCookie(nameValue.substr(0, nameEndsAt), nameValue.substr(nameEndsAt+1)));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "addCookie", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return addCookie(nameValue.substr(0, nameEndsAt), nameValue.substr(nameEndsAt+1));
		#endif
	}

}

int HTTPTransport::removeCookie(const string name)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HTTPTransport", "removeCookie", this, 1,
					TRACETYPE_STLSTRING, 0, ((void*)&name));	  /* AUTOINSERTED TRACE */
	#endif

     vector < std::pair < std::string,
     std::string > >::iterator currentCookie = m_vCookies.begin();
    while( currentCookie != m_vCookies.end())
    {
        if( strcmp( name.c_str(), (*currentCookie).first.c_str()) == 0)
        {
            m_vCookies.erase( currentCookie);
            	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "removeCookie", this, 1,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}

        }
        currentCookie++;
    }
    
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_FAIL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "removeCookie", this, 2,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_FAIL;
		#endif
	}

}

int HTTPTransport::removeAllCookies()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HTTPTransport", "removeAllCookies", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    m_vCookies.clear();
    // we also need to remove it from the header properties that we send.
    // This is done from the deleteTransportMethod before this one is called.
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "removeAllCookies", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}

}

int HTTPTransport::peekChunkLength( std::string& strNextChunk)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HTTPTransport", "peekChunkLength", this, 1,
					TRACETYPE_DATA, sizeof(std:: string&), ((void*)&strNextChunk));	  /* AUTOINSERTED TRACE */
	#endif

	if( strNextChunk.length() == 0)
	{
			{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (-1);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "peekChunkLength", this, 1,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return -1;
		#endif
	}

	}
	int iEndOfChunkSize = strNextChunk.find( ASCII_S_CRLF);
	if( iEndOfChunkSize == std::string::npos)
	{
			{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (-1);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "peekChunkLength", this, 2,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return -1;
		#endif
	}

	}
// Now get the size of the chunk from the data.  Look to see if there are any
// extensions - these are put in brackets so look for those.
	if( strNextChunk.substr( iEndOfChunkSize).find( ASCII_S_LEFTPAREN) != std::string::npos)
	{
		iEndOfChunkSize = strNextChunk.find( ASCII_S_LEFTPAREN);
	}
// Convert the hex string into the length of the chunk.
	char *	pszValue = (char *) strNextChunk.c_str();
		{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (axtoi( (char *) strNextChunk.substr( 0, iEndOfChunkSize).c_str()));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "peekChunkLength", this, 3,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return axtoi( (char *) strNextChunk.substr( 0, iEndOfChunkSize).c_str());
		#endif
	}

}

void HTTPTransport::trim(string& str)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("HTTPTransport", "trim", this, 1,
					TRACETYPE_STLSTRING, 0, ((void*)&str));	  /* AUTOINSERTED TRACE */
	#endif

    string::size_type pos = str.find_last_not_of(' ');
    if(pos != string::npos) 
    {
        str.erase(pos + 1);
        pos = str.find_first_not_of(' ');
        if(pos != string::npos) 
        {
            str.erase(0, pos);
        }
    }
    else 
    {
        str.erase(str.begin(), str.end());
    }
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("HTTPTransport", "trim", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

