#ifdef ENABLE_AXISTRACE
/* TRACE ADDED BY THE TRACE INSTRUMENTOR TOOL */
#include "../../common/AxisTrace.h"
#endif
// Copyright 2003-2004 The Apache Software Foundation.

// (c) Copyright IBM Corp. 2004, 2005 All Rights Reserved

// 

// Licensed under the Apache License, Version 2.0 (the "License");

// you may not use this file except in compliance with the License.

// You may obtain a copy of the License at

// 

//        http://www.apache.org/licenses/LICENSE-2.0

// 

// Unless required by applicable law or agreed to in writing, software

// distributed under the License is distributed on an "AS IS" BASIS,

// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.

// See the License for the specific language governing permissions and

// limitations under the License.

#if !defined(_AXIS_ICHANNEL_HPP)

#define _AXIS_ICHANNEL_HPP

#include <string>

#include <axis/GDefine.hpp>

#include "URL.hpp"

#include "HTTPTransportException.hpp"

const int BUF_SIZE = 1024 * 8;

using namespace std;

AXIS_CPP_NAMESPACE_USE

enum g_ChannelType 
{
	UnsecureChannel,
	SecureChannel,
	MaxChannelCount
};

class IChannel
{

public:
    virtual                     ~IChannel() {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("IChannel", "~IChannel", NULL, 0);	  /* AUTOINSERTED TRACE */
	#endif
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("IChannel", "~IChannel", NULL, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
};
	virtual const char*			getURL()=0;
	virtual void				setURL( const char* cpURL)=0;
    virtual URL &				getURLObject()=0;
    virtual bool				open() throw (HTTPTransportException&)=0;
    virtual bool				close()=0;
    virtual const std::string&	GetLastErrorMsg()=0;
    virtual const IChannel&		operator >> (char * msg)=0;
    virtual const IChannel&		operator << (const char * msg)=0;
	virtual int					setSecureProperties( const char *) {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("IChannel", "setSecureProperties", NULL, 1,
					TRACETYPE_ANONYMOUS, 0, NULL);	  /* AUTOINSERTED TRACE */
	#endif
	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (true);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("IChannel", "setSecureProperties", NULL, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return true;
		#endif
	}
};
	virtual const char *		getSecureProperties() {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("IChannel", "getSecureProperties", NULL, 0);	  /* AUTOINSERTED TRACE */
	#endif
	{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("IChannel", "getSecureProperties", NULL, 0,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}
};
    virtual void				setTimeout( long lSeconds)=0;
    virtual void				setSocket( unsigned int uiNewSocket)=0;
    virtual int					getSocket()=0;
	virtual bool				setTransportProperty( AXIS_TRANSPORT_INFORMATION_TYPE type, const char* value)=0;
	virtual const char *		getTransportProperty( AXIS_TRANSPORT_INFORMATION_TYPE type)=0;
	virtual void                setProxy(const char *pcProxyHost,unsigned int uiProxyPort) = 0;
	virtual bool				reopenRequired() throw() = 0;
	virtual void				closeQuietly( bool bNoExceptionOnForceClose) = 0;

};

#endif

