#ifdef ENABLE_AXISTRACE
/* TRACE ADDED BY THE TRACE INSTRUMENTOR TOOL */
#include "../../common/AxisTrace.h"
#endif
/*
 *   Copyright 2003-2004 The Apache Software Foundation.
// (c) Copyright IBM Corp. 2004, 2005 All Rights Reserved
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 */

/*
 *
 *
 * @author Lilantha Darshana (lilantha@virtusa.com)
 *
 */

#include "Platform.hpp"

#include "URL.hpp"

#include <ctype.h>

#include <iostream>

using namespace std;

URL::URL()
    : m_Protocol(unknown), m_Port(0), m_URL("")
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("URL", "URL", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

    m_Host = "";
    m_Path = "";
    m_Query = "";
    m_Password = "";
    m_User = "";
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("URL", "URL", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

URL::URL(std::string url)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("URL", "URL", this, 1,
					TRACETYPE_DATA, sizeof(std:: string), ((void*)&url));	  /* AUTOINSERTED TRACE */
	#endif

    m_URL = url;
    if(isascii((int)url.at(0)))
    {
        /* check this a valid URL */
        if((url.at(1) == (int)':') &&
            ((url.at(2) == (int)'/') || (url.at(2) == (int)'\\')))
            	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("URL", "URL", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}

        else /* assume valid URL hence, proceed with finding entries */
            ParseURL(url);
    }
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("URL", "URL", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

URL::~URL()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("URL", "~URL", this, 0);	  /* AUTOINSERTED TRACE */
	#endif


	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("URL", "~URL", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void URL::setProtocol(std::string prot)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("URL", "setProtocol", this, 1,
					TRACETYPE_DATA, sizeof(std:: string), ((void*)&prot));	  /* AUTOINSERTED TRACE */
	#endif

    m_Port = 0;
    if(prot == "http")
    {
        m_Protocol = http;
        m_Port = HTTP_PORT;
    }
    else if( prot == "file")
    {
        m_Protocol = file;
    }
    else if( prot == "ftp")
    {
        m_Protocol = ftp;
        m_Port = FTP_PORT;
    }
    else if( prot == "https")
    {
        m_Protocol = https;
        m_Port = HTTPS_PORT;
    }
    else
        m_Protocol = unknown;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("URL", "setProtocol", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void URL::setURL(const char* cpURL)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("URL", "setURL", this, 1,
					TRACETYPE_STRING, 0, ((void*)&cpURL));	  /* AUTOINSERTED TRACE */
	#endif

    m_URL = std::string(cpURL);
    if(isascii((int)m_URL.at(0)))
    {
        /* check this a valid URL */
        if((m_URL.at(1) == (int)':') &&
            ((m_URL.at(2) == (int)'/') || (m_URL.at(2) == (int)'\\')))
            	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("URL", "setURL", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}

        else /* assume valid URL hence, proceed with finding entries */
            ParseURL(m_URL);
    }
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("URL", "setURL", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

const char* URL::getURL()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("URL", "getURL", this, 0);	  /* AUTOINSERTED TRACE */
	#endif
 
    if (m_URL.length() > 0 )
        	{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (m_URL.c_str());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("URL", "getURL", this, 1,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_URL.c_str();
		#endif
	}

    else
        	{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (NULL);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("URL", "getURL", this, 2,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return NULL;
		#endif
	}

}

void URL::ParseURL(std::string url)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("URL", "ParseURL", this, 1,
					TRACETYPE_DATA, sizeof(std:: string), ((void*)&url));	  /* AUTOINSERTED TRACE */
	#endif

    std::string::size_type begpos, pos;
    /* try to find out the protocol */
    if((pos = url.find("://")) != std::string::npos)
    {
        setProtocol(url.substr(0, pos));
        /* find m_Host name */
        if(m_Protocol != unknown)
        {
            url = url.substr(pos + 3); /* rest of the URL string */
            begpos = pos = 0;
            std::string key(":/?");
            
            while((pos = url.find_first_of(key, begpos)))
            {
                if(pos == std::string::npos) /* only host name found */
                {
                    //if(m_Host.empty())
                        m_Host = url;
                    if (key == "?") /* found path */
                        m_Path = url.substr(begpos - 1);
                    break;
                }
                else
                    if(pos == 0) break;
                switch(url.at(pos))
                {
                    case ':': 
                        //if(m_Host.empty())
                            m_Host = url.substr(begpos, pos - begpos);
                        pos++;
                        begpos = pos;
                        key = "/?"; 
                        /* scan for the rest to get the path & query */
                        continue;
                    case '/':
                        if (key == "/?") /* found port number */
                        {
                            m_Port = atoi(url.substr(begpos, 
                                pos - begpos + 1).c_str());
                            //if(m_Host.empty())
                                m_Host = url.substr(0, begpos - 1);
                        }
                        else
                            m_Host = url.substr(0, pos);
                        pos++;
                        begpos = pos;
                        /*
                        key = "?";
                        continue;
                      case '?':
                        //not correctly supported
                        m_Query = url.substr(pos);                        
                        */
                        /* no need to find other ? */
                        m_Path = url.substr(begpos - 1);
                        break;
                }    
                break;
            }                            
        }
    }
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("URL", "ParseURL", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}
 
                    
                    
