#ifdef ENABLE_AXISTRACE
/* TRACE ADDED BY THE TRACE INSTRUMENTOR TOOL */
#include "../../common/AxisTrace.h"
#endif
/*
 *   Copyright 2003-2004 The Apache Software Foundation.
// (c) Copyright IBM Corp. 2004, 2005 All Rights Reserved
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 */

/*
 *
 *
 * @author Lilantha Darshana (lilantha@virtusa.com)
 *
 */

#if !defined(_AXIS_URL_HPP)

#define _AXIS_URL_HPP

#include <string>

/* Welknown ports */

const unsigned short HTTP_PORT  = 80;

const unsigned short HTTPS_PORT = 443;

const unsigned short FTP_PORT   = 21;

using namespace std;

/*
 *    Implementation of URL to manupulate URLs.  
 *
 *    This implementation only supports subset of a URL
 *    note that # references, userinfo query string 
 *    processing are not supported for this version.
 *
 *    URLs are of the form:
 * 
 *    URL              = protocol "://" server "/" [path]["?" query]
 *    server        = [userinfo "@"] hostname-port
 *    hostname-port = hostname [ ":" port ]
 *    userinfo      = user[:password]
 *
 *
 *    @brief    Manupulate URLs
 */

class URL  
{

public:
    enum Protocol { http, https, ftp, file, unknown}; 
    /* for our purpose currently we need http, https only. This is provided
     * To make extensible to support other transports for RPC but file????, 
     * yes we may require pipes; with web-service????
     */

public:
    URL();
    URL(std::string url);
    ~URL();
    void setProtocol(std::string prot);
    void setProtocol(Protocol prot){
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("URL", "setProtocol", NULL, 1,
					TRACETYPE_DATA, sizeof(Protocol), ((void*)&prot));	  /* AUTOINSERTED TRACE */
	#endif
m_Protocol = prot;	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("URL", "setProtocol", NULL, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}
    void setHostName(std::string host){
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("URL", "setHostName", NULL, 1,
					TRACETYPE_DATA, sizeof(std:: string), ((void*)&host));	  /* AUTOINSERTED TRACE */
	#endif
m_Host= host;	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("URL", "setHostName", NULL, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}
    void setResource(std::string path){
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("URL", "setResource", NULL, 1,
					TRACETYPE_DATA, sizeof(std:: string), ((void*)&path));	  /* AUTOINSERTED TRACE */
	#endif
m_Path = path;	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("URL", "setResource", NULL, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}
    void setPort(unsigned short port){
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("URL", "setPort", NULL, 1,
					TRACETYPE_USHORT, 0, ((void*)&port));	  /* AUTOINSERTED TRACE */
	#endif
m_Port = port;	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("URL", "setPort", NULL, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}
    Protocol    getProtocol(){
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("URL", "getProtocol", NULL, 0);	  /* AUTOINSERTED TRACE */
	#endif
	{
		#ifdef ENABLE_AXISTRACE
			Protocol traceRet = (m_Protocol);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("URL", "getProtocol", NULL, 0,
					TRACETYPE_DATA, sizeof(Protocol), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_Protocol;
		#endif
	}
}
    const char* getHostName(){
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("URL", "getHostName", NULL, 0);	  /* AUTOINSERTED TRACE */
	#endif
	{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (m_Host.c_str());
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("URL", "getHostName", NULL, 0,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_Host.c_str();
		#endif
	}
}
    std::string getResource(){
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("URL", "getResource", NULL, 0);	  /* AUTOINSERTED TRACE */
	#endif
	{
		#ifdef ENABLE_AXISTRACE
			std:: string traceRet = (m_Path);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("URL", "getResource", NULL, 0,
					TRACETYPE_DATA, sizeof(std:: string), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_Path;
		#endif
	}
}
    void setURL(const char* cpURL);
    const char* getURL(); 
    unsigned short getPort(){
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("URL", "getPort", NULL, 0);	  /* AUTOINSERTED TRACE */
	#endif
	{
		#ifdef ENABLE_AXISTRACE
			unsigned short traceRet = (m_Port);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("URL", "getPort", NULL, 0,
					TRACETYPE_USHORT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return m_Port;
		#endif
	}
}
    // other functions are not supported yet

private:
    void ParseURL(std::string url);
    
    Protocol        m_Protocol;
    std::string     m_Host;
    unsigned short  m_Port;
    std::string     m_Path;
    std::string     m_Query;
    std::string m_Password;
    std::string m_User;
    std::string m_URL;

};

#endif

