#ifdef ENABLE_AXISTRACE
/* TRACE ADDED BY THE TRACE INSTRUMENTOR TOOL */
#include "../../common/AxisTrace.h"
#endif
/*
 *   Copyright 2003-2004 The Apache Software Foundation.
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 *
 *  @author Dinesh Premalal (xydinesh@gmail.com
 *			     premalal@opensource.lk)
 *
 */

#include "XMLParserGuththila.hpp"

#include "AxisInputStream.hpp"

XMLParserGuththila::XMLParserGuththila ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XMLParserGuththila", "XMLParserGuththila", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

  parser = 0;
  stream = 0;
  element.m_pchNamespace = 0;
  element.m_pchNameOrValue = 0;
  element.m_pchAttributes[0] = 0;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XMLParserGuththila", "XMLParserGuththila", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

/* This constructor used only for Testing purpose*/

XMLParserGuththila::XMLParserGuththila (FileInputStream *fstr)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XMLParserGuththila", "XMLParserGuththila", this, 1,
					TRACETYPE_POINTER, sizeof(FileInputStream), ((void*)&fstr));	  /* AUTOINSERTED TRACE */
	#endif

  parser = new XmlPullParser (fstr);
  stream = 0;
  element.m_pchNamespace = 0;
  element.m_pchNameOrValue = 0;
  element.m_pchAttributes[0] = 0;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XMLParserGuththila", "XMLParserGuththila", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

XMLParserGuththila::~XMLParserGuththila ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XMLParserGuththila", "~XMLParserGuththila", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

  if (parser)
    delete parser;
  if (stream)
    delete stream;
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XMLParserGuththila", "~XMLParserGuththila", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

int
XMLParserGuththila::setInputStream (AxisIOStream* pInputStream)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XMLParserGuththila", "setInputStream", this, 1,
					TRACETYPE_POINTER, sizeof(AxisIOStream), ((void*)&pInputStream));	  /* AUTOINSERTED TRACE */
	#endif

  try{
    if (stream)
      delete stream;
    stream = new AxisInputStream (pInputStream);
    
    if (parser)
      delete parser;
    
    parser = new XmlPullParser(stream);
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (AXIS_SUCCESS);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XMLParserGuththila", "setInputStream", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return AXIS_SUCCESS;
		#endif
	}

  }
  catch (XmlPullParserException* e)
    {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceCatch("XMLParserGuththila", "setInputStream", this, 0,
					TRACETYPE_POINTER, sizeof(XmlPullParserException), ((void*)&e));	  /* AUTOINSERTED TRACE */
	#endif

      delete e;
      throw new AxisParseException (245, "set InputStream");
    }
}

const XML_Ch*
XMLParserGuththila::getNS4Prefix (const XML_Ch* pcPrefix)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XMLParserGuththila", "getNS4Prefix", this, 1,
					TRACETYPE_STRING, 0, ((void*)&pcPrefix));	  /* AUTOINSERTED TRACE */
	#endif

  	{
		#ifdef ENABLE_AXISTRACE
			const XML_Ch* traceRet = (parser->getNamespaceForPrefix ((char *)pcPrefix));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XMLParserGuththila", "getNS4Prefix", this, 0,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return parser->getNamespaceForPrefix ((char *)pcPrefix);
		#endif
	}

}

const XML_Ch*
XMLParserGuththila::getPrefix4NS (const XML_Ch* pcNS)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XMLParserGuththila", "getPrefix4NS", this, 1,
					TRACETYPE_STRING, 0, ((void*)&pcNS));	  /* AUTOINSERTED TRACE */
	#endif

  	{
		#ifdef ENABLE_AXISTRACE
			const XML_Ch* traceRet = (parser->getPrefixForNamespace ((char *)pcNS));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XMLParserGuththila", "getPrefix4NS", this, 0,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return parser->getPrefixForNamespace ((char *)pcNS);
		#endif
	}

}

int 
XMLParserGuththila::getStatus ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XMLParserGuththila", "getStatus", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

  	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (0);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XMLParserGuththila", "getStatus", this, 0,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return 0;
		#endif
	}

}

void
XMLParserGuththila::setNamespace ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XMLParserGuththila", "setNamespace", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

  int d, i, j;
  char *x = parser->getPrefix ();
  if (!x)
    {
      d = parser->getDepth ();
      if (d > 0)
	{
	  i = parser->getNamespaceCount (d);
	  if (d > 1)
	    j = parser->getNamespaceCount (d-1);
	  else 
	j = 0;
	  for (; j < i; j++)
	    element.m_pchNamespace = parser->getNamespaceUri (j);
	}
    }
  else
    element.m_pchNamespace = parser->getNamespaceForPrefix (x);
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XMLParserGuththila", "setNamespace", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void
XMLParserGuththila::setAttributes ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XMLParserGuththila", "setAttributes", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

  int count;
  count = parser->getAttributeCount ();
  element.m_pchAttributes[count*3];
  for (int ix = 0; ix<count; ix++)
    {
      int iAttx = ix*3;
      element.m_pchAttributes[iAttx] = parser->getAttributeName (ix);
      element.m_pchAttributes[iAttx+1] = parser->getAttributePrefix (ix);
      if (element.m_pchAttributes[iAttx+1])
	element.m_pchAttributes[iAttx+1] = parser->getNamespaceForPrefix ((char*)element.m_pchAttributes[iAttx+1]);
      element.m_pchAttributes[iAttx+2] = parser->getAttributeValue (ix);
    }
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XMLParserGuththila", "setAttributes", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void 
XMLParserGuththila::freeAttributes ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XMLParserGuththila", "freeAttributes", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

  for (int ix = 0; element.m_pchAttributes[ix]; ix +=3)
    {
      element.m_pchAttributes[ix] = NULL;
      if (element.m_pchAttributes[ix+1])
	element.m_pchAttributes[ix+1] = NULL;
      if (element.m_pchAttributes[ix+2])
	element.m_pchAttributes[ix+2] = NULL;
    }
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XMLParserGuththila", "freeAttributes", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

void
XMLParserGuththila::freeElements ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XMLParserGuththila", "freeElements", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

  if (element.m_pchNameOrValue)
    element.m_pchNameOrValue = NULL;
  if (element.m_pchNamespace)
    element.m_pchNamespace = NULL;
  freeAttributes ();
	{
		#ifdef ENABLE_AXISTRACE
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XMLParserGuththila", "freeElements", this, 0);	  /* AUTOINSERTED TRACE */
		#endif
		return;
	}
}

int 
XMLParserGuththila::isSpace (char *c)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XMLParserGuththila", "isSpace", this, 1,
					TRACETYPE_STRING, 0, ((void*)&c));	  /* AUTOINSERTED TRACE */
	#endif

  char *ic;
  ic = c;
  if (ic[0] == 0xa)
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (1);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XMLParserGuththila", "isSpace", this, 1,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return 1;
		#endif
	}

  else
    	{
		#ifdef ENABLE_AXISTRACE
			int traceRet = (0);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XMLParserGuththila", "isSpace", this, 2,
					TRACETYPE_INT, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return 0;
		#endif
	}

}

const AnyElement*
XMLParserGuththila::next (bool isCharData)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XMLParserGuththila", "next", this, 1,
					TRACETYPE_BOOL, 0, ((void*)&isCharData));	  /* AUTOINSERTED TRACE */
	#endif

  int nValue;
  freeElements ();
  try 
    {
      while (true)
	{
	start:
	  if ((nValue = parser->next ()) != -1)
	    {
	      switch (nValue)
		{
		case XmlPullParser::XMLDecl:
		  goto start;
		case XmlPullParser::STag:
		  element.m_type = START_ELEMENT;
		  element.m_type2 = START_ELEMENT;
		  element.m_pchNameOrValue = parser->getName ();
		  setNamespace ();
		  setAttributes ();
		  break;
		case XmlPullParser::EmptyElemTag:
		  element.m_type = START_ELEMENT;
		  element.m_type2 = END_ELEMENT;
		  element.m_pchNameOrValue = parser->getName ();
		  setNamespace ();
		  setAttributes ();
		  break;
		case XmlPullParser::ETag:
		  element.m_type = END_ELEMENT;
		  element.m_type2 = END_ELEMENT;
		  element.m_pchNameOrValue = parser->getName ();
		  setNamespace ();
		  break;
		case XmlPullParser::Content:
		  if (isCharData)
		    {
		      element.m_type = CHARACTER_ELEMENT;
		      element.m_type2 = CHARACTER_ELEMENT;
		      element.m_pchNameOrValue = parser->getValue ();
		    }
		  else
		    goto start;
		  
		  break;
		default:;
		}
	    }
	  	{
		#ifdef ENABLE_AXISTRACE
			const AnyElement* traceRet = (&element);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XMLParserGuththila", "next", this, 0,
					TRACETYPE_POINTER, sizeof(AnyElement), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return &element;
		#endif
	}

	}
    }
  catch (XmlPullParserException *e)
    {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceCatch("XMLParserGuththila", "next", this, 0,
					TRACETYPE_POINTER, sizeof(XmlPullParserException), ((void*)&e));	  /* AUTOINSERTED TRACE */
	#endif

      delete e;
      throw new AxisParseException (245, "next Method XMLParserGuththila");
    }
}

const AnyElement*
XMLParserGuththila::anyNext ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XMLParserGuththila", "anyNext", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

  int nValue;
  freeElements ();
  try 
    {
      while (true)
	{
	  if ((nValue = parser->next ()) != -1)
	    {
	      switch (nValue)
		{
		case XmlPullParser::STag:
		  element.m_type = START_ELEMENT;
		  element.m_type2 = START_ELEMENT;
		  element.m_pchNameOrValue = parser->getName ();
		  setNamespace ();
		  setAttributes ();
		  break;
		case XmlPullParser::EmptyElemTag:
		  element.m_type = START_ELEMENT;
		  element.m_type2 = END_ELEMENT;
		  element.m_pchNameOrValue = parser->getName ();
		  setNamespace ();
		  setAttributes ();
		  break;
		case XmlPullParser::ETag:
		  element.m_type = END_ELEMENT;
		  element.m_type2 = END_ELEMENT;
		  element.m_pchNameOrValue = parser->getName ();
		  setNamespace ();
		  break;
		case XmlPullParser::Content:
		  element.m_type = CHARACTER_ELEMENT;
		  element.m_type2 = CHARACTER_ELEMENT;
		  element.m_pchNameOrValue = parser->getValue ();
		  break;
		default:;
		}
	    }
	  	{
		#ifdef ENABLE_AXISTRACE
			const AnyElement* traceRet = (&element);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XMLParserGuththila", "anyNext", this, 0,
					TRACETYPE_POINTER, sizeof(AnyElement), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return &element;
		#endif
	}

	}
    }
  catch (XmlPullParserException *e)
    {
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceCatch("XMLParserGuththila", "anyNext", this, 0,
					TRACETYPE_POINTER, sizeof(XmlPullParserException), ((void*)&e));	  /* AUTOINSERTED TRACE */
	#endif

      delete e;
      throw new AxisParseException (245, "any next method XMLParserGuththila");
    }
}

const char*
XMLParserGuththila::peek ()
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry("XMLParserGuththila", "peek", this, 0);	  /* AUTOINSERTED TRACE */
	#endif

  int peekValue;
  const char* peekName;
  peekValue = parser->next ();
  while ( peekValue == XmlPullParser::Content || peekValue == XmlPullParser::XMLDecl)
      peekValue = parser->next ();
    peekName = parser->getName ();
    m_pchPeek = true;
    	{
		#ifdef ENABLE_AXISTRACE
			const char* traceRet = (peekName);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit("XMLParserGuththila", "peek", this, 0,
					TRACETYPE_STRING, 0, ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return peekName;
		#endif
	}

}

