#ifdef ENABLE_AXISTRACE
/* TRACE ADDED BY THE TRACE INSTRUMENTOR TOOL */
#include "../../../common/AxisTrace.h"
#endif
/*
 *   Copyright 2003-2004 The Apache Software Foundation.
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 */

#include "spp.h"

#ifndef PREFIX

#define PREFIX(ident) ident

#endif

static int PTRCALL PREFIX(add_ptr)(char *ptr, TokDataStruct *data)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, ")", NULL, 2,
					TRACETYPE_STRING, 0, ((void*)&ptr),
					TRACETYPE_POINTER, sizeof(TokDataStruct), ((void*)&data));	  /* AUTOINSERTED TRACE */
	#endif

    if (data->numOfPtrs == data->ptrBuffSize
        || !data->ptrBuff) 
    {
        int sz = data->ptrBuffSize << 1;
        char **ptrBuff = (char **)malloc(sz << 2);
        if (!ptrBuff)
            	{
		#ifdef ENABLE_AXISTRACE
			int PTRCALL PREFIX ( add_ptr traceRet = (SPP_ERROR_NO_MEMORY);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, ")", NULL, 1,
					TRACETYPE_DATA, sizeof(int PTRCALL PREFIX ( add_ptr), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return SPP_ERROR_NO_MEMORY;
		#endif
	}

        if (data->ptrBuff) 
        {
            memmove(ptrBuff, data->ptrBuff, data->numOfPtrs << 2);
            free(data->ptrBuff);
        }
        data->ptrBuff = ptrBuff;
        data->ptrBuffSize = sz;
        /* printf("sz:%d\n", sz); */
    }
    data->ptrBuff[data->numOfPtrs++] = ptr;
    
    	{
		#ifdef ENABLE_AXISTRACE
			int PTRCALL PREFIX ( add_ptr traceRet = (SPP_ERROR_NONE);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, ")", NULL, 2,
					TRACETYPE_DATA, sizeof(int PTRCALL PREFIX ( add_ptr), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return SPP_ERROR_NONE;
		#endif
	}

}

static int PTRCALL
PREFIX(is_letter)(int byteType)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, ")", NULL, 1,
					TRACETYPE_INT, 0, ((void*)&byteType));	  /* AUTOINSERTED TRACE */
	#endif

    if(BT_NMSTRT == byteType || BT_HEX == byteType)
        	{
		#ifdef ENABLE_AXISTRACE
			int PTRCALL PREFIX ( is_letter traceRet = (-1);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, ")", NULL, 1,
					TRACETYPE_DATA, sizeof(int PTRCALL PREFIX ( is_letter), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return -1;
		#endif
	}

    else 
        	{
		#ifdef ENABLE_AXISTRACE
			int PTRCALL PREFIX ( is_letter traceRet = (0);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, ")", NULL, 2,
					TRACETYPE_DATA, sizeof(int PTRCALL PREFIX ( is_letter), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return 0;
		#endif
	}

    
}

static int PTRCALL
PREFIX(is_name_start_char)(int byteType)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, ")", NULL, 1,
					TRACETYPE_INT, 0, ((void*)&byteType));	  /* AUTOINSERTED TRACE */
	#endif

    if(PREFIX(is_letter)(byteType) || BT_COLON == byteType)
        	{
		#ifdef ENABLE_AXISTRACE
			int PTRCALL PREFIX ( is_name_start_char traceRet = (-1);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, ")", NULL, 1,
					TRACETYPE_DATA, sizeof(int PTRCALL PREFIX ( is_name_start_char), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return -1;
		#endif
	}

    else
        	{
		#ifdef ENABLE_AXISTRACE
			int PTRCALL PREFIX ( is_name_start_char traceRet = (0);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, ")", NULL, 2,
					TRACETYPE_DATA, sizeof(int PTRCALL PREFIX ( is_name_start_char), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return 0;
		#endif
	}

}

static int PTRCALL
PREFIX(is_name_char)(int byteType)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, ")", NULL, 1,
					TRACETYPE_INT, 0, ((void*)&byteType));	  /* AUTOINSERTED TRACE */
	#endif

    if(PREFIX(is_letter)(byteType) || BT_DIGIT == byteType || 
        BT_COLON == byteType || BT_MINUS == byteType ||
         BT_NAME == byteType)
        	{
		#ifdef ENABLE_AXISTRACE
			int PTRCALL PREFIX ( is_name_char traceRet = (-1);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, ")", NULL, 1,
					TRACETYPE_DATA, sizeof(int PTRCALL PREFIX ( is_name_char), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return -1;
		#endif
	}

    else 	{
		#ifdef ENABLE_AXISTRACE
			int PTRCALL PREFIX ( is_name_char traceRet = (0);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, ")", NULL, 2,
					TRACETYPE_DATA, sizeof(int PTRCALL PREFIX ( is_name_char), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return 0;
		#endif
	}

}

static int PTRCALL
PREFIX(doHomeWork)(TokDataStruct* data, int* parserState, char** ptr, int* numOfChars)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, ")", NULL, 4,
					TRACETYPE_POINTER, sizeof(TokDataStruct), ((void*)&data),
					TRACETYPE_POINTER, sizeof(int), ((void*)&parserState),
					TRACETYPE_POINTER, sizeof(char*), ((void*)&ptr),
					TRACETYPE_POINTER, sizeof(int), ((void*)&numOfChars));	  /* AUTOINSERTED TRACE */
	#endif

    numOfChars -= MINBPC(enc);
    *ptr += MINBPC(enc);    
    data->numOfPtrs = 0;
    *parserState = S_0;
    	{
		#ifdef ENABLE_AXISTRACE
			int PTRCALL PREFIX ( doHomeWork traceRet = (SPP_ERROR_NONE);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, ")", NULL, 0,
					TRACETYPE_DATA, sizeof(int PTRCALL PREFIX ( doHomeWork), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return SPP_ERROR_NONE;
		#endif
	}

}

static int PTRCALL
PREFIX(tokenizeProlog)(int* parserState, TokDataStruct* data, const ENCODING *enc,
                       int *numOfChars, char* end, const char **ptr)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, ")", NULL, 6,
					TRACETYPE_POINTER, sizeof(int), ((void*)&parserState),
					TRACETYPE_POINTER, sizeof(TokDataStruct), ((void*)&data),
					TRACETYPE_POINTER, sizeof(ENCODING), ((void*)&enc),
					TRACETYPE_POINTER, sizeof(int), ((void*)&numOfChars),
					TRACETYPE_STRING, 0, ((void*)&end),
					TRACETYPE_POINTER, sizeof(char*), ((void*)&ptr));	  /* AUTOINSERTED TRACE */
	#endif
 
    /* printf("*numOfChars:%d\n", *numOfChars); */
    int byteType;
    while(*numOfChars >= MINBPC(enc))
    {
        byteType = BYTE_TYPE(enc, *ptr);
        /* printf("*ptr:%s\n", *ptr); */
        /* printf("byteType:%d\n", byteType); */
        /* printf("*parserState:%d\n", *parserState); */
        switch (*parserState) 
        {
            case S_0:
                if(BT_LT == byteType) 
                {
                    *parserState = S_13;
                }
                else
                    data->type = PCDATA;
                
                break;
        
            case S_1:
                if(BT_QUEST == byteType)
                {
                    data->type = END_TAG;
                    *parserState = S_10;
                }
                else if (PREFIX(is_name_start_char)(byteType)) 
                {
                    data->type = START_TAG;
                    *parserState = S_2;
                }
                else
                    *parserState = S_14;
                
                break;
        
            case S_2:
                if (BT_GT == byteType) 
                {
                    
                    *parserState = S_0;
                }
                else if (BT_QUEST == byteType) 
                {
                    *parserState = S_9;
                }
                else if (BT_S == byteType) 
                {
                    *parserState = S_3;
                }    
                else if (!PREFIX(is_name_char)(byteType))
                    *parserState = S_14;
                break;
        
            case S_3:
                if (BT_GT == byteType) 
                {
                        PREFIX(doHomeWork)(data, parserState, ptr, numOfChars);
                        	{
		#ifdef ENABLE_AXISTRACE
			int PTRCALL PREFIX ( tokenizeProlog traceRet = (SPP_ERROR_NONE);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, ")", NULL, 1,
					TRACETYPE_DATA, sizeof(int PTRCALL PREFIX ( tokenizeProlog), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return SPP_ERROR_NONE;
		#endif
	}

                    
                }
                else if (BT_QUEST == byteType)
                    *parserState = S_9;
                else if (PREFIX(is_name_start_char)(byteType)) 
                {
                    *parserState = S_4;
                }    
                else if (BT_S != byteType)
                    *parserState = S_14;
                break;
            
            case S_4:
                if (BT_EQUALS == byteType) 
                {
                    *parserState = S_6;
                }
                else if (BT_S == byteType) 
                {
                    *parserState = S_5;
                }
                else if (!PREFIX(is_name_char)(byteType))
                    *parserState = S_14;
                break;
        
            case S_5:
                if (BT_EQUALS == byteType)
                    *parserState = S_6;
                else if (BT_S != byteType)
                    *parserState = S_14;
                break;
        
            case S_6:
                if (BT_APOS == byteType) 
                {
                    *parserState = S_8;
                }
                else if (BT_QUOT == byteType) 
                {
                    *parserState = S_7;
                }
                else if (BT_S != byteType)
                    *parserState = S_14;
                break;
        
            case S_7:
                if (BT_QUOT == byteType) 
                {
                    *parserState = S_3;
                }
                else if(BT_LT == byteType || BT_AMP == byteType)
                {                    
                    *parserState = S_14;
                    /* some problem exists. fix! */
                }
                break;
            
            case S_8:
                if (BT_APOS == byteType) 
                {
                    *parserState = S_3;
                }
                else if (BT_LT == byteType || BT_AMP == byteType)
                    *parserState = S_14;
                break;
            
            case S_9:
                data->type = EMPTY_ELEMENT_TAG;
                
                if (BT_GT == byteType) 
                {
                    PREFIX(doHomeWork)(data, parserState, ptr, numOfChars);
                    	{
		#ifdef ENABLE_AXISTRACE
			int PTRCALL PREFIX ( tokenizeProlog traceRet = (SPP_ERROR_NONE);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, ")", NULL, 2,
					TRACETYPE_DATA, sizeof(int PTRCALL PREFIX ( tokenizeProlog), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return SPP_ERROR_NONE;
		#endif
	}
    
                }
                else
                    *parserState = S_14;
                break;
        
            case S_10:
                if (PREFIX(is_name_start_char)(byteType)) 
                {
                    *parserState = S_11;
                }
                else
                    *parserState = S_14;
                break;
        
            case S_11:
                if (BT_GT == byteType) 
                {
                    *parserState = S_0;
                }
                else if (BT_S == byteType) 
                {
                    *parserState = S_12;
                }
                else if (!PREFIX(is_name_char)(byteType))
                    *parserState = S_14;
                break;
        
            case S_12:
                if (BT_GT == byteType) 
                {
                    *parserState = S_0;
                }
                else if (BT_S != byteType)
                    *parserState = S_14;
                break;
        
            case S_13:
                if(BT_QUEST == byteType)
                    *parserState = S_1;
                else
                    *parserState = S_14;
                break;
            case S_14:
                printf("some problem exists. fix it!\n");
    
            default:
                
                	{
		#ifdef ENABLE_AXISTRACE
			int PTRCALL PREFIX ( tokenizeProlog traceRet = (-(*parserState));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, ")", NULL, 3,
					TRACETYPE_DATA, sizeof(int PTRCALL PREFIX ( tokenizeProlog), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return -(*parserState);
		#endif
	}

            }
            *numOfChars -= MINBPC(enc);
            *ptr += MINBPC(enc);
        }
        	{
		#ifdef ENABLE_AXISTRACE
			int PTRCALL PREFIX ( tokenizeProlog traceRet = (SPP_ERROR_TOKENIZER_FAILED);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, ")", NULL, 4,
					TRACETYPE_DATA, sizeof(int PTRCALL PREFIX ( tokenizeProlog), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return SPP_ERROR_TOKENIZER_FAILED;
		#endif
	}

}

static int PTRCALL
PREFIX(tokenizeContent)(int* parserState, TokDataStruct* data, const ENCODING *enc,
                        int *numOfChars, char* end, const char **ptr)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, ")", NULL, 6,
					TRACETYPE_POINTER, sizeof(int), ((void*)&parserState),
					TRACETYPE_POINTER, sizeof(TokDataStruct), ((void*)&data),
					TRACETYPE_POINTER, sizeof(ENCODING), ((void*)&enc),
					TRACETYPE_POINTER, sizeof(int), ((void*)&numOfChars),
					TRACETYPE_STRING, 0, ((void*)&end),
					TRACETYPE_POINTER, sizeof(char*), ((void*)&ptr));	  /* AUTOINSERTED TRACE */
	#endif
 
    /* printf("*numOfChars:%d\n", *numOfChars); */
    int byteType;
    while(*numOfChars >= MINBPC(enc))
    {
        byteType = BYTE_TYPE(enc, *ptr);
        /*printf("*ptr:%s\n", *ptr);*/
        /* printf("byteType:%d\n", byteType); */
        /* printf("*parserState:%d\n", *parserState); */
        switch (*parserState) 
        {
            case S_0:
                /* if ('<' == ch) { */
                if(BT_LT == byteType) 
                {
                    if (data->numOfPtrs) 
                    {
                        PREFIX(add_ptr)(*ptr - 1, data);
                        	{
		#ifdef ENABLE_AXISTRACE
			int PTRCALL PREFIX ( tokenizeContent traceRet = (SPP_ERROR_NONE);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, ")", NULL, 1,
					TRACETYPE_DATA, sizeof(int PTRCALL PREFIX ( tokenizeContent), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return SPP_ERROR_NONE;
		#endif
	}

                    }
                    
                    *parserState = S_1;
                }
                else
                if (!data->numOfPtrs) 
                {
                    data->type = PCDATA;
                    PREFIX(add_ptr)(*ptr, data); 
                }
                
                break;
        
            case S_1:
                /* if ('/' == ch) { */
                if(BT_SOL == byteType)
                {
                    data->type = END_TAG;
                    
                    *parserState = S_10;
                }
                else if (PREFIX(is_name_start_char)(byteType)) 
                {
                    data->type = START_TAG;
                    PREFIX(add_ptr)(*ptr, data);
                    
                    *parserState = S_2;
                }
                else
                    *parserState = S_13;
                
                break;
        
            case S_2:
                if (BT_GT == byteType) 
                {
                    if (data->numOfPtrs) 
                    {
                        PREFIX(add_ptr)(*ptr - 1, data);
                        	{
		#ifdef ENABLE_AXISTRACE
			int PTRCALL PREFIX ( tokenizeContent traceRet = (SPP_ERROR_NONE);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, ")", NULL, 2,
					TRACETYPE_DATA, sizeof(int PTRCALL PREFIX ( tokenizeContent), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return SPP_ERROR_NONE;
		#endif
	}

                    }
                    
                    *parserState = S_0;
                }
                else if (BT_SOL == byteType) 
                {
                    PREFIX(add_ptr)(*ptr - 1, data);
                    
                    *parserState = S_9;
                }
                /* else if (is_white_space(ch)) { */
                else if (BT_S == byteType) 
                {
                    PREFIX(add_ptr)(*ptr - 1, data);
                    
                    *parserState = S_3;
                }    
                /* else if (!is_name_char(ch)) */
                else if (!PREFIX(is_name_char)(byteType))
                    *parserState = S_13;
                break;
        
            case S_3:
                if (BT_GT == byteType) 
                {
                    if (data->numOfPtrs)
                        	{
		#ifdef ENABLE_AXISTRACE
			int PTRCALL PREFIX ( tokenizeContent traceRet = (SPP_ERROR_NONE);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, ")", NULL, 3,
					TRACETYPE_DATA, sizeof(int PTRCALL PREFIX ( tokenizeContent), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return SPP_ERROR_NONE;
		#endif
	}

                    
                    *parserState = S_0;
                }
                /* else if ('/' == ch) */
                else if (BT_SOL == byteType)
                    *parserState = S_9;
                else if (PREFIX(is_name_start_char)(byteType)) 
                {
                    PREFIX(add_ptr)(*ptr, data);
                    
                    *parserState = S_4;
                }    
                else if (BT_S != byteType)
                    *parserState = S_13;
                break;
            
            case S_4:
                /* if ('=' == ch) { */
                if (BT_EQUALS == byteType) 
                {
                    PREFIX(add_ptr)(*ptr - 1, data);
                    
                    *parserState = S_6;
                }
                else if (BT_S == byteType) 
                {
                    PREFIX(add_ptr)(*ptr - 1, data);
                    
                    *parserState = S_5;
                }
                else if (!PREFIX(is_name_char)(byteType))
                    *parserState = S_13;
                break;
        
            case S_5:
                /* if ('=' == ch) */
                if (BT_EQUALS == byteType)
                    *parserState = S_6;
                else if (BT_S != byteType)
                    *parserState = S_13;
                break;
        
            case S_6:
                /* if ('\'' == ch) { */
                if (BT_APOS == byteType) 
                {
                    PREFIX(add_ptr)(*ptr, data);
                    
                    *parserState = S_8;
                }
                /* else if ('"' == ch) { */
                else if (BT_QUOT == byteType) 
                {
                    PREFIX(add_ptr)(*ptr, data);
                    
                    *parserState = S_7;
                }
                else if (BT_S != byteType)
                    *parserState = S_13;
                break;
        
            case S_7:
                /* if ('"' == ch) { */
                if (BT_QUOT == byteType) 
                {
                    PREFIX(add_ptr)(*ptr, data);
                    
                    *parserState = S_3;
                }
                /* else if ('<' == ch || '&' == ch) */
                else if (BT_LT == byteType || BT_AMP == byteType)
                    *parserState = S_13;
                /* some problem exists. fix it!. */
                break;
            
            case S_8:
                /* if ('\'' == byteType) { */
                if (BT_APOS == byteType) 
                {
                    PREFIX(add_ptr)(*ptr, data);
                
                    *parserState = S_3;
                }
                /* else if ('<' == ch || '&' == ch) */
                else if (BT_LT == byteType || BT_AMP == byteType)
                    *parserState = S_13;
                break;
            
            case S_9:
                data->type = EMPTY_ELEMENT_TAG;
                
                /* if ('>' == ch) { */
                if (BT_GT == byteType) 
                {
                    if (data->numOfPtrs)
                        	{
		#ifdef ENABLE_AXISTRACE
			int PTRCALL PREFIX ( tokenizeContent traceRet = (SPP_ERROR_NONE);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, ")", NULL, 4,
					TRACETYPE_DATA, sizeof(int PTRCALL PREFIX ( tokenizeContent), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return SPP_ERROR_NONE;
		#endif
	}

                    
                    *parserState = S_0;
                }
                else
                    *parserState = S_13;
                break;
        
            case S_10:
                if (PREFIX(is_name_start_char)(byteType)) 
                {
                    PREFIX(add_ptr)(*ptr, data);
                    
                    *parserState = S_11;
                }
                else
                    *parserState = S_13;
                break;
        
            case S_11:
                /* if ('>' == ch) { */
                if (BT_GT == byteType) 
                {
                    if (data->numOfPtrs) 
                    {
                        PREFIX(add_ptr)(*ptr - 1, data);
                        	{
		#ifdef ENABLE_AXISTRACE
			int PTRCALL PREFIX ( tokenizeContent traceRet = (SPP_ERROR_NONE);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, ")", NULL, 5,
					TRACETYPE_DATA, sizeof(int PTRCALL PREFIX ( tokenizeContent), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return SPP_ERROR_NONE;
		#endif
	}

                    }
                    
                    *parserState = S_0;
                }
                else if (BT_S == byteType) 
                {
                    PREFIX(add_ptr)(*ptr - 1, data);
                    
                    *parserState = S_12;
                }
                else if (!PREFIX(is_name_char)(byteType))
                    *parserState = S_13;
                break;
        
            case S_12:
                /* if ('>' == ch) { */
                if (BT_GT == byteType) 
                {
                    if (data->numOfPtrs)
                        	{
		#ifdef ENABLE_AXISTRACE
			int PTRCALL PREFIX ( tokenizeContent traceRet = (SPP_ERROR_NONE);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, ")", NULL, 6,
					TRACETYPE_DATA, sizeof(int PTRCALL PREFIX ( tokenizeContent), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return SPP_ERROR_NONE;
		#endif
	}

                    
                    *parserState = S_0;
                }
                else if (BT_S != byteType)
                    *parserState = S_13;
                break;
        
            case S_13:
                /* some problem exists. fix it!. */
    
            default:
                
                	{
		#ifdef ENABLE_AXISTRACE
			int PTRCALL PREFIX ( tokenizeContent traceRet = (-(*parserState));
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, ")", NULL, 7,
					TRACETYPE_DATA, sizeof(int PTRCALL PREFIX ( tokenizeContent), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return -(*parserState);
		#endif
	}

            }
            *numOfChars -= MINBPC(enc);
            *ptr += MINBPC(enc);
        }
        	{
		#ifdef ENABLE_AXISTRACE
			int PTRCALL PREFIX ( tokenizeContent traceRet = (SPP_ERROR_TOKENIZER_FAILED);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, ")", NULL, 8,
					TRACETYPE_DATA, sizeof(int PTRCALL PREFIX ( tokenizeContent), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return SPP_ERROR_TOKENIZER_FAILED;
		#endif
	}

}

static int PTRCALL
PREFIX(nameMatchesAscii)(const ENCODING *enc, const char *ptr1,
                         const char *end1, const char *ptr2)
{
	#ifdef ENABLE_AXISTRACE
		if (axiscpp::AxisTrace::isTraceOn())
			axiscpp::AxisTrace::traceEntry(NULL, ")", NULL, 4,
					TRACETYPE_POINTER, sizeof(ENCODING), ((void*)&enc),
					TRACETYPE_STRING, 0, ((void*)&ptr1),
					TRACETYPE_STRING, 0, ((void*)&end1),
					TRACETYPE_STRING, 0, ((void*)&ptr2));	  /* AUTOINSERTED TRACE */
	#endif

  for (; *ptr2; ptr1 += MINBPC(enc), ptr2++) 
  {
    if (ptr1 == end1)
      	{
		#ifdef ENABLE_AXISTRACE
			int PTRCALL PREFIX ( nameMatchesAscii traceRet = (0);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, ")", NULL, 1,
					TRACETYPE_DATA, sizeof(int PTRCALL PREFIX ( nameMatchesAscii), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return 0;
		#endif
	}

    if (!CHAR_MATCHES(enc, ptr1, *ptr2))
      	{
		#ifdef ENABLE_AXISTRACE
			int PTRCALL PREFIX ( nameMatchesAscii traceRet = (0);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, ")", NULL, 2,
					TRACETYPE_DATA, sizeof(int PTRCALL PREFIX ( nameMatchesAscii), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return 0;
		#endif
	}

  }
  	{
		#ifdef ENABLE_AXISTRACE
			int PTRCALL PREFIX ( nameMatchesAscii traceRet = (ptr1 == end1);
			if (axiscpp::AxisTrace::isTraceOn())
				axiscpp::AxisTrace::traceExit(NULL, ")", NULL, 3,
					TRACETYPE_DATA, sizeof(int PTRCALL PREFIX ( nameMatchesAscii), ((void*)&traceRet));	  /* AUTOINSERTED TRACE */
			return traceRet;
		#else
			return ptr1 == end1;
		#endif
	}

}
            
